/**
 * @file common.h
 * @project invizible
 * @author Wesley Kenzie (wkenzie@gmail.com PGP 4c945266)
 * @created April 20, 2011
 * @modified June 18, 2011
 * @version 0.30
 * @copyright 2011. Arthur (Wesley) Kenzie. All Rights Reserved.
 * @license 3-clause BSD License. See accompanying LICENSE file.
 */

#ifndef _COMMON_H
#define	_COMMON_H

#ifdef	__cplusplus
extern "C" {
#endif

#define HAVE_STRLCPY 1
#define HAVE_STRLCAT 1

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>

#define TIME_FORMAT 3
#define _LOG_INFO 1
#define _LOG_WARNING 2
#define _LOG_ERROR 3
#define _LOG_LEVEL_DEBUG 2
#define MAX_LOG_ENTRY_LEN 1024
#define MAX_PRIVILEGED_PORT 1023
#define MAX_UNPRIVILEGED_PORT 65535
#define MAX_PATH_LEN 1024
#define DOD_WIPE_FILE_TIMES 7
#define DOD_WIPE_FILE_TIMES_STRING "7"
#define MIN_WIPE_FILE_TIMES 1
#define MAX_WIPE_FILE_TIMES 99
//
#define IO_READ_BUF_LEN 256
#define STANDARD_CMD_OUT_BUFSIZE 4096
#define CMD_BUFSIZE 256
#define EXTCMD_DEF_TIMEOUT 15 // default timeout for external command exec
char cmd_buf[CMD_BUFSIZE];
char err_buf[CMD_BUFSIZE];
char cmd_out[STANDARD_CMD_OUT_BUFSIZE];
//
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
//
#ifndef PATH_SEP
    #ifdef WIN32
    #define PATH_SEP      '\\'
    #else
    #define PATH_SEP      '/'
    #endif
#endif
//
/* The various return status states in which an external command result
 * may end up in.
*/
enum {
    EXTCMD_SETUID_ERROR             =   -4,
    EXTCMD_SELECT_ERROR             =   -3,
    EXTCMD_PIPE_ERROR               =   -2,
    EXTCMD_FORK_ERROR               =   -1,
    EXTCMD_SUCCESS_ALL_OUTPUT       = 0x00,
    EXTCMD_SUCCESS_PARTIAL_STDOUT   = 0x01,
    EXTCMD_SUCCESS_PARTIAL_STDERR   = 0x02,
    EXTCMD_STDOUT_READ_ERROR        = 0x04,
    EXTCMD_STDERR_READ_ERROR        = 0x08,
    EXTCMD_EXECUTION_ERROR          = 0x10,
    EXTCMD_EXECUTION_TIMEOUT        = 0x20
};
//
/* Some convenience macros for testing the extcmd return status.
*/
#define EXTCMD_IS_SUCCESS(x) (x == EXTCMD_SUCCESS_ALL_OUTPUT)
#define EXTCMD_IS_SUCCESS_PARTIAL_STDOUT(x) (x && EXTCMD_SUCCESS_PARTIAL_STDOUT)
#define EXTCMD_IS_SUCCESS_PARTIAL_STDERR(x) (x && EXTCMD_SUCCESS_PARTIAL_STDERR)
#define EXTCMD_IS_SUCCESS_PARTIAL_OUTPUT(x) \
    ((x && EXTCMD_SUCCESS_PARTIAL_STDOUT) \
        || x && EXTCMD_SUCCESS_PARTIAL_STDERR)
#define EXTCMD_STDOUT_READ_ERROR(x) (x && EXTCMD_STDOUT_READ_ERROR)
#define EXTCMD_STDERR_READ_ERROR(x) (x && EXTCMD_STDERR_READ_ERROR)
#define EXTCMD_READ_ERROR(x) \
    ((x && EXTCMD_STDOUT_READ_ERROR) \
        || x && EXTCMD_STDERR_READ_ERROR)
#define EXTCMD_EXECUTION_ERROR(x) (x && EXTCMD_EXECUTION_ERROR)

#define EXTCMD_NOERROR(x,y) ((y == 0) \
    && (EXTCMD_IS_SUCCESS(x) || EXTCMD_IS_SUCCESS_PARTIAL_OUTPUT(x))
//
volatile sig_atomic_t sig_interrupted;
//
// function prototype(s)
//
void set_time_snapshot(char *, int);
void get_time_snapshot(char *, int, struct timeval, struct tm *);
void logit(int, char *, int);
void logfileit(int, char *, FILE *, int);
unsigned int host_convert_ip4(char *);
unsigned short in_cksum(unsigned short *, int);
void set_char_to_hex(char *, char *, int, int);
void set_int_to_hex(int, char *, int);
char UTIL_TOLOWER(char c);
char UTIL_TOUPPER(char c);
int util_ascii_strcasecmp(const char *, const char *);
int util_ascii_strncasecmp(const char *, const char *, size_t);

#ifndef HAVE_STRLCPY
  int strlcpy(char *dst, const char *src, int siz);
#endif
#ifndef HAVE_STRLCAT
  int strlcat(char *dst, const char *src, int siz);
#endif

int run_extcmd_inviz(uid_t, char *, char *, size_t, int);
int wipefile(const char *, int);
int make_dir_path(const char *);
int check_dir_path(const char *, const char *, unsigned char);
int daemonize_process(const char *);
int write_pid_file(const char *);
pid_t get_running_pid(const char *);
unsigned int ev_linux_version (void);

#ifdef	__cplusplus
}
#endif

#endif	/* _COMMON_H */
