/**
 * @file hostnames.c
 * @project invizible
 * @author Wesley Kenzie (wkenzie@gmail.com PGP 4c945266)
 * @created June 17, 2011
 * @modified
 * @version 0.10
 * @copyright 2011. Arthur (Wesley) Kenzie. All Rights Reserved.
 * @license 3-clause BSD License. See accompanying LICENSE file.
 */

#include "common.h"
#include "parms.h"
#include "hostnames.h"

//
// sethostnames() function
//
void
set_hostnames() {
    int i, len;
    const char * coms[] = {"google", "apple", "yahoo", "amazon", "cnn", "facebook",
        "youtube", "microsoft", "adobe", "symantec", "opendns", "twitter", "msn",
        "skype", "baidu", "ebay", "live", "sony", "motorola", "hotmail", "starbucks",
        "walmart", "ip2location", "maxmind", "chevron", "godaddy", "verizon", "att",
        "myspace", "alexa", "mozilla", "bp", "sbc", "qwest", "showmyip", "expedia",
        "aa", "fedex", "etrade", "netcraft", "paypal", "googleusercontent", "bing", "aol",
        "flickr", "nytimes", "9down", "dhgate", "taobao", "livescore", "oasis",
        "globo", "gothicground", "isohunt", "xe", "thestar", "codeproject", "jfkait",
        "heathrowairport", "secunia", "mysql", "lego", "masrawy", "fifa", "mywot",
        "ampparit", "aastocks", "outblaze", "onlinesbi", "vippix", "flipkart", "detik",
        "spotify", "banorte", "bancomer", "elnorte", "ixquick", "torrentz", "sgx",
        "dbsvonline", "asiaone", "thaivisa", "dbs", "singaporebikes", "news24",
        "tuenti", "marca", "cdmon", "github", "salesforce", "oracle", "hp", "dell",
        "lenovo", "acer", "bestbuy", "blogger", "qq", "sedo", "sonystyle", "bankofamerica",
        "markmonitor", "linkedin", "imdb", "ask", "zedo", "cnet", "about", "livejournal",
        "alibaba", "ehow", "orkut", "netflix", "weather", "stackoverflow", "37signals",
        "trekbikes", "reddit", "digg", "groupon", "vimeo", "domaintools", "clickbank",
        "clubpenguin", "hsbc", "maktoob", "w3schools", "mashable", "yelp", "typepad",
        "wsj", "etsy", "wikia", "cj", "xing", "baomai", "megaupload", "hotfile",
        "fileserve", "youporn", "badoo", "smooch", "play", "jangomail", "chocolate"};
    for (i=0; (i < HOST_COM_COUNT) && coms[i]; i++) {
        len = strlen(coms[i]);
        hostcoms[i] = (char *)malloc(len+1);
        memset(hostcoms[i], 0, len+1);
        strncpy(hostcoms[i], coms[i], len);
    }
    //
    const char * orgs[] = {"wikipedia", "wordpress", "redcross", "torproject", "slashdot",
        "rutracker", "wikileaks", "craigslist", "thepiratebay", "pbs", "w3", "mozilla",
        "unfix", "backtrack-linux"};
    for (i=0; (i < HOST_ORG_COUNT) && orgs[i]; i++) {
        len = strlen(orgs[i]);
        hostorgs[i] = (char *)malloc(len+1);
        memset(hostorgs[i], 0, len+1);
        strncpy(hostorgs[i], orgs[i], len);
    }
    //
    const char * nets[] = {"jomax", "bodog", "gmx", "boingboing", "bac", "noscript",
        "flashgot", "rotana", "egynews", "sourceforge", "telus", "secureserver",
        "aljazeera", "comcast", "z5x", "vnexpress"};
    for (i=0; (i < HOST_NET_COUNT) && nets[i]; i++) {
        len = strlen(nets[i]);
        hostnets[i] = (char *)malloc(len+1);
        memset(hostnets[i], 0, len+1);
        strncpy(hostnets[i], nets[i], len);
    }
    //
    const char * others[] = {"bbc.co.uk", "google.de", "google.es", "spiegel.de",
        "t-online.de", "bild.de", "google.co.jp", "google.co.in", "google.co.uk",
        "google.nl", "web.de", "de.wikipedia.org", "it.wikipedia.org", "en.wikipedia.org",
        "es.wikipedia.org", "sv.wikipedia.org", "orange.fr", "sfr.fr", "free.fr",
        "bom.gov.au", "nab.com.au", "uol.com.br", "ig.com.br", "bb.com.br", "shaw.ca",
        "yvr.ca", "espn.go.com", "login.live.com", "login.yahoo.com", "seznam.cz",
        "novinky.cz", "dr.dk", "dev.mysql.com", "ihf.info", "neti.ee", "hot.ee",
        "shop.skype.com", "hs.fi", "digitoday.fi", "worldfriends.tv", "reliancenetconnect.co.in",
        "login.rediff.com", "amazon.de", "amazon.fr", "fr.ask.com", "haz.de", "nrg.co.il",
        "mfa.gov.il", "libero.it", "ansa.it", "fineco.it", "mixi.jp", "blog.livedoor.jp",
        "slashdot.jp", "europa.eu", "indepefndent.co.uk", "onet.pl", "nk.pl", "interia.pl",
        "allegro.pl", "tvn24.pl", "ceneo.pl", "gotyx.pl", "gazeta.pl", "profil.wp.pl",
        "ipko.pl", "forum.gazeta.pl", "vkontakte.ru", "yandex.ru", "win.mail.ru",
        "auth.mail.ru", "rambler.ru", "support.creative.com", "timeslive.co.za",
        "sanlam.co.za", "iol.co.za", "mybroadband.co.za", "ib.absa.co.za", "elmundo.es",
        "subcultura.es", "telefonica.es", "idg.se", "google.com.br", "google.com.au",
        "cbc.ca", "bit.ly", "google.com.ar", "terra.com.br", "icio.us", "me.zing.vn",
        "kicker.de", "search.conduit.com", "lemonde.fr", "last.fm", "ciao.de", "ad.z5x.net",
        "rai.tv", "20minutes.fr", "demonoid.me", "ex.ua", "hurriyet.com.tr", "manager.co.th",
        "bluewin.ch", "nzz.ch", "sport.onet.pl", "moto.allegro.pl", "co.cc", "mail.naver.com",
        "sympatico.ca", "download.oracle.com", "thairath.co.th", "ustream.tv", "orf.at",
        "krone.at", "gametwist.fr", "swr.de", "20min.ch", "set.or.th", "mail.google.com"};
    for (i=0; (i < HOST_OTHER_COUNT) && others[i]; i++) {
        len = strlen(others[i]);
        hostothers[i] = (char *)malloc(len+1);
        memset(hostothers[i], 0, len+1);
        strncpy(hostothers[i], others[i], len);
    }
}
