/**
 * @file mypcap.h
 * @project invizible
 * @author Wesley Kenzie (wkenzie@gmail.com PGP 4c945266)
 * @created April 30, 2011
 * @modified June 11, 2011
 * @version 0.19
 * @copyright 2011. Arthur (Wesley) Kenzie. All Rights Reserved.
 * @license 3-clause BSD License. See accompanying LICENSE file.
 */

#ifndef _MYPCAP_H
#define	_MYPCAP_H

#ifdef	__cplusplus
extern "C" {
#endif

#include "hpingwrap.h"
#include "parms.h"
#include <arpa/inet.h>
#include <netinet/if_ether.h>
#include <net/ethernet.h>
#include <netinet/ether.h>
//#include <netinet/ip.h>
#include "pcap/pcap-int.h"

#define MAX_PCAP_PACKET_COUNT 1024 // must be > 0
#define MIN_PCAP_PACKET_COUNT 1
#define DEFAULT_PCAP_PACKET_COUNT 64
//
// tcpdump header (ether.h) defines ETHER_HDRLEN) 
//
#ifndef ETHER_HDRLEN 
#define ETHER_HDRLEN 14
#endif

/*
 * Structure of an internet header, stripped of all options.
 *
 * This is taken directly from the tcpdump source
 *
 * We declare ip_len and ip_off to be short, rather than u_short
 * pragmatically since otherwise unsigned comparisons can result
 * against negative integers quite easily, and fail in subtle ways.
 */
struct my_ip {
	u_int8_t	ip_vhl;		/* header length, version */
#define IP_V(ip)	(((ip)->ip_vhl & 0xf0) >> 4)
#define IP_HL(ip)	((ip)->ip_vhl & 0x0f)
	u_int8_t	ip_tos;		/* type of service */
	u_int16_t	ip_len;		/* total length */
	u_int16_t	ip_id;		/* identification */
	u_int16_t	ip_off;		/* fragment offset field */
#define	IP_DF 0x4000			/* dont fragment flag */
#define	IP_MF 0x2000			/* more fragments flag */
#define	IP_OFFMASK 0x1fff		/* mask for fragmenting bits */
	u_int8_t	ip_ttl;		/* time to live */
	u_int8_t	ip_p;		/* protocol */
	u_int16_t	ip_sum;		/* checksum */
	struct	in_addr ip_src,ip_dst;	/* source and dest address */
};
int pcap_recv_buffer[MAX_PCAP_PACKET_COUNT];
uint pcap_recv_index;
//*****************************************************************************
//
// based on portions of libevent2/evdns.c
//
#undef MIN
#define MIN(a,b) ((a)<(b)?(a):(b))
#define u64 uint64_t
#define u32 uint32_t
#define u16 uint16_t
#define u8  uint8_t
//
//*****************************************************************************
//
// following is for reply_parse() function
//
#define DNS_SKIP_MATCH_TO_ORIGINAL
//
// function prototypes
//
char * pcap_lookupdevspec(char *);
u_int16_t handle_ethernet (u_char *, const struct pcap_pkthdr *, const u_char *);
u_char * handle_IP (u_char *, const struct pcap_pkthdr *, const u_char *, const int);
void pcap_server_packet_callback(u_char *, const struct pcap_pkthdr *, const u_char *);
void pcap_client_packet_callback(u_char *, const struct pcap_pkthdr *, const u_char *);
int name_parse(u8 *packet, int length, int *idx, char *name_out, int name_out_len);
int reply_parse(u8 *packet, int length, int serious_error);
int request_parse(u8 *packet, int length, struct sockaddr *addr, socklen_t addrlen);

#ifdef	__cplusplus
}
#endif

#endif	/* _MYPCAP_H */
