/**
 * @file pkclient.h
 * @project invizible
 * @author Wesley Kenzie (wkenzie@gmail.com PGP 4c945266)
 * @created May 15, 2011
 * @based on fwknop (Damien S. Stuart <dstuart@dstuart.org> and Michael Rash <mbr@cipherdyne.org>)
 * @modified June 18, 2011
 * @version 0.22
 * @copyright 2011. Arthur (Wesley) Kenzie. All Rights Reserved.
 * @license 3-clause BSD License. See accompanying LICENSE file.
 */

#ifndef _PKCLIENT_H
#define	_PKCLIENT_H

#ifdef	__cplusplus
extern "C" {
#endif

#include "fwknop/lib/fko.h"
#include "fwknop/netinet_common.h"
#include "pkcommon.h"
#include "parms.h"
#include <signal.h>

#if HAVE_CTYPE_H
  #include <ctype.h> /* Using this for isdigit() */
#else
  /* Fall-back does not account for locale */
  #define isdigit(c) (c >= 48 && c <= 57)
#endif

#define CRYPT_OP_ENCRYPT 1
#define CRYPT_OP_DECRYPT 2

/* My Name and Version
*/
#define MY_SPA_CLIENT_NAME     "fwknop"
#define MY_SPA_CLIENT_DESC     "Single Packet Authorization client"

/* Get our program version from VERSION (defined in config.h).
*/
#define MY_SPA_CLIENT_VERSION VERSION

/* Default config path, can override with -c
*/
#define DEF_SPA_CLIENT_CONFIG_FILE MY_SPA_CLIENT_NAME".conf"

/* For time offset handling
*/
#define MAX_TIME_STR_LEN        9
#define TIME_OFFSET_SECONDS     1
#define TIME_OFFSET_MINUTES     60
#define TIME_OFFSET_HOURS       3600
#define TIME_OFFSET_DAYS        86400

/* For resolving the allow IP via HTTP and sending SPA packets over
 * HTTP -  http://www.whatismyip.com/automation/n09230945.asp
    #define HTTP_RESOLVE_HOST          "www.whatismyip.com"
    #define HTTP_RESOLVE_URL           "/automation/n09230945.asp"
  * --DSS Note: The whatismyip.com site has some usage restrictions.
  *             so we will make the default run on cipherdyne website
  *             for now.
*/
//#define HTTP_RESOLVE_HOST          "www.cipherdyne.org"
#define HTTP_RESOLVE_HOST           "www.ippages.com"
//#define HTTP_RESOLVE_URL           "/cgi-bin/myip"
#define HTTP_RESOLVE_URL            "/simple/?se=1"
#define HTTP_MAX_REQUEST_LEN       2000
#define HTTP_MAX_RESPONSE_LEN      2000
#define HTTP_MAX_USER_AGENT_LEN    50
#define MAX_HOSTNAME_LEN           NI_MAXHOST

#ifndef HTTP_RESOLVE_HOSTS
#define HTTP_RESOLVE_HOSTS
#define HTTP_RESOLVE_HOST4 "www.showmyip.com"
#define HTTP_RESOLVE_URL4 "/simple/?se=1"
#define HTTP_RESOLVE_HOST2 "www.whatismyip.com"
#define HTTP_RESOLVE_URL2 "/automation/n09230945.asp"
#define HTTP_RESOLVE_HOST3 "www.cipherdyne.org"
#define HTTP_RESOLVE_URL3 "/cgi-bin/myip"
#endif

//
// function prototypes
//
unsigned short chksum_inviz(unsigned short *, int);
char* getpasswd_inviz(const char *);
char* getpasswd_file_inviz(const char *, const char *);
char * get_user_pw_inviz(int);
int send_spa_packet_tcp_or_udp_inviz(char *, int, char *, struct sockaddr *, int);
int send_spa_packet_raw_inviz(char *, int, struct sockaddr_in *, struct sockaddr_in *, int);
int send_spa_packet_icmp_inviz(char *, int, struct sockaddr_in *, struct sockaddr_in *, int);
int send_spa_packet_http_inviz(char *, int, char *, struct sockaddr *, int);
int send_spa_packet_inviz(fko_ctx_t, char *, struct sockaddr *, int);
int write_spa_packet_data_inviz(fko_ctx_t);
int resolve_ip_http_inviz(char *, char *);

#ifdef	__cplusplus
}
#endif

#endif
