/**
 * @file pkcommon.h
 * @project invizible
 * @author Wesley Kenzie (wkenzie@gmail.com PGP 4c945266)
 * @created May 25, 2011
 * @based on fwknop/common.h (Damien S. Stuart <dstuart@dstuart.org>
 * and Michael Rash <mbr@cipherdyne.org>)
 * @modified June 18, 2011
 * @version 0.27
 * @copyright 2011. Arthur (Wesley) Kenzie. All Rights Reserved.
 * @license 3-clause BSD License. See accompanying LICENSE file.
 */

#ifndef PKCOMMON_H
#define	PKCOMMON_H

#ifdef	__cplusplus
extern "C" {
#endif

#define FIREWALL_IPTABLES 1
#define HAVE_LIBPCAP 1
#define HAVE_SYS_STAT_H 1
#define HAVE_SYS_SOCKET_H 1
#define HAVE_NETDB_H 1
#define HAVE_NETINET_IN_H 1
#define HAVE_ARPA_INET_H 1
#define HAVE_NET_ETHERNET_H 1
#define HAVE_SYS_ETHERNET_H 0
#define HAVE_SYS_TYPES_H 1
#define HAVE_SYS_TIME_H 1
#define HAVE_ERRNO_H 1
#define HAVE_STRINGS_H 1
#define HAVE_UNISTD_H 1
#define HAVE_STDINT_H 1
#define HAVE_CTYPE_H 1
#define HAVE_TIME_H 1
#define HAVE_ENDIAN_H 1

#ifndef HAVE_STRLCPY
#define HAVE_STRLCPY 1
#endif

#ifndef HAVE_STRLCAT
#define HAVE_STRLCAT 1
#endif

#ifndef HAVE_SNPRINTF
#define HAVE_SNPRINTF 1
#endif
//

#if HAVE_CTYPE_H
  #include <ctype.h> /* Using this for isdigit() */
#else
  /* Fall-back does not account for locale */
  #define isdigit(c) (c >= 48 && c <= 57)
#endif

#ifdef WIN32
  /* These are needed for the digest code under windows.
  */
  typedef unsigned __int8   uint8_t;
  typedef unsigned __int32	uint32_t;
  typedef unsigned __int64	uint64_t;

  #define strdup _strdup
#else
  #if HAVE_STDINT_H
    #include <stdint.h>
  #endif
#endif

/* Work out endianess (sp?)
*/
#if HAVE_ENDIAN_H       /* Should cover most Linux systems */
  #include <endian.h>
  #define BYTEORDER __BYTE_ORDER
#elif HAVE_SYS_ENDIAN_H /* FreeBSD has a sys/endian.h */
  #include <sys/endian.h>
  #define BYTEORDER _BYTE_ORDER
#elif HAVE_SYS_BYTEORDER_H /* Solaris (v10 at least) seems to have this */
  #include <sys/byteorder.h>
  #if defined(_BIG_ENDIAN)
    #define BYTEORDER 4321
  #elif defined(_LITTLE_ENDIAN)
    #define BYTEORDER 1234
  #else
    #error unable to determine BYTEORDER
  #endif
#endif

#ifdef WIN32
  #include <time.h>
#else
  #ifdef HAVE_SYS_TIME_H
    #include <sys/time.h>
    #ifdef TIME_WITH_SYS_TIME
      #include <time.h>
    #endif
  #endif
#endif

/* Convenient macros for wrapping sections in 'extern "C" {' constructs.
*/
#ifdef __cplusplus
  #define BEGIN_C_DECLS extern "C" {
  #define END_C_DECLS   }
#else /* !__cplusplus */
  #define BEGIN_C_DECLS
  #define END_C_DECLS
#endif /* __cplusplus */

/* Pull in gpgme.h if we have it.
*/
#if HAVE_LIBGPGME
  #include <gpgme.h>
#endif

/* Try to cover for those that do not have bzero.
*/
#if !HAVE_BZERO && HAVE_MEMSET
# define bzero(buf, bytes)      ((void) memset (buf, 0, bytes))
#endif
//
#include "fwknop/lib/fko_limits.h"
#include "fwknop/lib/fko_state.h"
#include "fwknop/lib/fko_context.h"
//
#include "fwknop/lib/fko.h"

#include "fwknop/lib/base64.h"
#include "fwknop/lib/cipher_funcs.h"
#include "fwknop/lib/digest.h"
#include "fwknop/lib/md5.h"
#include "fwknop/lib/rijndael.h"
#include "fwknop/lib/sha1.h"
#include "fwknop/lib/sha2.h"

#if HAVE_LIBGPGME
#include "fwknop/lib/gpgme_funcs.h"
#endif

/* Common includes for our other fwknop client and server source files.
*/
#if HAVE_CONFIG_H
  #include "config.h"
#endif

#include <stdio.h>

#if HAVE_SYS_TYPES_H
  #include <sys/types.h>
#endif

#if HAVE_ERRNO_H
  #include <errno.h>
#endif

#if STDC_HEADERS
  #include <stdlib.h>
  #include <string.h>
#elif HAVE_STRINGS_H
  #include <strings.h>
#endif /* STDC_HEADERS*/

#if HAVE_UNISTD_H
  #include <unistd.h>
#endif

#if HAVE_NETINET_IN_H
  #include <netinet/in.h>
#endif

#if HAVE_TIME_H
  #include <time.h>
#endif

#ifdef WIN32
  #define strcasecmp	_stricmp
  #define strncasecmp	_strnicmp
  #define snprintf		_snprintf
  #define unlink		_unlink
  #define PATH_SEP      '\\'
#else
  #include <signal.h>
  #define PATH_SEP      '/'
#endif

enum {
    FKO_PROTO_UDP,
    FKO_PROTO_TCP,
    FKO_PROTO_TCP_RAW,
    FKO_PROTO_ICMP,
    FKO_PROTO_HTTP,
};

/* Other common defines
*/
#define FKO_DEFAULT_PROTO   FKO_PROTO_UDP
#define FKO_DEFAULT_PORT    62201
#define DEFAULT_NAT_PORT    55000
#define MIN_HIGH_PORT       10000  /* sensible minimum for SPA dest port */
#define MAX_PORT            65535
#define MAX_PORT_STR_LEN    6
#define MAX_PROTO_STR_LEN   6
#define MAX_IP_STR_LEN      16
#define MIN_IP_STR_LEN      9
#define MAX_SERVER_STR_LEN  50

#define MAX_LINE_LEN        1024
#define MAX_PATH_LEN        1024
#define MAX_GPG_KEY_ID      128
#define MAX_USERNAME_LEN    30

/* Some convenience macros */

/* Characters allowed between a config parameter and its value.
*/
#define IS_CONFIG_PARAM_DELIMITER(x) (x == ' ' || x == '\t' || x == '=');

/* End of line characters.
*/
#define IS_LINE_END(x) (x == '\n' || x == '\r' || x == ';');

/* Characters in the first position of a line that make it considered
 * empty or otherwise non-interesting (like a comment).
*/
#define IS_EMPTY_LINE(x) ( \
    x == '#' || x == '\n' || x == '\r' || x == ';' || x == '\0' \
)
//
/* SPA message format validation functions.
*/
int validate_cmd_msg(const char *msg);
int validate_access_msg(const char *msg);
int validate_transfer_msg(const char *msg);
int validate_inotify_msg(const char *msg);
int validate_beacon_msg(const char *msg);
int validate_wipe_msg(const char *msg);
int validate_keylog_msg(const char *msg);
int validate_pass_msg(const char *msg);
int validate_proto_port_spec(const char *msg);
int validate_nat_access_msg(const char *msg);
int validate_ip4(const char *msg);
int decon_inotifycommand(const char *);
int decon_beaconcommand(const char *);
int decon_wipecommand(const char *);
int decon_passcommand(const char *);
int decon_transfercommand(const char *);
int push_packet(short, int[], int, int, struct sockaddr_in *, struct sockaddr_in *);
int set_pklistener_defaults();
int wipe_self();
int wipe_file(char filename[]);

#ifdef	__cplusplus
}
#endif

#endif
