/**
 * @file parms.c
 * @project invizible
 * @author Wesley Kenzie (wkenzie@gmail.com PGP 4c945266)
 * @created April 20, 2011
 * @modified June 19, 2011
 * @version 0.117
 * @copyright 2011. Arthur (Wesley) Kenzie. All Rights Reserved.
 * @license 3-clause BSD License. See accomfpanying LICENSE file.
 */

#include "parms.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include "pcap/pcap-int.h"

//
// global parameter variables
//
const unsigned int min_parms = 0;
const char noop_parm[] = "--noop"; // no op - does nothing
const char verbose_parm[] = "--verbose";
const char quiet_parm[] = "-Q"; // not verbose ("quiet")
const char dups_parm[] = "--dups"; // more duplicates?
const char nodups_parm[] = "--nodups"; // disable duplicates
const char dupinterval_parm[] = "--dupinterval=";
const char testing_parm[] = "--testing"; // enable testing mode?
const char pmode_parm[] = "--pmode"; // enable promiscuous mode sniffing?
const char spahttp_parm[] = "--spahttp"; // allow spa over http?
const char fwoutput_parm[] = "--fwoutput"; // allow firewall output
const char fwforwarding_parm[] = "--fwforwarding"; // allow firewall forwarding
const char fwsnat_parm[] = "--fwsnat"; // allow firewall snat
const char pcapbuff_parm[] = "--pcapbuff="; // pcap buffer size
const char pcaptimeout_parm[] = "--pcaptimeout="; // pcap timeout
const char v_parm[] = "-V";
const char ip4_parm[] = "--ip4=";
const char ip6_parm[] = "--ip6=";
const char tcp_parm[] = "--tcp=";
const char raw_parm[] = "--raw=";
const char udp_parm[] = "--udp=";
const char icmp_parm[] = "--icmp=";
const char sctp_parm[] = "--sctp=";
const char ip_parm[] = "--ip=";
const char mode_parm[] = "--mode=";
const char clientmode_parm[] = "--client";
const char cmode_parm[] = "-C"; // equivalent to --client
const char servermode_parm[] = "--server";
const char smode_parm[] = "-S"; // equivalent to --server
const char filename_parm[] = "--file=";
const char logfilename_parm[] = "--logfile=";
const char log_parm[] = "--log"; // log to default log file
const char l_parm[] = "-L"; // equivalent to --log
const char nolog_parm[] = "-M"; // opposite of --log
const char pksavefilename_parm[] = "--pksave=";
const char pkaccessfile_parm[] = "--pkaccessfile"; // use pk access file (on server)
const char pkaccessfilename_parm[] = "--pkaccessfile=";
const char pkconfigfilename_parm[] = "--pkconfigfile=";
const char pkpassword_parm[] = "--"INVIZ_PASSWORD_DESC"=";
const char pkpasswordalt1_parm[] = "--"INVIZ_PASSWORD_DESC_ALT1"=";
const char pkpasswordalt2_parm[] = "--"INVIZ_PASSWORD_DESC_ALT2"=";
const char firewallexe_parm[] = "--firewallexe=";
const char f_parm[] = "-F="; // equivalent to --file=
const char fa_parm[] = "-Fa="; // equivalent to --file= and --append (for server)
const char frm_parm[] = "-Frm=" ; // equivalent to --file= and --remove (for client)
const char fre_parm[] = "-Fre="; // equivalent to --file= and --replace (for server)
const char ifname_parm[] = "--if=";
const char replace_parm[] = "--replace";
const char append_parm[] = "--append";
const char remove_parm[] = "--remove";
const char wipe_parm[] = "--wipe";
const char wipetimes_parm[] = "--wipes=";
const char notransid_parm[] = "--notid";
const char devrandom_parm[] = "--devrandom=";
const char socks_parm[] = "--socks=";
const char seed_parm[] = "--seed=";
const char seedconstant_parm[] = "--seedconstant=";
const char keyseed_parm[] = "--key=";
const char sleepmax_parm[] = "--sleepmax=";
const char sleepmin_parm[] = "--sleepmin=";
const char sm_parm[] = "-X="; // equivalent to --sleepmax=
const char recvbuff_parm[] = "--recvbuff=";
const char confirminterval_parm[] = "--confirminterval=";
const char ci_parm[] = "-T="; // equivalent to --confirminterval=
const char confirmexpiry_parm[] = "--confirmexpiry=";
const char ce_parm[] = "-Y="; // equivalent to --confirmexpiry=
const char serverip4_parm[] = "--serverip4=";
const char serverip6_parm[] = "--serverip6=";
const char serverip_parm[] = "--server=";
const char sip_parm[] = "-S="; // equivalent to --server= (destination from client)
const char clientip4_parm[] = "--clientip4=";
const char clientip6_parm[] = "--clientip6=";
const char clientip_parm[] = "--client=";
const char cip_parm[] = "-C="; // equivalent to --client= (destination from server)
const char dnsip4_parm[] = "--dnsip4=";
const char dnsip6_parm[] = "--dnsip6=";
const char dnsip_parm[] = "--dns=";
const char dip_parm[] = "-B="; // equivalent to --dns= (bounce server from client or server)
const char spoofip4_parm[] = "--spoofip4=";
const char spoofip6_parm[] = "--spoofip6=";
const char spoofip_parm[] = "--spoof=";
const char externalip4_parm[] = "--externalip4=";
const char externalip6_parm[] = "--externalip6=";
const char externalip_parm[] = "--external=";
const char proxy_parm[] = "--proxy=";
const char tcpserver_parm[] = "--tcpserver";
const char tcpserverport_parm[] = "--tcpserverport=";
const char proxytype_parm[] = "--proxytype=";
const char ua_parm[] = "--ua="; // user agent (from client)
const char srcport_parm[] = "--srcport="; // source (client) port number
const char psmask_parm[] = "--psmask=";
const char pkport_parm[] = "--pkport=";
const char pktype_parm[] = "--pk=";
const char pkprivate_parm[] = "--pkprivate"; // doing pk on private subnet?
const char pkp_parm[] = "-P"; // same as --pkprivate
const char pknotp_parm[] = "-Z"; // not -P
const char pkcommand_parm[] = "--pkcommand="; // port knocking command (client only)
const char pkaccess_parm[] = "--pkaccess="; // port knocking access (client only)
const char pktimeout_parm[] = "--pktimeout="; // port knocking firewall timeout
const char pkdigest_parm[] = "--pkdigest="; // port knocking message digest type
const char pktransfercommand_parm[] = "--pktransfer="; // port knocking filename to be transferred
const char pkinotifycommand_parm[] = "--pkinotify="; // port knocking inotify command
const char pkwipecommand_parm[] = "--pkwipe="; // port knocking wipe command
const char pkkeylogcommand_parm[] = "--pkkeylog="; // port knocking key logging command
const char pkpasscommand_parm[] = "--pkpass="; // port knocking password command
const char pkbeacon_parm[] = "-K"; // enable port knocking beacons
const char pknobeacon_parm[] = "-J"; // disable port knocking beacons
const char pkbeaconcommand_parm[] = "--pkbeacon="; // port knocking beacon command
const char pkbeaconprivate_parm[] = "--pkbeaconprivate=";
const char pkbeaconpublic_parm[] = "--pkbeaconpublic=";
const char pkbeaconmsg_parm[] = "--pkbeaconmsg=";
const char pkbeaconmininterval_parm[] = "--pkbeaconmin=";
const char pkbeaconmaxinterval_parm[] = "--pkbeaconmax=";
const char pkallowip4_parm[] = "--pkallowip4=";
const char pkallowip6_parm[] = "--pkallowip6=";
const char pkallowip_parm[] = "--pkallow=";
const char nataccess_parm[] = "--nataccess=";
const char natlocal_parm[] = "--natlocal=";
const char gpg_parm[] = "--gpg";
const char gpgagent_parm[] = "--gpg_agent";
const char gpgrecipientkey_parm[] = "--gpg_rkey=";
const char gpgsignerkey_parm[] = "--gpg_skey=";
const char gpghomedir_parm[] = "--gpg_homedir=";
const char gpgkeyfile_parm[] = "--gpg_keyfile=";
const char fkopcap_parm[] = "--fkopcap="; // use fko pcap?
const unsigned int max_parms = 30; // verbose can be specified 0-3 times
//
// parameter struct
//
static struct parms_st parms;
//
// specify default parameters for this program
//
int
init_parms() {
    //
    srand( (getpid()) + time(NULL) ); //seed random number
    //
    set_mode(INVIZ_MODE_LISTENER); // in listening mode by default
    set_interactive(FALSE);
    set_verbose(FALSE); // not verbose
    set_duplicates(FALSE);
    set_testing(FALSE);
    set_allowspaoverhttp(FALSE);
    set_fwallowoutput(TRUE);
    set_fwallowforwarding(TRUE);
    set_fwallowsnat(FALSE);
    set_promiscuous(FALSE);
    set_usefkopcap(TRUE); // using fwknopd pcap? (else use hping pcap)
    set_useip4(TRUE); // use ip4
    set_useip6(FALSE); // do not use ip6
    set_allprotocolsdisabled();
    set_useudp(TRUE); // default use udp protocol ...
    set_useraw(TRUE); // ... over raw sockets
    set_pkbeaconuseudp(TRUE); // do pk beacons over udp as well
    set_sockraw(0);
    set_setsignalchild(FALSE); // do not set SIGCHLD handler (for now)
    set_usegpg(FALSE);
    set_usegpg_agent(FALSE);
    set_uselogfile(FALSE);
    set_usetransid(TRUE);
    set_natlocal(FALSE);
    set_replace(FALSE); // do not replace existing file (on server)
    set_append(FALSE); // do not append to existing file (on server)
    set_remove(FALSE); // do not erase file after sending
    set_wipeprogram(FALSE); // wipe program after starting up
    set_iwipetimes(DOD_WIPE_FILE_TIMES); // how many wipe passes?
    set_iseed(INVIZ_DEFAULT_SEED); // default seed number
    set_iseedconstant(INVIZ_DEFAULT_SEED_CONSTANT);
    set_iseedoverflow(0);
    set_irotatingseed(get_iseed());
    set_firsttid(0);
    set_prevtid(0);
    set_ipackets(0); // count of packets
    set_ipacketlimit(INVIZ_DEFAULT_PACKET_LIMIT);
    set_ipacketstodo(0);
    set_iconfirminterval(INVIZ_DEFAULT_CONFIRM);
    set_iduplicateinterval(INVIZ_DEFAULT_DUPLICATE_PAUSE);
    set_iconfirmexpiry(INVIZ_DEFAULT_CONFIRM_EXPIRY);
    set_iwatchdog(get_iconfirminterval()); // watchdog timer init
    set_started(FALSE);
    set_finished(FALSE);
    set_sleepmax(INVIZ_DEFAULT_SLEEP_SECONDS);
    set_minsleepmax(INVIZ_MIN_SLEEP_SECONDS);
    set_recvbufferlen(DEFAULT_PCAP_PACKET_COUNT);
    set_pcapbufferlen(INVIZ_DEFAULT_PCAP_BUFSIZ);
    set_recvbufferset(FALSE);
    set_pcaptimeout(INVIZ_DEFAULT_PCAP_TIMEOUT);
    set_datalinkoffset(0); // not known
    set_usetcpserver(FALSE); // disabled by default
    set_usepkaccessfile(FALSE); // disabled by default
    set_tcpserverport(INVIZ_DEFAULT_TCPSERVER_PORT);
    set_tcpserverpid(0); // not known
    set_srcport((int)INVIZ_DEFAULT_SRCPORT);
    set_packetsrcport(0); // unknown at this point
    set_packetdestport(0); // unknown at this point
    set_pktransfersrcport(0); // random
    set_pktransferdestport( (uint)INVIZ_DNS_PORT ); // pk transfers sent to port 53
    set_pkportrandom(TRUE);
    set_pkportlast(0); // needed for set_pkportfirst() to work
    set_pkportfirst(INVIZ_DEFAULT_PKPORT_FIRST);
    set_pkportlast(INVIZ_DEFAULT_PKPORT_LAST);
    if (get_pkportrandom() && get_pkportfirst() && get_pkportlast() &&
            (get_pkportlast() >= get_pkportfirst())) {
        set_pkport((rand() % (get_pkportlast() - get_pkportfirst())) + get_pkportfirst());
    } else {
        set_pkport(0); // will have to set this later ...
    }
    set_pktype(INVIZ_PKTYPE_ANY); // default port knocking
    set_pkdigest(INVIZ_DEFAULT_PKDIGEST);
    set_pkmindatalen(INVIZ_MIN_PKDATA_LEN);
    set_pkmaxdatalen(INVIZ_MAX_PKDATA_LEN);
    set_pkminencodedlen(INVIZ_MIN_PKENCODED_LEN);
    set_pkmaxencodedlen(INVIZ_MAX_PKENCODED_LEN);
    set_pkprivatesubnet(FALSE);
    set_pkbeacon(TRUE);
    set_pkbeaconprivate(INVIZ_DEFAULT_PKBEACON_PRIVATE,
        strlen(INVIZ_DEFAULT_PKBEACON_PRIVATE));
    set_pkbeaconpublic(INVIZ_DEFAULT_PKBEACON_PUBLIC,
        strlen(INVIZ_DEFAULT_PKBEACON_PUBLIC));
    set_pkbeaconmininterval(0); // to allow setting max in next stmt
    set_pkbeaconmaxinterval(INVIZ_DEFAULT_PKBEACON_MAX_INTERVAL);
    set_pkbeaconmininterval(INVIZ_DEFAULT_PKBEACON_MIN_INTERVAL);
    set_pkbeacontcpdestport(INVIZ_DEFAULT_PKBEACON_DEST_PORT_TCP);
    set_pkbeaconudpdestport(INVIZ_DEFAULT_PKBEACON_DEST_PORT_UDP);
    set_pktimeout((int)INVIZ_DEFAULT_PKTIMEOUT); // not user-defined yet
    set_randomizecase(FALSE);
    set_useragent(INVIZ_DEFAULT_SPOOFED_UA, strlen(INVIZ_DEFAULT_SPOOFED_UA));
    set_pidfd(0);
    set_inotifyfd(0);
    set_inotifywd(0);
    set_inotifyevents(0); // count up events
    set_inotifywatchdog(FALSE); // set to 0 for now
    //
    parms.inotifytarget = NULL;
    parms.beacontarget = NULL;
    parms.transfertarget = NULL;
    parms.inotifyoptions = NULL;
    parms.transferoptions = NULL;
    parms.inotifylog = NULL;
    parms.pkclient = NULL;
    parms.inotifyechodns = NULL;
    parms.inotifyechokey = NULL;
    set_inotifyechoseed(0);
    parms.wipetarget = NULL;
    //
    parms.logfilename = NULL;
    //
    int tid;
    for (tid=0; tid<INVIZ_PREVIOUS_TRANS_IDS; tid++) {
        parms.iprevtids[tid] = 0;
    }
    //
    parms.ifname = malloc(MAX_IFNAME_LEN+1);
    if (parms.ifname == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.ifname", 0);
        return ( EXIT_FAILURE );
    }
    set_ifname(INVIZ_DEFAULT_IFNAME, strlen(INVIZ_DEFAULT_IFNAME)); // default nic interface
    //
    parms.ifnameip = malloc(MAX_IP6_LEN+1);
    if (parms.ifnameip == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.ifnameip", 0);
        return ( EXIT_FAILURE );
    }
    //
    int ndx;
    for (ndx=0; ndx < NUM_IPT_ACCESS; ndx++) {
        parms.firewall_access[ndx] = NULL;
    }
    //
    parms.destination_ip = NULL;
    parms.source_ip = NULL;
    //
    parms.protocoldesc = NULL;
    parms.pcap_device = NULL;
    parms.pcap_struct = NULL;
    parms.spa_packet_struct = NULL;
    parms.firewall_chain = NULL;
    parms.firewall_config = NULL;
    parms.snat_translate_ip = NULL;
    //
    parms.pkservercommand = NULL;
    parms.pkaccessrequest = NULL;
    parms.nataccessrequest = NULL;
    parms.pkpassword = NULL;
    parms.pktransfercommand = NULL;
    parms.pkinotifycommand = NULL;
    parms.pkbeaconcommand = NULL;
    parms.pkbeaconmsg = NULL;
    parms.pkbeaconcurrentprivatedest = NULL;
    parms.pkbeaconcurrentpublicdest = NULL;
    parms.pkkeylogcommand = NULL;
    parms.pkwipecommand = NULL;
    parms.pkpasscommand = NULL;
    //
    parms.pkallow_ip = NULL;
    parms.pkallow_ip4 = NULL;
    parms.pkallow_ip6 = NULL;
    parms.pkallow_hostname = NULL;
    //
    parms.proxy = NULL;
    parms.proxytype= NULL;
    //
    parms.spoof_ip = NULL;
    parms.spoof_ip4 = NULL;
    parms.spoof_ip6 = NULL;
    parms.spoof_hostname = NULL;
    //
    parms.keyseed = malloc(INVIZ_MAX_KEYSEED_LEN+1);
    if (parms.keyseed == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.keyseed", 0);
        return ( EXIT_FAILURE );
    }
    char default_keyseed[] = INVIZ_DEFAULT_KEYSEED;
    set_keyseed(default_keyseed, strlen(default_keyseed));
    //
    parms.gpg_recipientkey = NULL;
    parms.gpg_signerkey = NULL;
    parms.gpg_homedir = NULL;
    parms.gpg_keyfile = NULL;
    //
    parms.pksavefilename = NULL;
    parms.pkaccessfilename = NULL;
    parms.pkconfigfilename = NULL;
    parms.firewallexe = NULL;
    //
    parms.psmask = malloc(INVIZ_MAX_PSMASK_LEN+1);
    if (parms.psmask == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.psmask", 0);
        return ( EXIT_FAILURE );
    }
    memset(parms.psmask, 0x0, INVIZ_MAX_PSMASK_LEN+1);
    parms.originalpsname = malloc(INVIZ_MAX_PSMASK_LEN+1);
    if (parms.originalpsname == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.originalpsname", 0);
        return ( EXIT_FAILURE );
    }
    memset(parms.originalpsname, 0x0, INVIZ_MAX_PSMASK_LEN+1);
    parms.programname = malloc(MAX_FILENAME_LEN+1);
    if (parms.programname == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.programname", 0);
        return ( EXIT_FAILURE );
    }
    memset(parms.programname, 0x0, MAX_FILENAME_LEN+1);
    parms.devrandom = malloc(MAX_FILENAME_LEN+1);
    if (parms.devrandom == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.devrandom", 0);
        return ( EXIT_FAILURE );
    }
    set_devrandom(INVIZ_DEFAULT_DEVRANDOM, strlen(INVIZ_DEFAULT_DEVRANDOM));
    //
    parms.filename = malloc(MAX_FILENAME_LEN+1);
    if (parms.filename == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.filename", 0);
        return ( EXIT_FAILURE );
    }
    //
    parms.packetfilter = malloc(MAX_PACKETFILTER_LEN+1);
    if (parms.packetfilter == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.packetfilter", 0);
        return ( EXIT_FAILURE );
    }
    //
    parms.triggerhostname = malloc(NI_MAXHOST+1);
    if (parms.triggerhostname == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.triggerhostname", 0);
        return ( EXIT_FAILURE );
    }
    //
    parms.server_ip = NULL;
    parms.server_ip4 = malloc(MAX_IP4_LEN+1);
    if (parms.server_ip4 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.server_ip4", 0);
        return ( EXIT_FAILURE );
    }
    char default_server4[] = ""; // no default - use "127.0.0.1" default instead?
    set_serverip4(default_server4, strlen(default_server4)); // default server ip4
    //
    parms.server_ip6 = malloc(MAX_IP6_LEN+1);
    if (parms.server_ip6 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.server_ip6", 0);
        return ( EXIT_FAILURE );
    }
    char default_server6[] = ""; // no default - use "::1" default instead?
    set_serverip6(default_server6, strlen(default_server6)); // default server ip6
    parms.server_hostname = NULL;
    //
    parms.dns_ip = NULL;
    parms.dns_ip4 = malloc(MAX_IP4_LEN+1);
    if (parms.dns_ip4 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.dns_ip4", 0);
        return ( EXIT_FAILURE );
    }
    char default_dns4[] = INVIZ_DEFAULT_DNS_IP4;
    set_dnsip4(default_dns4, strlen(default_dns4)); // default dns server ip4
    //
    parms.dns_ip6 = malloc(MAX_IP6_LEN+1);
    if (parms.dns_ip6 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.dns_ip6", 0);
        return ( EXIT_FAILURE );
    }
    char default_dns6[] = INVIZ_DEFAULT_DNS_IP6;
    set_dnsip6(default_dns6, strlen(default_dns6)); // default dns server ip6
    parms.dns_hostname = NULL;
    //
    parms.client_ip = NULL;
    parms.client_ip4 = malloc(MAX_IP4_LEN+1);
    if (parms.client_ip4 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.client_ip4", 0);
        return ( EXIT_FAILURE );
    }
    char default_client4[] = ""; // no default
    set_clientip4(default_client4, strlen(default_client4)); // default client ip4
    //
    parms.client_ip6 = malloc(MAX_IP6_LEN+1);
    if (parms.client_ip6 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.client_ip6", 0);
        return ( EXIT_FAILURE );
    }
    char default_client6[] = ""; // no default
    set_clientip6(default_client6, strlen(default_client6)); // default client ip6
    parms.client_hostname = NULL;
    //
    parms.external_ip = NULL;
    parms.external_ip4 = NULL;
    parms.external_ip6 = NULL;
    parms.external_hostname = NULL;
    //
    parms.fko_ctx = NULL;
    //
    set_uname();
    //
    return ( EXIT_SUCCESS );
}
//
// cleanup parameters
//
void
cleanup_parms() {
    if (get_sockraw() > 0) {
        close(get_sockraw());
        set_sockraw(0);
    }
    if (get_pcapstruct()) {
        pcap_close(get_pcapstruct());
        set_pcapstruct(NULL);
    }
    if (parms.filename) {
        free(parms.filename);
        parms.filename = NULL;
    }
    if (parms.pksavefilename) {
        free(parms.pksavefilename);
        parms.pksavefilename= NULL;
    }
    if (parms.pkaccessfilename) {
        free(parms.pkaccessfilename);
        parms.pkaccessfilename= NULL;
    }
    if (parms.pkconfigfilename) {
        free(parms.pkconfigfilename);
        parms.pkconfigfilename= NULL;
    }
    if (parms.ifname) {
        free(parms.ifname);
        parms.ifname = NULL;
    }
    if (parms.ifnameip) {
        free(parms.ifnameip);
        parms.ifnameip = NULL;
    }
    if (parms.packetfilter) {
        free(parms.packetfilter);
        parms.packetfilter = NULL;
    }
    if (parms.triggerhostname) {
        free(parms.triggerhostname);
        parms.triggerhostname = NULL;
    }
    if (parms.server_ip4) {
        free(parms.server_ip4);
        parms.server_ip4 = NULL;
    }
    if (parms.server_ip6) {
        free(parms.server_ip6);
        parms.server_ip6 = NULL;
    }
    if (parms.server_ip) {
        free(parms.server_ip);
        parms.server_ip = NULL;
    }
    if (parms.server_hostname) {
        free(parms.server_hostname);
        parms.server_hostname = NULL;
    }
    if (parms.logfilename) {
        free(parms.logfilename);
        parms.logfilename = NULL;
    }
    if (parms.dns_ip4) {
        free(parms.dns_ip4);
        parms.dns_ip4 = NULL;
    }
    if (parms.dns_ip6) {
        free(parms.dns_ip6);
        parms.dns_ip6 = NULL;
    }
    if (parms.dns_ip) {
        free(parms.dns_ip);
        parms.dns_ip = NULL;
    }
    if (parms.dns_hostname) {
        free(parms.dns_hostname);
        parms.dns_hostname = NULL;
    }
    if (parms.keyseed) {
        free(parms.keyseed);
        parms.keyseed = NULL;
    }
    if (parms.psmask) {
        free(parms.psmask);
        parms.psmask = NULL;
    }
    if (parms.originalpsname) {
        free(parms.originalpsname);
        parms.originalpsname = NULL;
    }
    if (parms.programname) {
        free(parms.programname);
        parms.programname = NULL;
    }
    if (parms.devrandom) {
        free(parms.devrandom);
        parms.devrandom = NULL;
    }
    if (parms.client_ip4) {
        free(parms.client_ip4);
        parms.client_ip4 = NULL;
    }
    if (parms.client_ip6) {
        free(parms.client_ip6);
        parms.client_ip6 = NULL;
    }
    if (parms.client_ip) {
        free(parms.client_ip);
        parms.client_ip = NULL;
    }
    if (parms.client_hostname) {
        free(parms.client_hostname);
        parms.client_hostname = NULL;
    }
    if (parms.pktransfercommand) {
        free(parms.pktransfercommand);
        parms.pktransfercommand = NULL;
    }
    if (parms.pkwipecommand) {
        free(parms.pkwipecommand);
        parms.pkwipecommand = NULL;
    }
    if (parms.wipetarget) {
        free(parms.wipetarget);
        parms.wipetarget = NULL;
    }
    if (parms.pkpasscommand) {
        free(parms.pkpasscommand);
        parms.pkpasscommand = NULL;
    }
    if (parms.pkinotifycommand) {
        free(parms.pkinotifycommand);
        parms.pkinotifycommand = NULL;
    }
    if (parms.pkbeaconcurrentprivatedest) {
        free(parms.pkbeaconcurrentprivatedest);
        parms.pkbeaconcurrentprivatedest = NULL;
    }
    if (parms.pkbeaconcurrentpublicdest) {
        free(parms.pkbeaconcurrentpublicdest);
        parms.pkbeaconcurrentpublicdest = NULL;
    }
    if (parms.pkbeaconmsg) {
        free(parms.pkbeaconmsg);
        parms.pkbeaconmsg = NULL;
    }
    if (parms.pkbeaconcommand) {
        free(parms.pkbeaconcommand);
        parms.pkbeaconcommand = NULL;
    }
    if (parms.pkbeaconpublic) {
        free(parms.pkbeaconpublic);
        parms.pkbeaconpublic = NULL;
    }
    if (parms.pkbeaconprivate) {
        free(parms.pkbeaconprivate);
        parms.pkbeaconprivate = NULL;
    }
    if (parms.inotifytarget) {
        free(parms.inotifytarget);
        parms.inotifytarget = NULL;
    }
    if (parms.beacontarget) {
        free(parms.beacontarget);
        parms.beacontarget = NULL;
    }
    if (parms.transfertarget) {
        free(parms.transfertarget);
        parms.transfertarget = NULL;
    }
    if (parms.inotifyoptions) {
        free(parms.inotifyoptions);
        parms.inotifyoptions = NULL;
    }
    if (parms.transferoptions) {
        free(parms.transferoptions);
        parms.transferoptions = NULL;
    }
    if (parms.pkclient) {
        free(parms.pkclient);
        parms.pkclient = NULL;
    }
    if (parms.inotifylog) {
        free(parms.inotifylog);
        parms.inotifylog = NULL;
    }
    if (parms.inotifyechodns) {
        free(parms.inotifyechodns);
        parms.inotifyechodns = NULL;
    }
    if (parms.inotifyechokey) {
        free(parms.inotifyechokey);
        parms.inotifyechokey = NULL;
    }
    if (parms.pkkeylogcommand) {
        free(parms.pkkeylogcommand);
        parms.pkkeylogcommand = NULL;
    }
    if (parms.pkservercommand) {
        free(parms.pkservercommand);
        parms.pkservercommand = NULL;
    }
    if (parms.pkaccessrequest) {
        free(parms.pkaccessrequest);
        parms.pkaccessrequest = NULL;
    }
    if (parms.nataccessrequest) {
        free(parms.nataccessrequest);
        parms.nataccessrequest = NULL;
    }
    if (parms.external_ip4) {
        free(parms.external_ip4);
        parms.external_ip4 = NULL;
    }
    if (parms.external_ip6) {
        free(parms.external_ip6);
        parms.external_ip6 = NULL;
    }
    if (parms.external_ip) {
        free(parms.external_ip);
        parms.external_ip = NULL;
    }
    if (parms.external_hostname) {
        free(parms.external_hostname);
        parms.external_hostname = NULL;
    }
    if (parms.pkallow_ip4) {
        free(parms.pkallow_ip4);
        parms.pkallow_ip4 = NULL;
    }
    if (parms.pkallow_ip6) {
        free(parms.pkallow_ip6);
        parms.pkallow_ip6 = NULL;
    }
    if (parms.pkallow_ip) {
        free(parms.pkallow_ip);
        parms.pkallow_ip = NULL;
    }
    if (parms.pkallow_hostname) {
        free(parms.pkallow_hostname);
        parms.pkallow_hostname = NULL;
    }
    if (parms.gpg_recipientkey) {
        free(parms.gpg_recipientkey);
        parms.gpg_recipientkey = NULL;
    }
    if (parms.gpg_signerkey) {
        free(parms.gpg_signerkey);
        parms.gpg_signerkey = NULL;
    }
    if (parms.gpg_homedir) {
        free(parms.gpg_homedir);
        parms.gpg_homedir = NULL;
    }
    if (parms.gpg_keyfile) {
        free(parms.gpg_keyfile);
        parms.gpg_keyfile = NULL;
    }
    if (parms.proxy) {
        free(parms.proxy);
        parms.proxy = NULL;
    }
    if (parms.proxytype) {
        free(parms.proxytype);
        parms.proxytype = NULL;
    }
    if (parms.spoof_ip4) {
        free(parms.spoof_ip4);
        parms.spoof_ip4 = NULL;
    }
    if (parms.spoof_ip6) {
        free(parms.spoof_ip6);
        parms.spoof_ip6 = NULL;
    }
    if (parms.spoof_ip) {
        free(parms.spoof_ip);
        parms.spoof_ip = NULL;
    }
    if (parms.spoof_hostname) {
        free(parms.spoof_hostname);
        parms.spoof_hostname = NULL;
    }
    if (parms.protocoldesc) {
        free(parms.protocoldesc);
        parms.protocoldesc = NULL;
    }
    if (parms.destination_ip) {
        free(parms.destination_ip);
        parms.destination_ip = NULL;
    }
    if (parms.source_ip) {
        free(parms.source_ip);
        parms.source_ip = NULL;
    }
    if (parms.pcap_device) {
        free(parms.pcap_device);
        parms.pcap_device = NULL;
    }
    if (parms.spa_packet_struct) {
        free(parms.spa_packet_struct);
        parms.spa_packet_struct = NULL;
    }
    if (parms.firewall_chain) {
        free(parms.firewall_chain);
        parms.firewall_chain = NULL;
    }
    if (parms.firewall_config) {
        free(parms.firewall_config);
        parms.firewall_config = NULL;
    }
    if (parms.snat_translate_ip) {
        free(parms.snat_translate_ip);
        parms.snat_translate_ip = NULL;
    }
    if (parms.pkpassword) {
        free(parms.pkpassword);
        parms.pkpassword = NULL;
    }
    if (parms.uname) {
        free(parms.uname);
        parms.uname = NULL;
    }
}
//
// get protocol enabled/disabled status
//
void
set_allprotocolsdisabled() {
    set_useudp(FALSE);
    set_usetcp(FALSE);
    set_useip(FALSE);
    set_useicmp(FALSE);
    set_usesctp(FALSE);
    set_useraw(FALSE);
    set_useproxy(FALSE);
}
uint8_t
get_usedprotocol() {
    if (get_useudp()) {
        return IPPROTO_UDP;
    } else if (get_usetcp()) {
        return IPPROTO_TCP;
    } else if (get_useicmp()) {
        return IPPROTO_ICMP;
    } else if (get_useraw()) {
        return IPPROTO_RAW;
    } else if (get_useproxy()) {
        if (get_usehttpproxy()) {
            return INVIZ_PROTOCOL_HTTP_PROXY;
        } else if (get_usesocksproxy()) {
            return INVIZ_PROTOCOL_SOCKS_PROXY;
        } else {
            return INVIZ_PROTOCOL_UNKNOWN_PROXY;
        }
    } else if (get_useip()) {
        return IPPROTO_IP;
    } else if (get_usesctp()) {
        return IPPROTO_SCTP;
    } else {
        return INVIZ_PROTOCOL_UNKNOWN;
    }
}
char *
get_usedprotocoldesc(uint8_t p) {
    if (p == IPPROTO_UDP) {
        return IPPROTO_UDP_DESC;
    } else if (p == IPPROTO_TCP) {
        return IPPROTO_TCP_DESC;
    } else if (p == IPPROTO_RAW) {
        return IPPROTO_RAW_DESC;
    } else if (p == IPPROTO_IP) {
        return IPPROTO_IP_DESC;
    } else if (p == IPPROTO_SCTP) {
        return IPPROTO_SCTP_DESC;
    } else if (p == INVIZ_PROTOCOL_HTTP_PROXY) {
        return IPPROTO_HTTPPROXY_DESC" "IPPROTO_PROXY_DESC;
    } else if (p == INVIZ_PROTOCOL_SOCKS_PROXY) {
        return IPPROTO_SOCKSPROXY_DESC" "IPPROTO_PROXY_DESC;
    } else if (p == INVIZ_PROTOCOL_UNKNOWN_PROXY) {
        return INVIZ_PROTOCOL_UNKNOWN_DESC" "IPPROTO_PROXY_DESC;
    } else if (p == IPPROTO_ICMP) {
        return IPPROTO_ICMP_DESC;
    } else {
        return INVIZ_PROTOCOL_UNKNOWN_DESC;
    }
}
void
set_protocoldesc(char * fn, int len) {
    if (parms.protocoldesc == NULL) {
        parms.protocoldesc = malloc(MAX_PROTOCOL_DESC_LEN+1);
    }
    if (parms.protocoldesc == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.protocoldesc", 0);
    } else {
        if ( len <= 0 ) {
            *parms.protocoldesc = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_PROTOCOL_DESC_LEN) ) {
            strncpy(parms.protocoldesc, fn, len);
            parms.protocoldesc[len] = '\0';
        }
    }
}
char *
get_protocoldesc() {
    return parms.protocoldesc;
}
//
// get/set client source port
//
inline int
get_srcport() {
    return parms.srcport;
}
int
get_minsrcport() {
    return (int)INVIZ_MIN_SRCPORT;
}
int
get_maxsrcport() {
    return (int)INVIZ_MAX_SRCPORT;
}
void
set_srcport(int p) {
    if (p <= get_maxsrcport() && p >= get_minsrcport()) {
        parms.srcport = p;
    }
}
//
// get/set server destination port for packet
//
inline uint
get_packetdestport() {
    return parms.packetdestport;
}
void
set_packetdestport(uint p) {
    parms.packetdestport = p;
}
//
// get/set pk beacon destination port
//
inline uint
get_pkbeacontcpdestport() {
    return parms.pkbeacontcpdestport; // tcp
}
inline uint
get_pkbeaconudpdestport() {
    return parms.pkbeaconudpdestport; // udp
}
void
set_pkbeacontcpdestport(uint p) {
    parms.pkbeacontcpdestport = p; // tcp
}
void
set_pkbeaconudpdestport(uint p) {
    parms.pkbeaconudpdestport = p; // udp
}
//
// get/set file transfer destination port
//
inline uint
get_pktransferdestport() {
    return parms.pktransferdestport;
}
void
set_pktransferdestport(uint p) {
    parms.pktransferdestport = p;
}
//
// get/set file transfer source port
//
inline uint
get_pktransfersrcport() {
    return parms.pktransfersrcport;
}
void
set_pktransfersrcport(uint p) {
    parms.pktransfersrcport = p;
}
//
// get/set client source port for packet
//
inline uint
get_packetsrcport() {
    return parms.packetsrcport;
}
void
set_packetsrcport(uint p) {
    parms.packetsrcport = p;
}
//
// get/set pid file descriptor
//
void
set_pidfd(int fd) {
    parms.pidfd = fd;
}
int
get_pidfd() {
    return parms.pidfd;
}
//
// get/set raw socket fd
//
inline int
get_sockraw() {
    return parms.sockraw;
}
void
set_sockraw(int sock) {
    parms.sockraw = sock;
}
//
// get/set inotify file descriptor
//
void
set_inotifyfd(int fd) {
    parms.inotifyfd = fd;
}
int
get_inotifyfd() {
    return parms.inotifyfd;
}
//
// get/set inotify watch descriptor
//
void
set_inotifywd(int wd) {
    parms.inotifywd = wd;
}
int
get_inotifywd() {
    return parms.inotifywd;
}
//
// get/set SIGCHLD signal handler
//
inline int
get_setsignalchild() {
    return parms.bsigchld;
}
void
set_setsignalchild(int s) {
    if ((s == TRUE) || (s == FALSE)) {
        parms.bsigchld = s;
    }
}
//
// get/set port knocking beacon minimum/maximum interval (seconds)
//
int
get_pkbeaconmininterval() {
    return parms.pkbeaconmininterval;
}
int
get_pkbeaconmaxinterval() {
    return parms.pkbeaconmaxinterval;
}
void
set_pkbeaconmininterval(int s) {
    if ( (s >= 0) && (s <= INVIZ_MAX_PKBEACON_INTERVAL) ) {
        if ( (get_pkbeaconmaxinterval() == 0) || (get_pkbeaconmaxinterval() >= s) ) {
            parms.pkbeaconmininterval = s;
        } else {
            parms.pkbeaconmininterval = get_pkbeaconmaxinterval();
        }
    }
}
void
set_pkbeaconmaxinterval(int s) {
    if ( (s >= 0) && (s <= INVIZ_MAX_PKBEACON_INTERVAL) ) {
        if ( (get_pkbeaconmininterval() == 0) || (get_pkbeaconmininterval() <= s) ) {
            parms.pkbeaconmaxinterval = s;
        } else {
            parms.pkbeaconmaxinterval = get_pkbeaconmininterval();
        }
    }
}
//
// get/set port knocking min/max encoded len
//
inline int
get_pkminencodedlen() {
    return parms.pkminencodedlen;
}
inline int
get_pkmaxencodedlen() {
    return parms.pkmaxencodedlen;
}
void
set_pkminencodedlen(int len) {
    parms.pkminencodedlen = len;
}
void
set_pkmaxencodedlen(int len) {
    parms.pkmaxencodedlen = len;
}
//
// get/set port knocking min/max data len
//
inline int
get_pkmindatalen() {
    return parms.pkmindatalen;
}
inline int
get_pkmaxdatalen() {
    return parms.pkmaxdatalen;
}
void
set_pkmindatalen(int len) {
    parms.pkmindatalen = len;
}
void
set_pkmaxdatalen(int len) {
    parms.pkmaxdatalen = len;
}
//
// get/set port knocking port
//
int
get_pkport() {
    return parms.pkport;
}
char *
get_pkportparm() {
    return (char *)pkport_parm;
}
int
get_pkminport() {
    return (int)INVIZ_MIN_PKPORT;
}
int
get_pkmaxport() {
    return (int)INVIZ_MAX_PKPORT;
}
int
get_pkportfirst() {
    return parms.pkportfirst;
}
int
get_pkportlast() {
    return parms.pkportlast;
}
int
get_pkportrandom() {
    return parms.pkportrandom;
}
void
set_pkportrandom(int b) {
    if (b == TRUE || b == FALSE) {
        parms.pkportrandom = b;
    }
}
void
set_pkportfirst(int p) {
    if (p >= get_pkminport() && p <= get_pkmaxport()) {
        if (!get_pkportlast() || p <= get_pkportlast()) {
            parms.pkportfirst = p;
        }
    }
}
void
set_pkportlast(int p) {
    if (p >= get_pkminport() && p <= get_pkmaxport()) {
        if (!get_pkportfirst() || p >= get_pkportfirst()) {
            parms.pkportlast = p;
        }
    }
}
void
set_pkport(int p) {
    if (p <= get_pkmaxport() && p >= get_pkminport()) {
        parms.pkport = p;
    }
}
//
// get/set data link offset (for packet encapsulation calcs)
//
int
get_datalinkoffset() {
    return parms.datalinkoffset;
}
void
set_datalinkoffset(int p) {
    parms.datalinkoffset = p;
}
//
// get/set tcp server pid (server only)
//
int
get_tcpserverpid() {
    return parms.tcpserverpid;
}
void
set_tcpserverpid(int p) {
    parms.tcpserverpid = p;
}
//
// get/set tcp server port (server only)
//
int
get_tcpserverport() {
    return parms.tcpserverport;
}
int
get_maxtcpserverport() {
    return MAX_UNPRIVILEGED_PORT;
}
int
get_mintcpserverport() {
    return MAX_PRIVILEGED_PORT+1;
}
void
set_tcpserverport(int p) {
    if (p >= get_mintcpserverport() && p <= get_maxtcpserverport()) {
        parms.tcpserverport = p;
    }
}
//
// get/set use port knocking access file (on server)
//
int
get_usepkaccessfile() {
    return parms.bpkaccessfile;
}
void
set_usepkaccessfile(int p) {
    if ( (p == TRUE) || (p == FALSE) ) {
        parms.bpkaccessfile = p;
    }
}
//
// get/set port knocking beacon enabled or disabled
//
int
get_pkbeacon() {
    return parms.pkbeacon;
}
void
set_pkbeacon(int p) {
    if ( (p == TRUE) || (p == FALSE) ) {
        parms.pkbeacon = p;
    }
}
//
// get/set port knocking beacon on private subnet or public internet
//
char *
get_pkbeaconprivate() {
    return parms.pkbeaconprivate;
}
char *
get_pkbeaconpublic() {
    return parms.pkbeaconpublic;
}
void
set_pkbeaconprivate(char *fn, int len) {
    if (parms.pkbeaconprivate == NULL) {
        parms.pkbeaconprivate = malloc(INVIZ_MAX_PKBEACON_LEN+1);
    }
    if (parms.pkbeaconprivate == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkbeaconprivate", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkbeaconprivate = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKBEACON_LEN) ) {
            strncpy(parms.pkbeaconprivate, fn, len);
            parms.pkbeaconprivate[len] = '\0';
        }
    }
}
void
set_pkbeaconpublic(char *fn, int len) {
    if (parms.pkbeaconpublic == NULL) {
        parms.pkbeaconpublic = malloc(INVIZ_MAX_PKBEACON_LEN+1);
    }
    if (parms.pkbeaconpublic == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkbeaconpublic", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkbeaconpublic = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKBEACON_LEN) ) {
            strncpy(parms.pkbeaconpublic, fn, len);
            parms.pkbeaconpublic[len] = '\0';
        }
    }
}
char *
get_pkbeaconprivateparm() {
    return (char *)pkbeaconprivate_parm;
}
char *
get_pkbeaconpublicparm() {
    return (char *)pkbeaconpublic_parm;
}
//
// get/set port knocking on private subnet
//
int
get_pkprivatesubnet() {
    return parms.pkprivate;
}
void
set_pkprivatesubnet(int p) {
    if ( (p == TRUE) || (p == FALSE) ) {
        parms.pkprivate = p;
    }
}
char *
get_pkprivateparm() {
    return (char *)pkp_parm;
}
char *
get_pknotprivateparm() {
    return (char *)pknotp_parm;
}
//
// get/set current port knocking beacon current private destination IP address
//
char *
get_pkbeaconcurrentprivatedest() {
    return parms.pkbeaconcurrentprivatedest;
}
void
set_pkbeaconcurrentprivatedest(char *fn, int len) {
    if (parms.pkbeaconcurrentprivatedest == NULL) {
        parms.pkbeaconcurrentprivatedest = malloc(MAX_IP4_LEN+1);
    }
    if (parms.pkbeaconcurrentprivatedest == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkbeaconcurrentprivatedest", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkbeaconcurrentprivatedest = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_IP4_LEN) ) {
            strncpy(parms.pkbeaconcurrentprivatedest, fn, len);
            parms.pkbeaconcurrentprivatedest[len] = '\0';
        }
    }
}
//
// get/set current port knocking beacon current public destination IP address
//
char *
get_pkbeaconcurrentpublicdest() {
    return parms.pkbeaconcurrentpublicdest;
}
void
set_pkbeaconcurrentpublicdest(char *fn, int len) {
    if (parms.pkbeaconcurrentpublicdest == NULL) {
        parms.pkbeaconcurrentpublicdest = malloc(MAX_IP4_LEN+1);
    }
    if (parms.pkbeaconcurrentpublicdest == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkbeaconcurrentpublicdest", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkbeaconcurrentpublicdest = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_IP4_LEN) ) {
            strncpy(parms.pkbeaconcurrentpublicdest, fn, len);
            parms.pkbeaconcurrentpublicdest[len] = '\0';
        }
    }
}
//
// get next port knocking beacon private destination
//
char *
get_pkbeaconnextprivatedest() {
    uint dest_suffix;
    uint self_suffix;
    uint lowest = 1;
    uint highest = 254;
    if (!get_pkbeacon()) {
        return (NULL);
    }
    char self_suffix_str[4];
    memset(self_suffix_str, 0x0, 4);
    char result[MAX_IP4_LEN+1];
    memset(result, 0x0, MAX_IP4_LEN+1);
    char self[MAX_IP4_LEN+1];
    memset(self, 0x0, MAX_IP4_LEN+1);
    if (validate_ip4(get_ifnameip()) == EXIT_SUCCESS) {
        strncpy(self, get_ifnameip(), strlen(get_ifnameip()));
    } else {
        return (NULL);
    }
    const char *ndx = self;
    char next[MAX_IP4_LEN+1];
    int i = 0;
    int dots = 0;
    memset(next, 0x0, MAX_IP4_LEN+1);
    while (*ndx != '\0') {
        next[i] = *ndx;
        if (*ndx == '.') {
            dots++;
        }
        if (dots == 3) {
            sprintf(self_suffix_str, "%d", *ndx+1);
            self_suffix = (uint)atoi(self_suffix_str);
            dest_suffix = ((rand() % (highest - lowest)) + lowest);
            while (dest_suffix == self_suffix) {
                //
                // do not generate destination IP address
                // same as self
                //
                dest_suffix = ((rand() % (highest - lowest)) + lowest);
            }
            sprintf(result, "%s%d", next, dest_suffix);
            break;
        }
        i++;
        ndx++;
    }
    set_pkbeaconcurrentprivatedest(result, strlen(result));
    return (get_pkbeaconcurrentprivatedest());
}
//
// get next port knocking beacon private destination
//
char *
get_pkbeaconnextpublicdest() {
    uint dest_suffix;
    uint self_suffix;
    uint lowest = 1;
    uint highest = 254;
    if (!get_pkbeacon()) {
        return (NULL);
    }
    char self_suffix_str[4];
    memset(self_suffix_str, 0x0, 4);
    char result[MAX_IP4_LEN+1];
    memset(result, 0x0, MAX_IP4_LEN+1);
    char self[MAX_IP4_LEN+1];
    memset(self, 0x0, MAX_IP4_LEN+1);
    if (validate_ip4(get_externalip4()) == EXIT_SUCCESS) {
        strncpy(self, get_externalip4(), strlen(get_externalip4()));
    } else {
        return (NULL);
    }
    const char *ndx = self;
    char next[MAX_IP4_LEN+1];
    int i = 0;
    int dots = 0;
    memset(next, 0x0, MAX_IP4_LEN+1);
    while (*ndx != '\0') {
        next[i] = *ndx;
        if (*ndx == '.') {
            dots++;
        }
        if (dots == 3) {
            sprintf(self_suffix_str, "%d", *ndx+1);
            self_suffix = (uint)atoi(self_suffix_str);
            dest_suffix = ((rand() % (highest - lowest)) + lowest);
            while (dest_suffix == self_suffix) {
                //
                // do not generate destination IP address
                // same as self
                //
                dest_suffix = ((rand() % (highest - lowest)) + lowest);
            }
            sprintf(result, "%s%d", next, dest_suffix);
            break;
        }
        i++;
        ndx++;
    }
    set_pkbeaconcurrentpublicdest(result, strlen(result));
    return (get_pkbeaconcurrentpublicdest());
}
//
// get/set port knocking beacon message
//
char *
get_pkbeaconmsg() {
    return parms.pkbeaconmsg;
}
void
set_pkbeaconmsg(char *fn, int len) {
    if (parms.pkbeaconmsg == NULL) {
        parms.pkbeaconmsg = malloc(INVIZ_MAX_PKBEACON_LEN+1);
    }
    if (parms.pkbeaconmsg == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkbeaconmsg", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkbeaconmsg = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKBEACON_LEN) ) {
            strncpy(parms.pkbeaconmsg, fn, len);
            parms.pkbeaconmsg[len] = '\0';
        }
    }
}
char *
get_pkbeaconmsgparm() {
    return (char *)pkbeaconmsg_parm;
}
//
// get/set use transaction id field for file transfer contents
//
int
get_usetransid() {
    return parms.btransid;
}
void
set_usetransid(int p) {
    if ( (p == TRUE) || (p == FALSE) ) {
        parms.btransid = p;
    }
}
//
// get/set port knocking type
//
int
get_pktype() {
    return parms.pktype;
}
void
set_pktype(int pkt) {
    enum pktypes pktype;
    char logmsg[512];
    int current_pkt = get_pktype();
    if ( pkt == (pktype=PKTYPE0) ||
            pkt == (pktype=PKTYPE1) ||
            pkt == (pktype=PKTYPE2) ||
            pkt == (pktype=PKTYPE3) ||
            pkt == (pktype=PKTYPE4) ||
            pkt == (pktype=PKTYPE5) ||
            pkt == (pktype=PKTYPE6) ||
            pkt == (pktype=PKTYPE7) ||
            pkt == (pktype=PKTYPE8) ||
            pkt == (pktype=PKTYPE9)) {
        if ((current_pkt != pkt) && (current_pkt != INVIZ_PKTYPE_ANY)
                && (current_pkt != INVIZ_PKTYPE_DISABLED)) {
            sprintf(logmsg, "port knocking type %i changed to %i", current_pkt, pkt);
            logit(_LOG_WARNING, logmsg, 0);
        }
        parms.pktype = pkt;
    }
    set_usepk(get_pktype() != (int)INVIZ_PKTYPE_DISABLED); // port knocking being used?
}
char *
get_pktypeparm() {
    return (char *)pktype_parm;
}
//
// get/set port knocking digest type
//
short
get_pkdigest() {
    return parms.pkdigest;
}
void
set_pkdigest(short pkd) {
    if ( pkd == FKO_DIGEST_UNKNOWN ||
            pkd == FKO_DIGEST_MD5 ||
            pkd == FKO_DIGEST_SHA1 ||
            pkd == FKO_DIGEST_SHA256 ||
            pkd == FKO_DIGEST_SHA384 ||
            pkd == FKO_DIGEST_SHA512) {
        parms.pkdigest = pkd;
    }
}
char *
get_pkdigestdesc(short pkd) {
    switch(pkd)
    {
        case FKO_DIGEST_UNKNOWN:
            return "n/a";
        case FKO_DIGEST_MD5:
            return "MD5";
        case FKO_DIGEST_SHA1:
            return "SHA1";
        case FKO_DIGEST_SHA256:
            return "SHA256";
        case FKO_DIGEST_SHA384:
            return "SHA384";
        case FKO_DIGEST_SHA512:
            return "SHA512";
        default:
            return "";
    }
}
char *
get_pkdigestparm() {
    return (char *)pkdigest_parm;
}
//
// get/set port knocking client timeout
//
int
get_pktimeout() {
    return parms.pktimeout;
}
void
set_pktimeout(int s) {
    if ( s >= get_minpktimeout() && s <= get_maxpktimeout() ) {
        parms.pktimeout = s;
    }
}
int
get_minpktimeout() {
    return INVIZ_MIN_PKTIMEOUT;
}
int
get_maxpktimeout() {
    return INVIZ_MAX_PKTIMEOUT;
}
char *
get_pktimeoutparm() {
    return (char *)pktimeout_parm;
}
//
// get/set use port knocking
//
int
get_usepk() {
    return parms.bportknocking;
}
void
set_usepk(int pk) {
    if ((pk == TRUE) || (pk == FALSE)) {
        parms.bportknocking = pk;
    }
}
//
// get/set use gpg
//
int
get_usegpg() {
    return parms.bgpg;
}
void
set_usegpg(int g) {
    if ((g == TRUE) || (g == FALSE)) {
        parms.bgpg = g;
    }
}
//
// get/set use gpg agent
//
int
get_usegpg_agent() {
    return parms.bgpg_agent;
}
void
set_usegpg_agent(int g) {
    if ((g == TRUE) || (g == FALSE)) {
        parms.bgpg_agent = g;
    }
}
//
// get/set verbose level parameter
//
void
set_verbose(int v) {
    parms.iverbose = v;
}
inline int
get_verbose() {
    return parms.iverbose;
}
char *
get_verboseparm() {
    return (char *)v_parm;
}
char *
get_quietparm() {
    return (char *)quiet_parm;
}
//
// get/set duplicates parameter
//
void
set_duplicates(int v) {
    parms.iduplicates = v;
}
inline int
get_duplicates() {
    return parms.iduplicates;
}
//
// get/set testing
//
void
set_testing(int v) {
    if ( (v == TRUE) || (v == FALSE) ) {
        parms.btesting = v;
    }
}
inline int
get_testing() {
    return parms.btesting;
}
//
// get/set allow firewall output
//
void
set_fwallowoutput(int v) {
    if ( (v == TRUE) || (v == FALSE) ) {
        parms.bfwoutput = v;
    }
}
inline int
get_fwallowoutput() {
    return parms.bfwoutput;
}
//
// get/set allow firewall forwarding
//
void
set_fwallowforwarding(int v) {
    if ( (v == TRUE) || (v == FALSE) ) {
        parms.bfwforwarding = v;
    }
}
inline int
get_fwallowforwarding() {
    return parms.bfwforwarding;
}
//
// get/set allow firewall snat
//
void
set_fwallowsnat(int v) {
    if ( (v == TRUE) || (v == FALSE) ) {
        parms.bfwsnat = v;
    }
}
inline int
get_fwallowsnat() {
    return parms.bfwsnat;
}
//
// get/set allow spa over http
//
void
set_allowspaoverhttp(int v) {
    if ( (v == TRUE) || (v == FALSE) ) {
        parms.bspaoverhttp = v;
    }
}
inline int
get_allowspaoverhttp() {
    return parms.bspaoverhttp;
}
char *
get_allowspaoverhttpparm() {
    return (char *)spahttp_parm;
}
//
// get/set promiscuous mode
//
void
set_promiscuous(int v) {
    if ( (v == TRUE) || (v == FALSE) ) {
        parms.bpromiscuous = v;
    }
}
inline int
get_promiscuous() {
    return parms.bpromiscuous;
}
//
// get/set use ip6 parameter
//
int
get_useip6() {
    return parms.bip6;
}
void
set_useip6(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bip6 = u;
    }
}
//
// get/set use libfko pcap logic
//
int
get_usefkopcap() {
    return parms.bfkopcap;
}
void
set_usefkopcap(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bfkopcap = u;
    }
}
//
// get/set use ip4 parameter
//
int
get_useip4() {
    return parms.bip4;
}
void
set_useip4(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bip4 = u;
    }
}
char *
get_ip4parm() {
    return (char *)ip4_parm;
}
//
// get/set use udp parameter for port knocking beacons
//
inline int
get_pkbeaconuseudp() {
    return parms.bpkbeaconudp;
}
void
set_pkbeaconuseudp(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bpkbeaconudp = u;
        parms.bpkbeacontcp = !u; // either one or the other
    }
}
//
// get/set use tcp parameter for port knocking beacons
//
inline int
get_pkbeaconusetcp() {
    return parms.bpkbeacontcp;
}
void
set_pkbeaconusetcp(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bpkbeacontcp = u;
        parms.bpkbeaconudp = !u; // either one or the other
    }
}
//
// get/set use udp parameter
//
inline int
get_useudp() {
    return parms.budp;
}
void
set_useudp(int u) {
    if (u == FALSE || u == TRUE) {
        parms.budp = u;
    }
}
//
// get/set use tcp server parameter
//
inline int
get_usetcpserver() {
    return parms.btcpserver;
}
void
set_usetcpserver(int u) {
    if (u == FALSE || u == TRUE) {
        parms.btcpserver = u;
    }
}
//
// get/set use proxy parameter
//
inline int
get_useproxy() {
    return parms.bproxy;
}
void
set_useproxy(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bproxy = u;
    }
}
//
// get/set interactive parameter
//
inline int
get_interactive() {
    return parms.binteractive;
}
void
set_interactive(int u) {
    if (u == FALSE || u == TRUE) {
        parms.binteractive = u;
    }
}
//
// get/set use tcp parameter
//
inline int
get_usetcp() {
    return parms.btcp;
}
void
set_usetcp(int u) {
    if (u == FALSE || u == TRUE) {
        parms.btcp = u;
    }
}
//
// get/set use sctp parameter
//
inline int
get_usesctp() {
    return parms.bsctp;
}
void
set_usesctp(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bsctp = u;
    }
}
//
// get/set use ip parameter
//
inline int
get_useip() {
    return parms.bip;
}
void
set_useip(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bip = u;
    }
}
//
// get/set use raw protocol parameter
//
inline int
get_useraw() {
    return parms.braw;
}
void
set_useraw(int u) {
    if (u == FALSE || u == TRUE) {
        parms.braw = u;
    }
}
//
// get/set use log file parameter
//
inline int
get_uselogfile() {
    return parms.blogfile;
}
void
set_uselogfile(int u) {
    if (u == FALSE || u == TRUE) {
        parms.blogfile = u;
    }
}
char *
get_uselogfileparm() {
    return (char *)l_parm;
}
char *
get_nologfileparm() {
    return (char *)nolog_parm;
}
//
// get/set nat local parameter
//
int
get_natlocal() {
    return parms.bnatlocal;
}
void
set_natlocal(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bnatlocal = u;
    }
}
//
// get/set use icmp parameter
//
inline int
get_useicmp() {
    return parms.bicmp;
}
void
set_useicmp(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bicmp = u;
    }
}
//
// get/set mode parameter
//
void
set_mode(int m) {
    if (m == INVIZ_MODE_LISTENER ||
            m == INVIZ_MODE_SENDER ||
            m == INVIZ_MODE_SENDER_LISTENING) {
        parms.imode = m;
    }
}
int
get_mode() {
    return parms.imode;
}
//
// get/set log file name
//
void
set_logfilename(char * fw, int len) {
    if (parms.logfilename == NULL) {
        parms.logfilename = malloc(MAX_FILENAME_LEN+1);
    }
    if (parms.logfilename == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.logfilename", 0);
    } else {
        if ( len <= 0 ) {
            *parms.logfilename = '\0';
        } else if ( (len >= 0) && (strlen(fw) > 0) && (len <= (int)MAX_FILENAME_LEN) ) {
            strncpy(parms.logfilename, fw, len);
            parms.logfilename[len] = '\0';
        }
    }
}
char *
get_logfilename() {
    return parms.logfilename;
}
//
// get/set port knocking firewall exe name
//
void
set_firewallexe(char * fw, int len) {
    if (parms.firewallexe == NULL) {
        parms.firewallexe = malloc(MAX_FILENAME_LEN+1);
    }
    if (parms.firewallexe == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.firewallexe", 0);
    } else {
        if ( len <= 0 ) {
            *parms.firewallexe = '\0';
        } else if ( (len >= 0) && (strlen(fw) > 0) && (len <= (int)MAX_FILENAME_LEN) ) {
            strncpy(parms.firewallexe, fw, len);
            parms.firewallexe[len] = '\0';
        }
    }
}
char *
get_firewallexe() {
    return parms.firewallexe;
}
//
// get/set port knocking filename for saving pk packets (client only)
//
void
set_pksavefilename(char * fn, int len) {
    if (parms.pksavefilename == NULL) {
        parms.pksavefilename = malloc(MAX_FILENAME_LEN+1);
    }
    if (parms.pksavefilename == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pksavefilename", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pksavefilename = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_FILENAME_LEN) ) {
            strncpy(parms.pksavefilename, fn, len);
            parms.pksavefilename[len] = '\0';
        }
    }
}
char *
get_pksavefilename() {
    return parms.pksavefilename;
}
//
// get/set port knocking access filename (server only)
//
void
set_pkaccessfilename(char * fn, int len) {
    if (parms.pkaccessfilename == NULL) {
        parms.pkaccessfilename = malloc(MAX_FILENAME_LEN+1);
    }
    if (parms.pkaccessfilename == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkaccessfilename", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkaccessfilename = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_FILENAME_LEN) ) {
            strncpy(parms.pkaccessfilename, fn, len);
            parms.pkaccessfilename[len] = '\0';
        }
    }
}
char *
get_pkaccessfilename() {
    return parms.pkaccessfilename;
}
//
// get/set port knocking config filename (server only)
//
void
set_pkconfigfilename(char * fn, int len) {
    if (parms.pkconfigfilename == NULL) {
        parms.pkconfigfilename = malloc(MAX_FILENAME_LEN+1);
    }
    if (parms.pkconfigfilename == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkconfigfilename", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkconfigfilename = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_FILENAME_LEN) ) {
            strncpy(parms.pkconfigfilename, fn, len);
            parms.pkconfigfilename[len] = '\0';
        }
    }
}
char *
get_pkconfigfilename() {
    return parms.pkconfigfilename;
}
//
// get/set pcap device
//
void
set_pcapdevice(char * fn, int len) {
    if (parms.pcap_device == NULL) {
        parms.pcap_device = malloc(INVIZ_MAX_PCAP_DEVICE_LEN+1);
    }
    if (parms.pcap_device == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pcap_device", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pcap_device = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PCAP_DEVICE_LEN) ) {
            strncpy(parms.pcap_device, fn, len);
            parms.pcap_device[len] = '\0';
        }
    }
}
char *
get_pcapdevice() {
    return parms.pcap_device;
}
//
// get/set access stanza struct
//
void
set_accstanzastruct(acc_stanza_t * st) {
    if (parms.acc_stanza_struct == NULL) {
        parms.acc_stanza_struct = calloc(1, sizeof(acc_stanza_t));
    }
    if (parms.acc_stanza_struct == NULL) {
        logit(_LOG_ERROR, "unable to calloc parms.acc_stanza_struct", 0);
    } else {
        parms.acc_stanza_struct->source_list = calloc(1, sizeof(acc_int_list_t));
        parms.acc_stanza_struct->oport_list = calloc(1, sizeof(acc_port_list_t));
        parms.acc_stanza_struct->rport_list = calloc(1, sizeof(acc_port_list_t));
        parms.acc_stanza_struct->gpg_remote_id_list = calloc(1, sizeof(acc_string_list_t));
        if ((parms.acc_stanza_struct->source_list == NULL) ||
            (parms.acc_stanza_struct->oport_list == NULL) ||
            (parms.acc_stanza_struct->rport_list == NULL) ||
            (parms.acc_stanza_struct->gpg_remote_id_list == NULL)) {
            logit(_LOG_ERROR, "unable to calloc parms.acc_stanza_struct", 0);
        } else {
            if (st) {
                parms.acc_stanza_struct = st;
            }
        }
    }
}
acc_stanza_t *
get_accstanzastruct() {
    return parms.acc_stanza_struct;
}
//
// get/set firewall chain
//
void
set_firewallchain(struct fw_chain *pchain) {
    if (parms.firewall_chain == NULL) {
        parms.firewall_chain = calloc(1, sizeof(struct fw_chain)+1);
    }
    if (parms.firewall_chain == NULL) {
        logit(_LOG_ERROR, "unable to calloc parms.firewall_chain", 0);
    } else {
        //memset(parms.firewall_chain, 0, sizeof(struct fw_chain)+1);
        memcpy(parms.firewall_chain, (struct fw_chain *)pchain, sizeof(struct fw_chain));
        //parms.firewall_chain = pchain;
    }
}
struct fw_chain *
get_firewallchain() {
    return parms.firewall_chain;
}
//
// get/set firewall config
//
void
set_firewallconfig(struct fw_config *pconfig) {
    if (parms.firewall_config == NULL) {
        parms.firewall_config = calloc(1, sizeof(struct fw_config)+1);
    }
    if (parms.firewall_config == NULL) {
        logit(_LOG_ERROR, "unable to calloc parms.firewall_config", 0);
    } else {
        //memset(parms.firewall_config, 0, sizeof(struct fw_config)+1);
        memcpy(parms.firewall_config, (struct fw_config *)pconfig, sizeof(struct fw_config));
        //parms.firewall_config = pconfig;
    }
}
struct fw_config *
get_firewallconfig() {
    return parms.firewall_config;
}
//
// get/set pcap struct
//
void
set_pcapstruct(pcap_t *st) {
    if (parms.pcap_struct == NULL) {
        parms.pcap_struct = calloc(1, sizeof(pcap_t)+1);
    }
    if (parms.pcap_struct == NULL) {
        logit(_LOG_ERROR, "unable to calloc parms.pcap_struct", 0);
    } else {
        //memset(parms.pcap_struct, 0, sizeof(pcap_t)+1);
        //memcpy(parms.pcap_struct, (pcap_t *)st, sizeof(pcap_t));
        parms.pcap_struct = st;
    }
}
pcap_t *
get_pcapstruct() {
    return parms.pcap_struct;
}
//
// get/set spa packet struct
//
void
set_spapacketstruct(spa_pkt_info_t *st) {
    if (parms.spa_packet_struct == NULL) {
        parms.spa_packet_struct = calloc(1, sizeof(spa_pkt_info_t)+1);
    }
    if (parms.spa_packet_struct == NULL) {
        logit(_LOG_ERROR, "unable to calloc parms.spa_packet_struct", 0);
    } else {
        //memset(parms.spa_packet_struct, 0, sizeof(spa_pkt_info_t)+1);
        memcpy(parms.spa_packet_struct, (spa_pkt_info_t *)st, sizeof(spa_pkt_info_t));
        //parms.spa_packet_struct = st;
    }
}
spa_pkt_info_t *
get_spapacketstruct() {
    return parms.spa_packet_struct;
}
//
// get/set spa packet data
//
void
set_spapacketdata(char * pdata, unsigned int plen) {
    if (get_spapacketstruct() == NULL) {
        spa_pkt_info_t * sp;
        set_spapacketstruct((spa_pkt_info_t *)sp);
    }
    if (get_spapacketstruct() == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.spa_packet_struct", 0);
    } else {
        memset(parms.spa_packet_struct->packet_data, 0, sizeof(parms.spa_packet_struct->packet_data));
        if ( (plen >= 0) && (strlen(pdata) > 0) && (plen <= MAX_SPA_PACKET_LEN) ) {
            strncpy((char *)parms.spa_packet_struct->packet_data, (char *)pdata, plen);
            parms.spa_packet_struct->packet_data_len = plen;
        }
    }
}
char *
get_spapacketdata() {
    return (char *)parms.spa_packet_struct->packet_data;
}
//
// get spa packet data len
// set spa packet data len is done inside set_spapacketdata()
//
unsigned int
get_spapacketdatalen() {
    return parms.spa_packet_struct->packet_data_len;
}
//
// get/set spa packet source ip address (ip4? ip6?)
//
void
set_spapacketsourceip(unsigned int sip) {
    if (get_spapacketstruct() == NULL) {
        spa_pkt_info_t * sp;
        memset((spa_pkt_info_t *)sp, 0, sizeof(spa_pkt_info_t));
        set_spapacketstruct((spa_pkt_info_t *)sp);
    }
    if (get_spapacketstruct() == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.spa_packet_struct", 0);
    } else {
        parms.spa_packet_struct->packet_src_ip = sip;
    }
}
unsigned int
get_spapacketsourceip() {
    return parms.spa_packet_struct->packet_src_ip;
}
//
// get/set spa packet destination port
//
void
set_spapacketdestinationport(unsigned short dport) {
    struct spa_pkt_info * sp;
    if (get_spapacketstruct() == NULL) {
        memset(sp, 0, sizeof(struct spa_pkt_info));
        set_spapacketstruct((struct spa_pkt_info *)sp);
    }
    if (get_spapacketstruct() == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.spa_packet_struct", 0);
    } else {
        parms.spa_packet_struct->packet_dest_port = dport;
    }
}
unsigned short
get_spapacketdestinationport() {
    return parms.spa_packet_struct->packet_dest_port;
}
//
//
// get/set filename to be sent (if client) or received (if server)
//
void
set_filename(char * fn, int len) {
    if ( len <= 0 ) {
        *parms.filename = '\0';
    } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_FILENAME_LEN) ) {
        strncpy(parms.filename, fn, len);
        parms.filename[len] = '\0';
    }
}
char *
get_filename() {
    return parms.filename;
}
//
// get/set firewall access/chain string
//
char *
get_firewallaccess(int ndx) {
    char logmsg[512];
    if (ndx < 0 || ndx > NUM_IPT_ACCESS) {
        sprintf(logmsg, "firewall access index (%i) must be between 0 and %i", ndx, NUM_IPT_ACCESS);
        logit(_LOG_ERROR, logmsg, 0);
        return NULL;
    }
    return parms.firewall_access[ndx];
}
void
set_firewallaccess(int ndx, char * str, int len) {
    char logmsg[512];
    if (ndx < 0 || ndx > NUM_IPT_ACCESS) {
        sprintf(logmsg, "firewall access index (%i) must be between 0 and %i", ndx, NUM_IPT_ACCESS);
        logit(_LOG_ERROR, logmsg, 0);
        return;
    }
    if (parms.firewall_access[ndx] == NULL) {
        parms.firewall_access[ndx] = malloc(len+1);
    }
    if (parms.firewall_access[ndx] == NULL) {
        sprintf(logmsg, "unable to malloc parms.firewall_access[%i]", ndx);
        logit(_LOG_ERROR, logmsg, 0);
        return;
    }
    memset(parms.firewall_access[ndx], 0, len+1);
    strncpy(parms.firewall_access[ndx], str, len);
}
//
// get/set firewall snat translate ip
//s
void
set_fwsnatip(char * fn, int len) {
    int maxiplen = MAX_IP4_LEN;
    if (MAX_IP6_LEN > maxiplen) {
        maxiplen = MAX_IP6_LEN;
    }
    if (parms.snat_translate_ip == NULL) {
        parms.snat_translate_ip = malloc(maxiplen+1);
    }
    if (parms.snat_translate_ip == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.snat_translate_ip", 0);
    } else {
        if ( len <= 0 ) {
            *parms.snat_translate_ip = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)maxiplen) ) {
            strncpy(parms.snat_translate_ip, fn, len);
            parms.snat_translate_ip[len] = '\0';
        }
    }
}
char *
get_fwsnatip() {
    return parms.snat_translate_ip;
}
//
// get/set destination ip
//
void
set_destinationip(char * fn, int len) {
    int maxiplen = MAX_IP4_LEN;
    if (MAX_IP6_LEN > maxiplen) {
        maxiplen = MAX_IP6_LEN;
    }
    if (parms.destination_ip == NULL) {
        parms.destination_ip = malloc(maxiplen+1);
    }
    if (parms.destination_ip == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.destination_ip", 0);
    } else {
        if ( len <= 0 ) {
            *parms.destination_ip = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)maxiplen) ) {
            strncpy(parms.destination_ip, fn, len);
            parms.destination_ip[len] = '\0';
        }
    }
}
char *
get_destinationip() {
    return parms.destination_ip;
}
//
// get/set source ip
//
void
set_sourceip(char * fn, int len) {
    int maxiplen = MAX_IP4_LEN;
    if (MAX_IP6_LEN > maxiplen) {
        maxiplen = MAX_IP6_LEN;
    }
    if (parms.source_ip == NULL) {
        parms.source_ip = malloc(maxiplen+1);
    }
    if (parms.source_ip == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.source_ip", 0);
    } else {
        if ( len <= 0 ) {
            *parms.source_ip = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)maxiplen) ) {
            strncpy(parms.source_ip, fn, len);
            parms.source_ip[len] = '\0';
        }
    }
}
char *
get_sourceip() {
    return parms.source_ip;
}
//
// get/set gpg recipient key
//
void
set_gpg_recipientkey(char * fn, int len) {
    if (parms.gpg_recipientkey == NULL) {
        parms.gpg_recipientkey = malloc(INVIZ_MAX_GPGKEY_LEN+1);
    }
    if (parms.gpg_recipientkey == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.gpg_recipientkey", 0);
    } else {
        if ( len <= 0 ) {
            *parms.gpg_recipientkey = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_GPGKEY_LEN) ) {
            strncpy(parms.gpg_recipientkey, fn, len);
            parms.gpg_recipientkey[len] = '\0';
        }
    }
}
char *
get_gpg_recipientkey() {
    return parms.gpg_recipientkey;
}
//
// get/set gpg signer key
//
void
set_gpg_signerkey(char * fn, int len) {
    if (parms.gpg_signerkey == NULL) {
        parms.gpg_signerkey = malloc(INVIZ_MAX_GPGKEY_LEN+1);
    }
    if (parms.gpg_signerkey == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.gpg_signerkey", 0);
    } else {
        if ( len <= 0 ) {
            *parms.gpg_signerkey = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_GPGKEY_LEN) ) {
            strncpy(parms.gpg_signerkey, fn, len);
            parms.gpg_signerkey[len] = '\0';
        }
    }
}
char *
get_gpg_signerkey() {
    return parms.gpg_signerkey;
}
//
// get/set gpg home directory
//
void
set_gpg_homedir(char * fn, int len) {
    if (parms.gpg_homedir == NULL) {
        parms.gpg_homedir = malloc(INVIZ_MAX_PATH_LEN+1);
    }
    if (parms.gpg_homedir == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.gpg_homedir", 0);
    } else {
        if ( len <= 0 ) {
            *parms.gpg_homedir = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PATH_LEN) ) {
            strncpy(parms.gpg_homedir, fn, len);
            parms.gpg_homedir[len] = '\0';
        }
    }
}
char *
get_gpg_homedir() {
    return parms.gpg_homedir;
}
//
// get/set gpg key file
//
void
set_gpg_keyfile(char * fn, int len) {
    if (parms.gpg_keyfile == NULL) {
        parms.gpg_keyfile = malloc(INVIZ_MAX_PATH_LEN+1);
    }
    if (parms.gpg_keyfile == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.gpg_keyfile", 0);
    } else {
        if ( len <= 0 ) {
            *parms.gpg_keyfile = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PATH_LEN) ) {
            strncpy(parms.gpg_keyfile, fn, len);
            parms.gpg_keyfile[len] = '\0';
        }
    }
}
char *
get_gpg_keyfile() {
    return parms.gpg_keyfile;
}
//
// get/set key seed to be used by both client and server
//
void
set_keyseed(char * fn, int len) {
    if ( len <= 0 ) {
        *parms.keyseed = '\0';
    } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_KEYSEED_LEN) ) {
        strncpy(parms.keyseed, fn, len);
        parms.keyseed[len] = '\0';
    }
}
char *
get_keyseed() {
    return parms.keyseed;
}
//
// get/set port knocking wipe target
//
void
set_wipetarget(char * fn, int len) {
    if (parms.wipetarget == NULL) {
        parms.wipetarget = malloc(MAX_FILENAME_LEN+1);
    }
    if (parms.wipetarget == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.wipetarget", 0);
    } else {
        if ( len <= 0 ) {
            *parms.wipetarget = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_FILENAME_LEN) ) {
            strncpy(parms.wipetarget, fn, len);
            parms.wipetarget[len] = '\0';
        }
    }
}
char *
get_wipetarget() {
    return parms.wipetarget;
}
//
// get/set port knocking inotify ftarget
//
void
set_inotifytarget(char * fn, int len) {
    if (parms.inotifytarget == NULL) {
        parms.inotifytarget = malloc(MAX_FILENAME_LEN+1);
    }
    if (parms.inotifytarget == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.inotifytarget", 0);
    } else {
        if ( len <= 0 ) {
            *parms.inotifytarget = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_FILENAME_LEN) ) {
            strncpy(parms.inotifytarget, fn, len);
            parms.inotifytarget[len] = '\0';
        }
    }
}
char *
get_inotifytarget() {
    return parms.inotifytarget;
}
//
// get/set port knocking beacon target
//
void
set_beacontarget(char * fn, int len) {
    if (parms.beacontarget == NULL) {
        parms.beacontarget = malloc(MAX_FILENAME_LEN+1);
    }
    if (parms.beacontarget == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkbeacontarget", 0);
    } else {
        if ( len <= 0 ) {
            *parms.beacontarget = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_FILENAME_LEN) ) {
            strncpy(parms.beacontarget, fn, len);
            parms.beacontarget[len] = '\0';
        }
    }
}
char *
get_beacontarget() {
    return parms.beacontarget;
}
//
// get/set port knocking transfer target
//
void
set_transfertarget(char * fn, int len) {
    if (parms.transfertarget == NULL) {
        parms.transfertarget = malloc(MAX_FILENAME_LEN+1);
    }
    if (parms.transfertarget == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.transfertarget", 0);
    } else {
        if ( len <= 0 ) {
            *parms.transfertarget = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_FILENAME_LEN) ) {
            strncpy(parms.transfertarget, fn, len);
            parms.transfertarget[len] = '\0';
        }
    }
}
char *
get_transfertarget() {
    return parms.transfertarget;
}
//
// get/set port knocking context
//
inline fko_ctx_t
get_pkcontext() {
    return parms.fko_ctx;
}
int
set_pkcontext() {
    char logmsg[512];
    int rc = fko_new(&parms.fko_ctx);
    if ( rc != FKO_SUCCESS ) {
        sprintf(logmsg, "unable to set port knocking context [%s]", fko_errstr(rc));
        logit(_LOG_ERROR, logmsg, 0);
        return (EXIT_FAILURE);
    }
    return (EXIT_SUCCESS);
}
//
// get/set port knocking client IP address
//
void
set_pkclient(char * fn, int len) {
    if (parms.pkclient == NULL) {
        parms.pkclient = malloc(MAX_IP6_LEN+1);
    }
    if (parms.pkclient == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkclient", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkclient = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_IP6_LEN) ) {
            strncpy(parms.pkclient, fn, len);
            parms.pkclient[len] = '\0';
        }
    }
}
char *
get_pkclient() {
    return parms.pkclient;
}
//
// get/set port knocking beacon options
//
void
set_beaconoptions(char * fn, int len) {
    if (parms.beaconoptions == NULL) {
        parms.beaconoptions = malloc(INVIZ_MAX_PKCOMMAND_LEN+1);
    }
    if (parms.beaconoptions == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.beaconoptions", 0);
    } else {
        if ( len <= 0 ) {
            *parms.beaconoptions = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKCOMMAND_LEN) ) {
            strncpy(parms.beaconoptions, fn, len);
            parms.beaconoptions[len] = '\0';
        }
    }
}
char *
get_beaconoptions() {
    return parms.beaconoptions;
}
//
// get/set port knocking inotify options
//
void
set_inotifyoptions(char * fn, int len) {
    if (parms.inotifyoptions == NULL) {
        parms.inotifyoptions = malloc(INVIZ_MAX_PKCOMMAND_LEN+1);
    }
    if (parms.inotifyoptions == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.inotifyoptions", 0);
    } else {
        if ( len <= 0 ) {
            *parms.inotifyoptions = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKCOMMAND_LEN) ) {
            strncpy(parms.inotifyoptions, fn, len);
            parms.inotifyoptions[len] = '\0';
        }
    }
}
char *
get_inotifyoptions() {
    return parms.inotifyoptions;
}
int
get_inotifyfork() {
    if (!get_inotifyoptions() || strlen(get_inotifyoptions()) < 1) {
        return (INVIZ_DEFAULT_PKINOTIFY_FORK);
    }
    char *match = strstr((char *)get_inotifyoptions(), INVIZ_FORK_DESC);
    if (match == NULL) {
        return (FALSE);
    } else {
        return (TRUE);
    }
}
int
get_inotifyecho() {
    if (!get_inotifyoptions() || strlen(get_inotifyoptions()) < 1) {
        return (INVIZ_DEFAULT_PKINOTIFY_ECHO);
    }
    char *match = strstr((char *)get_inotifyoptions(), INVIZ_ECHO_DESC);
    if (match == NULL) {
        return (FALSE);
    } else {
        return (TRUE);
    }
}
int
get_inotifynonblocking() {
    if (!get_inotifyoptions() || strlen(get_inotifyoptions()) < 1) {
        return (INVIZ_DEFAULT_PKINOTIFY_NONBLOCKING);
    }
    char *match = strstr((char *)get_inotifyoptions(), INVIZ_NONBLOCKING_DESC);
    if (match == NULL) {
        return (FALSE);
    } else {
        return (TRUE);
    }
}
int
get_inotifyonce() {
    if (!get_inotifyoptions() || strlen(get_inotifyoptions()) < 1) {
        return (INVIZ_DEFAULT_PKINOTIFY_ONCE);
    }
    char *match = strstr((char *)get_inotifyoptions(), INVIZ_ONCE_DESC);
    if (match == NULL) {
        return (FALSE);
    } else {
        return (TRUE);
    }
}
int
get_inotifycount() {
    //
    // number of inotify events before stopping
    //
    if (!get_inotifyoptions() || strlen(get_inotifyoptions()) < 1) {
        return (INVIZ_DEFAULT_PKINOTIFY_MAX_COUNT);
    }
    char str[16];
    memset(str, 0x0, sizeof(str));
    char *match = strstr((char *)get_inotifyoptions(), INVIZ_COUNT_DESC"=");
    if (match == NULL) {
        return (INVIZ_DEFAULT_PKINOTIFY_MAX_COUNT);
    } else {
        match += (strlen(INVIZ_COUNT_DESC)+1);
        int namelen = strcspn(match, INVIZ_SEPARATORS);
        if (namelen == 0) {
            sprintf(str, "%s", match);
        } else {
            strncpy(str, match, namelen);
        }
        return (atoi(str));
    }
}
int
get_inotifyexpiry() {
    //
    // number of elapsed seconds of inotify events before stopping
    //
    if (!get_inotifyoptions() || strlen(get_inotifyoptions()) < 1) {
        return (INVIZ_DEFAULT_PKINOTIFY_MAX_TIME);
    }
    char str[16];
    memset(str, 0x0, sizeof(str));
    char *match = strstr((char *)get_inotifyoptions(), INVIZ_EXPIRY_DESC"=");
    if (match == NULL) {
        return (INVIZ_DEFAULT_PKINOTIFY_MAX_TIME);
    } else {
        match += (strlen(INVIZ_EXPIRY_DESC)+1);
        int namelen = strcspn(match, INVIZ_SEPARATORS);
        if (namelen == 0) {
            sprintf(str, "%s", match);
        } else {
            strncpy(str, match, namelen);
        }
        return (atoi(str));
    }
}
//
// get/set port knocking transfer options
//
void
set_transferoptions(char * fn, int len) {
    if (parms.transferoptions == NULL) {
        parms.transferoptions = malloc(INVIZ_MAX_PKCOMMAND_LEN+1);
    }
    if (parms.transferoptions == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.transferoptions", 0);
    } else {
        if ( len <= 0 ) {
            *parms.transferoptions = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKCOMMAND_LEN) ) {
            strncpy(parms.transferoptions, fn, len);
            parms.transferoptions[len] = '\0';
        }
    }
}
char *
get_transferoptions() {
    return parms.transferoptions;
}
int
get_transferecho() {
    if (!get_transferoptions() || strlen(get_transferoptions()) < 1) {
        return (INVIZ_DEFAULT_PKTRANSFER_ECHO);
    }
    char *match = strstr((char *)get_transferoptions(), INVIZ_ECHO_DESC);
    if (match == NULL) {
        return (FALSE);
    } else {
        return (TRUE);
    }
}
int
get_transferwipe() {
    if (!get_transferoptions() || strlen(get_transferoptions()) < 1) {
        return (INVIZ_DEFAULT_PKTRANSFER_WIPE);
    }
    char *match = strstr((char *)get_transferoptions(), INVIZ_WIPE_DESC);
    if (match == NULL) {
        return (FALSE);
    } else {
        return (TRUE);
    }
}
int
get_transferremove() {
    if (!get_transferoptions() || strlen(get_transferoptions()) < 1) {
        return (INVIZ_DEFAULT_PKTRANSFER_REMOVE);
    }
    char *match = strstr((char *)get_transferoptions(), INVIZ_REMOVE_DESC);
    if (match == NULL) {
        return (FALSE);
    } else {
        return (TRUE);
    }
}
int
get_transfererase() {
    if (!get_transferoptions() || strlen(get_transferoptions()) < 1) {
        return (INVIZ_DEFAULT_PKTRANSFER_ERASE);
    }
    char *match = strstr((char *)get_transferoptions(), INVIZ_ERASE_DESC);
    if (match == NULL) {
        return (FALSE);
    } else {
        return (TRUE);
    }
}
//
// get no op parm
//
char *
get_noopparm() {
    return (char *)noop_parm;
}
//
// get/set inotify log filename
//
char *
get_inotifylog() {
    return parms.inotifylog;
}
void
set_inotifylog(int t) {
    if (t == FALSE) {
        //
        // clear/reset
        //
        if (parms.inotifylog) {
            free(parms.inotifylog);
            parms.inotifylog = NULL;
        }
        return;
    }
    if (parms.inotifylog == NULL) {
        parms.inotifylog = malloc(MAX_FILENAME_LEN+1);
    }
    if (parms.inotifylog == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.inotifylog", 0);
        return;
    }
    int len = strlen(INVIZ_DEFAULT_PKINOTIFY_SERVER_LOG);
    if (!get_inotifyoptions() || strlen(get_inotifyoptions()) < 1) {
        strncpy(parms.inotifylog, INVIZ_DEFAULT_PKINOTIFY_SERVER_LOG, len);
        parms.inotifylog[len] = '\0';
        return;
    }
    char *match = strstr((char *)get_inotifyoptions(), INVIZ_LOG_DESC"=");
    if (match == NULL) {
        //
        // no log specified
        //
        strncpy(parms.inotifylog, INVIZ_DEFAULT_PKINOTIFY_SERVER_LOG, len);
        parms.inotifylog[len] = '\0';
        return;
    }
    match += (strlen(INVIZ_LOG_DESC)+1);
    int namelen = strcspn(match, INVIZ_SEPARATORS);
    if (namelen == 0) {
        memset(parms.inotifylog, 0x0, MAX_FILENAME_LEN+1);
        strncpy(parms.inotifylog, match, MAX_FILENAME_LEN);
    } else {
        strncpy(parms.inotifylog, match, namelen);
        parms.inotifylog[namelen] = '\0';
    }
}
//
// get/set inotify DNS bounce server IP address
// for echoing/transferring log file back to client
//
char *
get_inotifyechodns() {
    return parms.inotifyechodns;
}
void
set_inotifyechodns(int t) {
    if (t == FALSE) {
        //
        // clear/reset
        //
        if (parms.inotifyechodns) {
            free(parms.inotifyechodns);
            parms.inotifyechodns = NULL;
        }
        return;
    }
    if (parms.inotifyechodns == NULL) {
        parms.inotifyechodns = malloc(MAX_IP4_LEN+1);
    }
    if (parms.inotifyechodns == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.inotifyechodns", 0);
        return;
    }
    int len = strlen(INVIZ_DEFAULT_DNS_IP4);
    if (!get_inotifyoptions() || strlen(get_inotifyoptions()) < 1) {
        strncpy(parms.inotifyechodns, INVIZ_DEFAULT_DNS_IP4, len);
        parms.inotifyechodns[len] = '\0';
        return;
    }
    char *match = strstr((char *)get_inotifyoptions(), INVIZ_DNS_DESC"=");
    if (match == NULL) {
        //
        // no echo dns specified
        //
        strncpy(parms.inotifyechodns, INVIZ_DEFAULT_DNS_IP4, len);
        parms.inotifyechodns[len] = '\0';
        return;
    }
    match += (strlen(INVIZ_DNS_DESC)+1);
    int namelen = strcspn(match, INVIZ_SEPARATORS);
    if (namelen == 0) {
        memset(parms.inotifyechodns, 0x0, MAX_IP4_LEN+1);
        strncpy(parms.inotifyechodns, match, MAX_IP4_LEN);
    } else {
        strncpy(parms.inotifyechodns, match, namelen);
        parms.inotifyechodns[namelen] = '\0';
    }
}
//
// get/set first/previous transaction ID
//
inline int
get_firsttid() {
    return parms.ifirsttid;
}
inline int
get_prevtid() {
    return parms.iprevtid;
}
void
set_firsttid(int tid) {
    parms.ifirsttid = tid;
}
void
set_prevtid(int tid) {
    parms.iprevtid = tid;
    int prev;
    int addtid = TRUE;
    int prevcount = 0;
    for (prev=0; prev<INVIZ_PREVIOUS_TRANS_IDS; prev++) {
        if (parms.iprevtids[prev] == tid) {
            // already in previous list
            addtid = FALSE;
            break;
        } else if (parms.iprevtids[prev] > 0) {
            prevcount++;
        }
    }
    if (addtid) {
        if (prevcount < INVIZ_PREVIOUS_TRANS_IDS) {
            // put in 1st vacant slot
            for (prev=0; prev<INVIZ_PREVIOUS_TRANS_IDS; prev++) {
                if (parms.iprevtids[prev] == 0) {
                    parms.iprevtids[prev] = tid;
                    break;
                }
            }
        } else {
            // replace oldest vacant slot
            for (prev=0; prev<(INVIZ_PREVIOUS_TRANS_IDS-1); prev++) {
                parms.iprevtids[prev] = parms.iprevtids[prev+1];
            }
            parms.iprevtids[INVIZ_PREVIOUS_TRANS_IDS-1] = tid;
        }
    }
}
int
is_prevtid(int tid) {
    int iresult = FALSE;
    int prev;
    for (prev=0; prev<INVIZ_PREVIOUS_TRANS_IDS; prev++) {
        if (parms.iprevtids[prev] == tid) {
            iresult = TRUE;
            break;
        }
    }
    return (iresult);
}
//
// get/set trans id seed constant
//
inline int
get_iseedconstant() {
    return parms.iseedconstant;
}
void
set_iseedconstant(int s) {
    if ( (s >= 0) && (s <= INVIZ_MAX_SEED_CONSTANT) ) {
        parms.iseedconstant = s;
    }
}
//
// get/set trans id seed overflow
//
inline int
get_iseedoverflow() {
    return parms.iseedoverflow;
}
void
set_iseedoverflow(int s) {
    if (s >= 0) {
        parms.iseedoverflow = s;
    }
}
//
// get/set inotify key/seed
// for use in echoing/transferring log file back to client
//
char *
get_inotifyechokey() {
    return parms.inotifyechokey;
}
void
set_inotifyechokey(int t) {
    if (t == FALSE) {
        //
        // clear/reset
        //
        if (parms.inotifyechokey) {
            free(parms.inotifyechokey);
            parms.inotifyechokey = NULL;
        }
        return;
    }
    if (parms.inotifyechokey == NULL) {
        parms.inotifyechokey = malloc(INVIZ_MAX_KEYSEED_LEN+1);
    }
    if (parms.inotifyechokey == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.inotifyechokey", 0);
        return;
    }
    int len = strlen(INVIZ_DEFAULT_KEYSEED);
    if (!get_inotifyoptions() || strlen(get_inotifyoptions()) < 1) {
        strncpy(parms.inotifyechokey, INVIZ_DEFAULT_KEYSEED, len);
        parms.inotifyechokey[len] = '\0';
        return;
    }
    char *match = strstr((char *)get_inotifyoptions(), INVIZ_KEY_DESC"=");
    if (match == NULL) {
        //
        // no echo key specified
        //
        strncpy(parms.inotifyechokey, INVIZ_DEFAULT_KEYSEED, len);
        parms.inotifyechokey[len] = '\0';
        return;
    }
    match += (strlen(INVIZ_KEY_DESC)+1);
    int namelen = strcspn(match, INVIZ_SEPARATORS);
    if (namelen == 0) {
        memset(parms.inotifyechokey, 0x0, INVIZ_MAX_KEYSEED_LEN+1);
        strncpy(parms.inotifyechokey, match, INVIZ_MAX_KEYSEED_LEN);
    } else {
        strncpy(parms.inotifyechokey, match, namelen);
        parms.inotifyechokey[namelen] = '\0';
    }
}
//
// get/set inotify seed (numeric)
// for use in echoing/transferring log file back to client
//
int
get_inotifyechoseed() {
    return parms.inotifyechoseed;
}
void
set_inotifyechoseed(int t) {
    if (t == FALSE) {
        //
        // clear/reset
        //
        parms.inotifyechoseed = INVIZ_DEFAULT_SEED;
        return;
    }
    if (!get_inotifyoptions() || strlen(get_inotifyoptions()) < 1) {
        parms.inotifyechoseed = INVIZ_DEFAULT_SEED;
        return;
    }
    char *match = strstr((char *)get_inotifyoptions(), INVIZ_SEED_DESC"=");
    if (match == NULL) {
        //
        // no echo seed specified
        //
        parms.inotifyechoseed = INVIZ_DEFAULT_SEED;
        return;
    }
    match += (strlen(INVIZ_SEED_DESC)+1);
    int valuelen = strcspn(match, INVIZ_SEPARATORS);
    if (valuelen > INVIZ_MAX_SEED_LEN) {
        //
        // invalid echo seed specified
        //
        logit(_LOG_WARNING, "invalid inotify "INVIZ_SEED_DESC" value specified", 0);
        parms.inotifyechoseed = INVIZ_DEFAULT_SEED;
        return;
    }
    char seedvalue[INVIZ_MAX_SEED_LEN+1];
    memset(seedvalue, 0x0, sizeof(seedvalue));
    if (valuelen == 0) {
        strncpy(seedvalue, match, INVIZ_MAX_SEED_LEN);
    } else {
        strncpy(seedvalue, match, valuelen);
        seedvalue[valuelen] = '\0';
    }
    int iseedvalue = atoi(seedvalue);
    if ( (iseedvalue < get_miniseed()) || (iseedvalue > get_maxiseed()) ) {
        logit(_LOG_WARNING, "invalid inotify "INVIZ_SEED_DESC" value specified", 0);
        parms.inotifyechoseed = INVIZ_DEFAULT_SEED;
    } else {
        parms.inotifyechoseed = iseedvalue;
    }
}
//
// get/set port knocking inotify event count
//
int
get_inotifyevents() {
    return parms.inotifyevents;
}
void
set_inotifyevents(int i) {
    parms.inotifyevents = i;
}
void
inc_inotifyevents() {
    parms.inotifyevents++;
}
//
// get/set port knocking inotify watchdog timer
//
double
get_inotifywatchdog() {
    return parms.inotifywatchdog;
}
void
set_inotifywatchdog(int doit) {
    struct timeval tprun;
    if (doit == FALSE) {
        parms.inotifywatchdog = 0.0;
    } else {
        //
        // set to current time
        //
        gettimeofday(&tprun, NULL);
        parms.inotifywatchdog = tprun.tv_sec+(tprun.tv_usec/1000000.0);
    }
}
//
// get/set port knocking inotify command (to be executed on server)
//
void
set_pkinotifycommand(char * fn, int len) {
    if (parms.pkinotifycommand == NULL) {
        parms.pkinotifycommand = malloc(INVIZ_MAX_PKCOMMAND_LEN+1);
    }
    if (parms.pkinotifycommand == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkinotifycommand", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkinotifycommand = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKCOMMAND_LEN) ) {
            strncpy(parms.pkinotifycommand, fn, len);
            parms.pkinotifycommand[len] = '\0';
        }
    }
}
char *
get_pkinotifycommand() {
    return parms.pkinotifycommand;
}
char *
get_pkinotifyparm() {
    return (char *)pkinotifycommand_parm;
}
//
// get/set port knocking beacon command (to be executed on server)
//
void
set_pkbeaconcommand(char * fn, int len) {
    if (parms.pkbeaconcommand == NULL) {
        parms.pkbeaconcommand = malloc(INVIZ_MAX_PKCOMMAND_LEN+1);
    }
    if (parms.pkbeaconcommand == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkbeaconcommand", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkbeaconcommand = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKCOMMAND_LEN) ) {
            strncpy(parms.pkbeaconcommand, fn, len);
            parms.pkbeaconcommand[len] = '\0';
        }
    }
}
char *
get_pkbeaconcommand() {
    return parms.pkbeaconcommand;
}
char *
get_pkbeaconparm() {
    return (char *)pkbeaconcommand_parm;
}
//
// get/set port knocking wipe command (to be executed on server)
//
void
set_pkwipecommand(char * fn, int len) {
    if (parms.pkwipecommand == NULL) {
        parms.pkwipecommand = malloc(INVIZ_MAX_PKCOMMAND_LEN+1);
    }
    if (parms.pkwipecommand == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkwipecommand", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkwipecommand = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKCOMMAND_LEN) ) {
            strncpy(parms.pkwipecommand, fn, len);
            parms.pkwipecommand[len] = '\0';
        }
    }
}
char *
get_pkwipecommand() {
    return parms.pkwipecommand;
}
char *
get_pkwipeparm() {
    return (char *)pkwipecommand_parm;
}
//
// get/set port knocking password command
//
void
set_pkpasscommand(char * fn, int len) {
    if (parms.pkpasscommand == NULL) {
        parms.pkpasscommand = malloc(INVIZ_MAX_PKCOMMAND_LEN+1);
    }
    if (parms.pkpasscommand == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkpasscommand", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkpasscommand = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKCOMMAND_LEN) ) {
            strncpy(parms.pkpasscommand, fn, len);
            parms.pkpasscommand[len] = '\0';
        }
    }
}
char *
get_pkpasscommand() {
    return parms.pkpasscommand;
}
char *
get_pkpasscommandparm() {
    return (char *)pkpasscommand_parm;
}
//
// get/set port knocking key logging command (to be executed on server)
//
void
set_pkkeylogcommand(char * fn, int len) {
    if (parms.pkkeylogcommand == NULL) {
        parms.pkkeylogcommand = malloc(INVIZ_MAX_PKCOMMAND_LEN+1);
    }
    if (parms.pkkeylogcommand == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkkeylogcommand", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkkeylogcommand = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKCOMMAND_LEN) ) {
            strncpy(parms.pkkeylogcommand, fn, len);
            parms.pkkeylogcommand[len] = '\0';
        }
    }
}
char *
get_pkkeylogcommand() {
    return parms.pkkeylogcommand;
}
char *
get_pkkeylogparm() {
    return (char *)pkkeylogcommand_parm;
}
//
// get/set port knocking filename (to be transferred to client)
//
void
set_pktransfercommand(char * fn, int len) {
    if (parms.pktransfercommand == NULL) {
        parms.pktransfercommand = malloc(MAX_FILENAME_LEN+1);
    }
    if (parms.pktransfercommand == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pktransfercommand", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pktransfercommand = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_FILENAME_LEN) ) {
            strncpy(parms.pktransfercommand, fn, len);
            parms.pktransfercommand[len] = '\0';
        }
    }
}
char *
get_pktransfercommand() {
    return parms.pktransfercommand;
}
char *
get_pktransferparm() {
    return (char *)pktransfercommand_parm;
}
//
// get/set port knocking server command (to be executed)
//
void
set_pkservercommand(char * fn, int len) {
    if (parms.pkservercommand == NULL) {
        parms.pkservercommand = malloc(INVIZ_MAX_PKCOMMAND_LEN+1);
    }
    if (parms.pkservercommand == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkservercommand", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkservercommand = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKCOMMAND_LEN) ) {
            strncpy(parms.pkservercommand, fn, len);
            parms.pkservercommand[len] = '\0';
        }
    }
}
char *
get_pkservercommand() {
    return parms.pkservercommand;
}
//
// get/set port knocking password (server and client)
//
void
set_pkpassword(char * fn, int len) {
    if (parms.pkpassword == NULL) {
        parms.pkpassword = malloc(INVIZ_MAX_PASSWORD_LEN+1);
    }
    if (parms.pkpassword == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkpassword", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkpassword = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PASSWORD_LEN) ) {
            strncpy(parms.pkpassword, fn, len);
            parms.pkpassword[len] = '\0';
        }
    }
}
char *
get_pkpassword() {
    return parms.pkpassword;
}
char *
get_pkpasswordparm() {
    return (char *)pkpassword_parm;
}
//
// get/set port knocking access ip4
//
void
set_pkallowip4(char * fn, int len) {
    if (parms.pkallow_ip4 == NULL) {
        parms.pkallow_ip4 = malloc(MAX_IP4_LEN+1);
    }
    if (parms.pkallow_ip4 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkallow_ip4", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkallow_ip4 = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_IP4_LEN) ) {
            strncpy(parms.pkallow_ip4, fn, len);
            parms.pkallow_ip4[len] = '\0';
        }
        if (get_useip4()) {
            set_pkallowip(fn, len);
        }
    }
}
char *
get_pkallowip4() {
    return parms.pkallow_ip4;
}
char *
get_pkallowipparm() {
    return (char *)pkallowip_parm;
}
//
// get/set port knocking access ip6
//
void
set_pkallowip6(char * fn, int len) {
    if (parms.pkallow_ip6 == NULL) {
        parms.pkallow_ip6 = malloc(MAX_IP6_LEN+1);
    }
    if (parms.pkallow_ip6 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkallow_ip6", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkallow_ip6 = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_IP6_LEN) ) {
            strncpy(parms.pkallow_ip6, fn, len);
            parms.pkallow_ip6[len] = '\0';
        }
        if (get_useip6()) {
            set_pkallowip(fn, len);
        }
    }
}
char *
get_pkallowip6() {
    return parms.pkallow_ip6;
}
//
// get/set port knocking access request (sent from client)
//
void
set_pkaccessrequest(char * fn, int len) {
    if (parms.pkaccessrequest == NULL) {
        parms.pkaccessrequest = malloc(INVIZ_MAX_PKACCESS_LEN+1);
    }
    if (parms.pkaccessrequest == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkaccessrequest", 0);
    } else {
        char logmsg[512];
        if ( len <= 0 ) {
            *parms.pkaccessrequest = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKACCESS_LEN) ) {
            char * ndx = strchr(fn, '/');
            if (ndx == NULL) {
                sprintf(logmsg, "invalid %s parameter format (protocol/port)", (char *)get_pkaccessrequestparm());
                logit(_LOG_ERROR, logmsg, 0);
                return;
            }
            int pkplen = (ndx-fn);
            if (pkplen <= 0 || pkplen > INVIZ_MAX_PROTOCOL_LEN) {
                sprintf(logmsg, "invalid %s parameter format (protocol/port)", (char *)get_pkaccessrequestparm());
                logit(_LOG_ERROR, logmsg, 0);
                return;
            }
            char pkprotocol[INVIZ_MAX_PROTOCOL_LEN];
            strncpy(pkprotocol, fn, pkplen);
            pkprotocol[pkplen] = '\0';
            if (get_protocolnumber(pkprotocol, pkplen) <= 0) {
                sprintf(logmsg, "invalid %s parameter format (protocol/port)", (char *)get_pkaccessrequestparm());
                logit(_LOG_ERROR, logmsg, 0);
                return;
            }
            int pkport = atoi(ndx+1);
            if (pkport < get_pkminport() || pkport > get_pkmaxport()) {
                sprintf(logmsg, "invalid %s parameter format (protocol/port)", (char *)get_pkaccessrequestparm());
                logit(_LOG_ERROR, logmsg, 0);
                return;
            }
            strncpy(parms.pkaccessrequest, fn, len);
            parms.pkaccessrequest[len] = '\0';
        }
    }
}
char *
get_pkaccessrequest() {
    return parms.pkaccessrequest;
}
char *
get_pkaccessrequestparm() {
    return (char *)pkaccess_parm;
}
void
set_pkaccessprotocol(char * str, int len) {
    set_allprotocolsdisabled();
    if (strncmp(str, IPPROTO_UDP_DESC, len) == 0) {
        set_useudp(TRUE);
        set_useraw(TRUE);
    } else if (strncmp(str, IPPROTO_TCP_DESC, len) == 0) {
        set_usetcp(TRUE);
        set_useraw(TRUE);
    } else if (strncmp(str, IPPROTO_ICMP_DESC, len) == 0) {
        set_useicmp(TRUE);
        set_useraw(TRUE);
    } else if (strncmp(str, IPPROTO_RAW_DESC, len) == 0) {
        set_useraw(TRUE);
    } else if (strncmp(str, IPPROTO_IP_DESC, len) == 0) {
        set_useip(TRUE);
        set_useraw(TRUE);
    } else if (strncmp(str, IPPROTO_SCTP_DESC, len) == 0) {
        set_usesctp(TRUE); // not yet supported
        set_useraw(TRUE);
    } else if (strncmp(str, IPPROTO_HTTPPROXY_DESC, len) == 0) {
        set_useproxy(TRUE);
        set_proxytype(IPPROTO_HTTPPROXY_DESC, strlen(IPPROTO_HTTPPROXY_DESC));
    } else if (strncmp(str, IPPROTO_SOCKSPROXY_DESC, len) == 0) {
        set_useproxy(TRUE);
        set_proxytype(IPPROTO_SOCKSPROXY_DESC, strlen(IPPROTO_SOCKSPROXY_DESC));
    }
    // no set proxy protocol?
}
uint8_t
get_protocolnumber(char * str, int len) {
    if (strncmp(str, IPPROTO_UDP_DESC, len) == 0) {
        return IPPROTO_UDP;
    } else if (strncmp(str, IPPROTO_TCP_DESC, len) == 0) {
        return IPPROTO_TCP;
    } else if (strncmp(str, IPPROTO_ICMP_DESC, len) == 0) {
        return IPPROTO_ICMP;
    } else if (strncmp(str, IPPROTO_RAW_DESC, len) == 0) {
        return IPPROTO_RAW;
    } else if (strncmp(str, IPPROTO_IP_DESC, len) == 0) {
        return IPPROTO_IP;
    } else if (strncmp(str, IPPROTO_SCTP_DESC, len) == 0) {
        return IPPROTO_SCTP;
    } else {
        return 0;
    }
}
//
// get/set nat access request (sent from client)
//
void
set_nataccessrequest(char * fn, int len) {
    if (parms.nataccessrequest == NULL) {
        parms.nataccessrequest = malloc(INVIZ_MAX_PKACCESS_LEN+1);
    }
    if (parms.nataccessrequest == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.nataccessrequest", 0);
    } else {
        if ( len <= 0 ) {
            *parms.nataccessrequest = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PKACCESS_LEN) ) {
            strncpy(parms.nataccessrequest, fn, len);
            parms.nataccessrequest[len] = '\0';
        }
    }
}
char *
get_nataccessrequest() {
    return parms.nataccessrequest;
}
//
// get/set proxy address
//
void
set_proxy(char * fn, int len) {
    int maxiplen = (int)MAX_IP4_LEN;
    if (get_useip6()) {
        maxiplen = (int)MAX_IP6_LEN;
    }
    if (parms.proxy == NULL) {
        parms.proxy = malloc(maxiplen+1);
    }
    if (parms.proxy == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.proxy", 0);
    } else {
        if ( len <= 0 ) {
            *parms.proxy = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= maxiplen) ) {
            strncpy(parms.proxy, fn, len);
            parms.proxy[len] = '\0';
        }
    }
    if (!get_proxytype() || (strlen(get_proxytype()) <= 0)) {
        set_proxytype(INVIZ_DEFAULT_PROXY_TYPE, strlen(INVIZ_DEFAULT_PROXY_TYPE));
    }
    set_useproxy(TRUE);
}
char *
get_proxy() {
    return parms.proxy;
}
//
// get/set pk access ip address
//
void
set_pkallowip(char * fn, int len) {
    int maxiplen = (int)MAX_IP4_LEN;
    if (get_useip6()) {
        maxiplen = (int)MAX_IP6_LEN;
    }
    if (parms.pkallow_ip == NULL) {
        parms.pkallow_ip = malloc(maxiplen+1);
    }
    if (parms.pkallow_ip == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.pkallow_ip", 0);
    } else {
        if ( len <= 0 ) {
            *parms.pkallow_ip = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= maxiplen) ) {
            strncpy(parms.pkallow_ip, fn, len);
            parms.pkallow_ip[len] = '\0';
        }
    }
}
char *
get_pkallowip() {
    return parms.pkallow_ip;
}
//
// get/set proxy type
//
void
set_proxytype(char * fn, int len) {
    if (parms.proxytype == NULL) {
        parms.proxytype = malloc(INVIZ_MAX_PROXYTYPE_LEN+1);
    }
    if (parms.proxytype == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.proxytype", 0);
    } else {
        if ( len <= 0 ) {
            *parms.proxytype = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_PROXYTYPE_LEN) ) {
            strncpy(parms.proxytype, fn, len);
            parms.proxytype[len] = '\0';
        }
    }
}
char *
get_proxytype() {
    return parms.proxytype;
}
int
get_usehttpproxy() {
    if (!get_proxy() || !get_proxytype()) {
        return FALSE;
    }
    if (strlen(get_proxytype()) != strlen(IPPROTO_HTTPPROXY_DESC)) {
        return FALSE;
    }
    char * proxytype = get_proxytype();
    if (strcmp(proxytype, IPPROTO_HTTPPROXY_DESC) == 0) {
        return TRUE;
    }
    if (strcmp(proxytype, IPPROTO_HTTPPROXY_DESC_UPPER) == 0) {
        return TRUE;
    }
    return FALSE;
}
int
get_usesocksproxy() {
    if (!get_proxy() || !get_proxytype()) {
        return FALSE;
    }
    if (strlen(get_proxytype()) != strlen(IPPROTO_SOCKSPROXY_DESC)) {
        return FALSE;
    }
    char * proxytype = get_proxytype();
    if (strcmp(proxytype, IPPROTO_SOCKSPROXY_DESC) == 0) {
        return TRUE;
    }
    if (strcmp(proxytype, IPPROTO_SOCKSPROXY_DESC_UPPER) == 0) {
        return TRUE;
    }
    return FALSE;
}
//
// get/set user agent (for client to use)
//
void
set_useragent(char * fn, int len) {
    if (parms.useragent == NULL) {
        parms.useragent = malloc(INVIZ_MAX_UA_LEN+1);
    }
    if (parms.useragent == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.useragent", 0);
    } else {
        if ( len <= 0 ) {
            *parms.useragent = '\0';
        } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)INVIZ_MAX_UA_LEN) ) {
            strncpy(parms.useragent, fn, len);
            parms.useragent[len] = '\0';
        }
    }
}
char *
get_useragent() {
    return parms.useragent;
}
//
// get/set ps mask to be used by either client or server (if set)
//
void
set_psmask(char * fn, int len) {
    char logmsg[512];
    char newmask[INVIZ_MAX_PSMASK_LEN+1];
    memset(newmask, 0x0, INVIZ_MAX_PSMASK_LEN+1);
    if ( len <= 0 ) {
        memset(parms.psmask, 0x0, INVIZ_MAX_PSMASK_LEN+1);
    } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= INVIZ_MAX_PSMASK_LEN) ) {
        memset(parms.psmask, 0x0, INVIZ_MAX_PSMASK_LEN+1);
        strncpy(parms.psmask, fn, len);
        parms.psmask[len] = '\0';
    }
    int iset = prctl(PR_SET_NAME, (char *)get_psmask(), 0, 0, 0);
    if (iset == EXIT_FAILURE ) {
        sprintf(logmsg, "unable to prctl PR_SET_NAME mask process [%s]", strerror(errno));
        logit(_LOG_WARNING, logmsg, 0);
    }
    int iget = prctl(PR_GET_NAME, newmask, 0, 0, 0);
    if (iget == EXIT_SUCCESS) {
        if (strncmp((char *)get_psmask(), newmask, INVIZ_MAX_PSMASK_LEN) != 0) {
            sprintf(logmsg, "unable to prctl PR_SET_NAME mask process");
            logit(_LOG_WARNING, logmsg, 0);
        }
    }
}
void
set_defaultpsmask() {
    if (strncmp(INVIZ_DEFAULT_PSMASK, INVIZ_RANDOM_DESC, strlen(INVIZ_RANDOM_DESC)) == 0) {
        // --psmask=random
        set_randompsmask();
        set_randomizepsmask(TRUE);
    } else {
        char default_psmask[] = INVIZ_DEFAULT_PSMASK;
        set_psmask(default_psmask, strlen(default_psmask));
        set_randomizepsmask(FALSE);
    }
}
char *
get_psmask() {
    return parms.psmask;
}
int
get_randompsmin() {
    return INVIZ_MIN_PSMASK_RANDOM;
}
int
get_randompsmax() {
    return INVIZ_MAX_PSMASK_RANDOM;
}
int
get_randomizepsmask() {
    return parms.brandomizeps;
}
void
set_randomizepsmask(int r) {
    if (r == TRUE || r == FALSE) {
        parms.brandomizeps = r;
    }
}
void
set_randomizepstimer(double t) {
    parms.irandomizetimer = t;
}
double
get_randomizepstimer() {
    return parms.irandomizetimer;
}
void
get_randompsmask(char aname[INVIZ_MAX_PSMASK_LEN]) {
    int ires, i, x;
    char logmsg[512];
    char *token, *ptr, *ntoken;
    memset(aname, 0x0, INVIZ_MAX_PSMASK_LEN);
    snprintf(cmd_buf, CMD_BUFSIZE-1, "ps -e -o pid= -o cmd= 2>&1");
    ires = run_extcmd_inviz(0, cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
    if(!EXTCMD_IS_SUCCESS(ires)) {
        if (get_verbose()) {
            sprintf(logmsg, "unable to exec %s [%i]", cmd_buf, ires);
            logit(_LOG_ERROR, logmsg, 0);
        }
        return;
    }
    cmd_out[STANDARD_CMD_OUT_BUFSIZE] = '\0';
    char *ndx = cmd_out;
    int ps_count = 0;
    while (TRUE) {
        ptr = strchr(ndx, '\n');
        if (ptr == NULL) {
            break;
        }
        ps_count++; // count of processes found by ps -e
        ndx = (ptr+1);
    }
    x = ((rand() % (ps_count - get_randompsmin())) + get_randompsmin());
    ndx = cmd_out; // reset index
    for (i=0;;i++) {
        ptr = strchr(ndx, '\n');
        if (ptr == NULL) {
            break; // no more processes to look at
        }
        token = malloc((ptr-ndx)+1);
        strlcpy(token, ndx, (ptr-ndx)+1);
        ntoken = token;
        ndx = (ptr+1); // point to next process for next for() iteration
        for (;; ntoken++) {
            // strip leading blanks off token
            ptr = strchr(ntoken, ' ');
            if (ptr != ntoken) {
                break; // out of this for() loop
            }
        }
        ptr = strchr(ntoken, ' ');
        if (ptr == NULL) {
            break; // something wrong - should be space between pid and cmd
        }
        ptr++; // now pointing to command
        ntoken = ptr;
        ptr = strchr(ntoken, ' ');
        memset(aname, 0x0, INVIZ_MAX_PSMASK_LEN);
        if ((ptr == NULL) && (strlen(ntoken) >= INVIZ_MIN_PSMASK_LEN) &&
                (strlen(ntoken) <= INVIZ_MAX_PSMASK_LEN)) {
            sprintf(aname, "%s", ntoken);
        } else if ((ptr != NULL) && ((ptr-ntoken) >= INVIZ_MIN_PSMASK_LEN) &&
                ((ptr-ntoken) <= INVIZ_MAX_PSMASK_LEN)) {
            strlcpy(aname, ntoken, (ptr-ntoken)+1);
        }
        free(token);
        if (i >= x) {
            break; // have match to random process
        }
    }
    return;
}
void
set_randompsmask() {
    char rname[INVIZ_MAX_PSMASK_LEN];
    while (TRUE) {
        get_randompsmask(rname);
        if (strlen(rname) > 0) {
            set_psmask(rname, strlen(rname));
            break;
        }
    }
    struct timeval tprun;
    double time_snapshot;
    gettimeofday(&tprun, NULL);
    time_snapshot = tprun.tv_sec+(tprun.tv_usec/1000000.0);
    set_randomizepstimer(time_snapshot);
}
//
// get/set original ps name (before being masked)
//
void
set_originalpsname() {
    char logmsg[512];
    if ( prctl(PR_GET_NAME, parms.originalpsname, 0, 0, 0) == EXIT_FAILURE ) {
        sprintf(logmsg, "unable to prctl PR_GET_NAME of process [%s]", strerror(errno));
        logit(_LOG_WARNING, logmsg, 0);
    }
}
char *
get_originalpsname() {
    return parms.originalpsname;
}
//
// get/set uname
//
char *
get_uname() {
    return parms.uname;
}
void
set_uname() {
    int ires;
    char logmsg[512];
    //
    if (parms.uname == NULL) {
        parms.uname = malloc(STANDARD_CMD_OUT_BUFSIZE+1);
    }
    if (parms.uname == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.uname", 0);
        return;
    }
    memset(parms.uname, 0x0, STANDARD_CMD_OUT_BUFSIZE+1);
    snprintf(cmd_buf, CMD_BUFSIZE-1, "uname -a 2>&1");
    ires = run_extcmd_inviz(0, cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
    if(!EXTCMD_IS_SUCCESS(ires)) {
        if (get_verbose()) {
            sprintf(logmsg, "unable to exec %s [%i]", cmd_buf, ires);
            logit(_LOG_ERROR, logmsg, 0);
        }
        return;
    }
    cmd_out[STANDARD_CMD_OUT_BUFSIZE] = '\0';
    char *ndx = strchr(cmd_out, '\n');
    if (ndx == NULL) {
        strncpy(parms.uname, cmd_out, strlen(cmd_out));
    } else {
        strncpy(parms.uname, cmd_out, ndx-cmd_out);
    }
    return;
}
//
// get/set program name
//
void
set_programname(char * str, int len) {
    if ( len <= 0 ) {
        *parms.programname = '\0';
    } else if ( (len >= 0) && (strlen(str) >= 1) && (len <= MAX_FILENAME_LEN) ) {
        strncpy(parms.programname, str, len);
        parms.programname[len] = '\0';
    }
}
char *
get_programname() {
    return parms.programname;
}
//
// get/set dev random
//
void
set_devrandom(char * str, int len) {
    if ( len <= 0 ) {
        *parms.devrandom = '\0';
    } else if ( (len >= 0) && (strlen(str) >= 1) && (len <= MAX_FILENAME_LEN) ) {
        strncpy(parms.devrandom, str, len);
        parms.devrandom[len] = '\0';
    }
}
char *
get_devrandom() {
    return parms.devrandom;
}
//
// get/set interface name to use
//
void
set_ifname(char * fn, int len) {
    if ( len <= 0 ) {
        *parms.ifname = '\0';
    } else if ( (len >= 0) && (strlen(fn) >= get_minifnamelen()) && (len <= get_maxifnamelen()) ) {
        strncpy(parms.ifname, fn, len);
        parms.ifname[len] = '\0';
    }
}
char *
get_ifname() {
    return parms.ifname;
}
char *
get_ifnameparm() {
    return (char *)ifname_parm;
}
int
get_minifnamelen() {
    return MIN_IFNAME_LEN;
}
int
get_maxifnamelen() {
    return MAX_IFNAME_LEN;
}
//
// get/set interface name IP address to use
//
void
set_ifnameip(char * fn, int len) {
    if ( len <= 0 ) {
        *parms.ifnameip = '\0';
    } else if ( (len >= 0) && (strlen(fn) >= 0) && (len <= MAX_IP6_LEN) ) {
        strncpy(parms.ifnameip, fn, len);
        parms.ifnameip[len] = '\0';
    }
}
char *
get_ifnameip() {
    return parms.ifnameip;
}
//
// get/set packet filter
//
void
set_packetfilter(char * fn, int len) {
    if ( len <= 0 ) {
        *parms.packetfilter = '\0';
    } else if ( (len >= 0) && (strlen(fn) > 0) && (len <= (int)MAX_PACKETFILTER_LEN) ) {
        strncpy(parms.packetfilter, fn, len);
        parms.packetfilter[len] = '\0';
    }
}
char *
get_packetfilter() {
    return parms.packetfilter;
}
//
// get/set trigger hostname
//
void
set_triggerhostname(char trigger_hostname[NI_MAXHOST], char seed_hostname[NI_MAXHOST], char hextension[16]) {
    sprintf(trigger_hostname, "%s.%s%s", (char *)get_keyseed(), seed_hostname, hextension);
    int tlen = strlen(trigger_hostname);
    if (tlen <= 0) {
        *parms.triggerhostname = '\0';
    } else {
        strncpy(parms.triggerhostname, trigger_hostname, tlen);
        parms.triggerhostname[tlen] = '\0';
    }
}
char *
get_triggerhostname() {
    return parms.triggerhostname;
}
//
// get/set file size
//
void
set_filesize(long l) {
    parms.lfilesize = l;
}
inline long
get_filesize() {
    return parms.lfilesize;
}
//
// get/set rotating seed
//
void
set_irotatingseed(int c) {
    if (c >= get_miniseed() && c <= get_maxiseed()) {
        parms.irotatingseed = c;
    }
}
inline int
get_irotatingseed() {
    return parms.irotatingseed;
}
inline void
inc_irotatingseed() {
    parms.irotatingseed++;
    if (parms.irotatingseed > get_maxiseed()) { // 11 > 10
        int x = (parms.irotatingseed - get_miniseed()); // 11 - 3
        int current_overflow = get_iseedoverflow(); // 15
        set_iseedoverflow(current_overflow + x); // 23
        parms.irotatingseed = get_miniseed(); // 3
    }
}
inline void
dec_irotatingseed() {
    parms.irotatingseed--;
    if (parms.irotatingseed < get_miniseed()) { // 2 < 3
        int x = (get_maxiseed() - parms.irotatingseed); // 10 - 2
        int current_overflow = get_iseedoverflow(); // 23
        set_iseedoverflow(current_overflow - x); // 15
        parms.irotatingseed = get_maxiseed(); // 10
    }
}
//
// get/set packet count
//
void
set_ipackets(int c) {
    parms.ipackets = c;
}
inline int
get_ipackets() {
    return parms.ipackets;
}
inline void
inc_ipackets() {
    parms.ipackets++;
}
void
set_ipacketstodo(int total) {
    parms.ipacketstodo = total;
}
int
get_ipacketstodo() {
    return parms.ipacketstodo;
}
//
// get/set packet count limit
//
void
set_ipacketlimit(int c) {
    parms.ipacketlimit = c;
}
inline int
get_ipacketlimit() {
    return parms.ipacketlimit;
}
//
// get/set confirmation interval packets (client only)
//
void
set_iconfirminterval(int c) {
    if (c >= 0) {
        parms.iconfirminterval = c;
    }
}
int
get_iconfirminterval() {
    return parms.iconfirminterval;
}
void
set_iwatchdog(int c) {
    parms.iwatchdog = c;
}
int
dec_iwatchdog() {
    return --parms.iwatchdog;
}
//
// get/set duplicate interval pause (seconds)
//
void
set_iduplicateinterval(int c) {
    if (c >= 0) {
        parms.iduplicateinterval = c;
    }
}
int
get_iduplicateinterval() {
    return parms.iduplicateinterval;
}
//
// get/set confirmation expiry seconds (client only)
//
void
set_iconfirmexpiry(int c) {
    if (c >= get_minconfirmexpiry() && c <= get_maxconfirmexpiry()) {
        parms.iconfirmexpiry = c;
    }
}
int
get_iconfirmexpiry() {
    return parms.iconfirmexpiry;
}
int
get_maxconfirmexpiry() {
    return INVIZ_MAX_CONFIRM_EXPIRY;
}
int
get_minconfirmexpiry() {
    return INVIZ_MIN_CONFIRM_EXPIRY;
}
//
// get/set file size transferred to date
//
void
set_filedone(long l) {
    parms.lfiledone = l;
}
void
set_filedonemore(long m) {
    // NOTE: this is additive
    parms.lfiledone += m;
}
inline long
get_filedone() {
    return parms.lfiledone;
}
//
// get/set server ip4
//
void
set_serverip4(char * ip, int len) {
    if ( len <= 0 ) {
        *parms.server_ip4 = '\0';
    } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)MAX_IP4_LEN) ) {
        strncpy(parms.server_ip4, ip, len);
        parms.server_ip4[len] = '\0';
    }
    if (get_useip4()) {
        set_serverip((char *)ip, len);
    }
}
char *
get_serverip4() {
    return parms.server_ip4;
}
//
// get/set server hostname
//
void
set_serverhostname(char * ip, int len) {
    if (parms.server_hostname == NULL) {
        parms.server_hostname = malloc((int)NI_MAXHOST+1);
    }
    if (parms.server_hostname == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.server_hostname", 0);
    } else {
        if ( len <= 0 ) {
            *parms.server_hostname = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)NI_MAXHOST) ) {
            strncpy(parms.server_hostname, ip, len);
            parms.server_hostname[len] = '\0';
        }
    }
}
char *
get_serverhostname() {
    return parms.server_hostname;
}
//
// get/set spoof ip4
//
void
set_spoofip4(char * ip, int len) {
    if (parms.spoof_ip4 == NULL) {
        parms.spoof_ip4 = malloc((int)MAX_IP4_LEN+1);
    }
    if (parms.spoof_ip4 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.spoof_ip4", 0);
    } else {
        if ( len <= 0 ) {
            *parms.spoof_ip4 = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)MAX_IP4_LEN) ) {
            strncpy(parms.spoof_ip4, ip, len);
            parms.spoof_ip4[len] = '\0';
        }
        if (get_useip4()) {
            set_spoofip((char *)ip, len);
        }
    }
}
char *
get_spoofip4() {
    return parms.spoof_ip4;
}
//
// get/set spoof ip6
//
void
set_spoofip6(char * ip, int len) {
    if (parms.spoof_ip6 == NULL) {
        parms.spoof_ip6 = malloc((int)MAX_IP6_LEN+1);
    }
    if (parms.spoof_ip6 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.spoof_ip6", 0);
    } else {
        if ( len <= 0 ) {
            *parms.spoof_ip6 = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)MAX_IP6_LEN) ) {
            strncpy(parms.spoof_ip6, ip, len);
            parms.spoof_ip6[len] = '\0';
        }
        if (get_useip6()) {
            set_spoofip((char *)ip, len);
        }
    }
}
char *
get_spoofip6() {
    return parms.spoof_ip6;
}
//
// get/set spoof hostname
//
void
set_spoofhostname(char * ip, int len) {
    if (parms.spoof_hostname == NULL) {
        parms.spoof_hostname = malloc((int)NI_MAXHOST+1);
    }
    if (parms.spoof_hostname == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.spoof_hostname", 0);
    } else {
        if ( len <= 0 ) {
            *parms.spoof_hostname = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)NI_MAXHOST) ) {
            strncpy(parms.spoof_hostname, ip, len);
            parms.spoof_hostname[len] = '\0';
        }
    }
}
char *
get_spoofhostname() {
    return parms.spoof_hostname;
}
char *
get_spoofipparm() {
    return (char *)spoofip_parm;
}
//
// get/set dns server ip4
//
void
set_dnsip4(char * ip, int len) {
    if ( len <= 0 ) {
        *parms.dns_ip4 = '\0';
    } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)MAX_IP4_LEN) ) {
        strncpy(parms.dns_ip4, ip, len);
        parms.dns_ip4[len] = '\0';
    }
    if (get_useip4()) {
        set_dnsip((char *)ip, len);
    }
}
char *
get_dnsip4() {
    return parms.dns_ip4;
}
//
// get/set dns hostname
//
void
set_dnshostname(char * ip, int len) {
    if (parms.dns_hostname == NULL) {
        parms.dns_hostname = malloc((int)NI_MAXHOST+1);
    }
    if (parms.dns_hostname == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.dns_hostname", 0);
    } else {
        if ( len <= 0 ) {
            *parms.dns_hostname = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)NI_MAXHOST) ) {
            strncpy(parms.dns_hostname, ip, len);
            parms.dns_hostname[len] = '\0';
        }
    }
}
char *
get_dnshostname() {
    return parms.dns_hostname;
}
//
// get/set external ip4
//
void
set_externalip4(char * ip, int len) {
    if (parms.external_ip4 == NULL) {
        parms.external_ip4 = malloc((int)MAX_IP4_LEN+1);
    }
    if (parms.external_ip4 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.external_ip4", 0);
    } else {
        if ( len <= 0 ) {
            *parms.external_ip4 = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)MAX_IP4_LEN) ) {
            strncpy(parms.external_ip4, ip, len);
            parms.external_ip4[len] = '\0';
        }
        if (get_useip4()) {
            set_externalip((char *)ip, len);
        }
    }
}
char *
get_externalip4() {
    return parms.external_ip4;
}
//
// get/set external hostname
//
void
set_externalhostname(char * ip, int len) {
    if (parms.external_hostname == NULL) {
        parms.external_hostname = malloc((int)NI_MAXHOST+1);
    }
    if (parms.external_hostname == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.external_hostname", 0);
    } else {
        if ( len <= 0 ) {
            *parms.external_hostname = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)NI_MAXHOST) ) {
            strncpy(parms.external_hostname, ip, len);
            parms.external_hostname[len] = '\0';
        }
    }
}
char *
get_externalhostname() {
    return parms.external_hostname;
}
//
// get/set client server ip4 (if sending back confirmation packets)
//
void
set_clientip4(char * ip, int len) {
    if ( len <= 0 ) {
        *parms.client_ip4 = '\0';
    } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)MAX_IP4_LEN) ) {
        strncpy(parms.client_ip4, ip, len);
        parms.client_ip4[len] = '\0';
    }
    if (get_useip4()) {
        set_clientip((char *)ip, len);
    }
}
char *
get_clientip4() {
    return parms.client_ip4;
}
//
// get/set client hostname
//
void
set_clienthostname(char * ip, int len) {
    if (parms.client_hostname == NULL) {
        parms.client_hostname = malloc((int)NI_MAXHOST+1);
    }
    if (parms.client_hostname == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.client_hostname", 0);
    } else {
        if ( len <= 0 ) {
            *parms.client_hostname = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)NI_MAXHOST) ) {
            strncpy(parms.client_hostname, ip, len);
            parms.client_hostname[len] = '\0';
        }
    }
}
char *
get_clienthostname() {
    return parms.client_hostname;
}
//
// get/set server ip6
//
void
set_serverip6(char * ip, int len) {
    if ( len <= 0 ) {
        *parms.server_ip6 = '\0';
    } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)MAX_IP6_LEN) ) {
        strncpy(parms.server_ip6, ip, len);
        parms.server_ip6[len] = '\0';
    }
    if (get_useip6()) {
        set_serverip((char *)ip, len);
    }
}
char *
get_serverip6() {
    return parms.server_ip6;
}
//
// get/set server ip
//
void
set_serverip(char * ip, int len) {
    int maxiplen = (int)MAX_IP4_LEN;
    if (get_useip6()) {
        maxiplen = (int)MAX_IP6_LEN;
    }
    if (parms.server_ip == NULL) {
        parms.server_ip = malloc(maxiplen+1);
    }
    if (parms.server_ip == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.server_ip", 0);
    } else {
        if ( len <= 0 ) {
            *parms.server_ip = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= maxiplen) ) {
            strncpy(parms.server_ip, ip, len);
            parms.server_ip[len] = '\0';
        }
    }
}
char *
get_serverip() {
    return parms.server_ip;
}
//
// get/set dns ip
//
void
set_dnsip(char * ip, int len) {
    int maxiplen = (int)MAX_IP4_LEN;
    if (get_useip6()) {
        maxiplen = (int)MAX_IP6_LEN;
    }
    if (parms.dns_ip == NULL) {
        parms.dns_ip = malloc(maxiplen+1);
    }
    if (parms.dns_ip == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.dns_ip", 0);
    } else {
        if ( len <= 0 ) {
            *parms.dns_ip = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= maxiplen) ) {
            strncpy(parms.dns_ip, ip, len);
            parms.dns_ip[len] = '\0';
        }
    }
}
char *
get_dnsip() {
    return parms.dns_ip;
}
//
// get/set client ip
//
void
set_clientip(char * ip, int len) {
    int maxiplen = (int)MAX_IP4_LEN;
    if (get_useip6()) {
        maxiplen = (int)MAX_IP6_LEN;
    }
    if (parms.client_ip == NULL) {
        parms.client_ip = malloc(maxiplen+1);
    }
    if (parms.client_ip == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.client_ip", 0);
    } else {
        if ( len <= 0 ) {
            *parms.client_ip = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= maxiplen) ) {
            strncpy(parms.client_ip, ip, len);
            parms.client_ip[len] = '\0';
        }
    }
}
char *
get_clientip() {
    return parms.client_ip;
}
//
// get/set external ip
//
void
set_externalip(char * ip, int len) {
    int maxiplen = (int)MAX_IP4_LEN;
    if (get_useip6()) {
        maxiplen = (int)MAX_IP6_LEN;
    }
    if (parms.external_ip == NULL) {
        parms.external_ip = malloc(maxiplen+1);
    }
    if (parms.external_ip == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.external_ip", 0);
    } else {
        if ( len <= 0 ) {
            *parms.external_ip = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= maxiplen) ) {
            strncpy(parms.external_ip, ip, len);
            parms.external_ip[len] = '\0';
        }
    }
}
char *
get_externalip() {
    return parms.external_ip;
}
char *
get_externalipparm() {
    return (char *)externalip_parm;
}
//
// get/set spoof ip
//
void
set_spoofip(char * ip, int len) {
    int maxiplen = (int)MAX_IP4_LEN;
    if (get_useip6()) {
        maxiplen = (int)MAX_IP6_LEN;
    }
    if (parms.spoof_ip == NULL) {
        parms.spoof_ip = malloc(maxiplen+1);
    }
    if (parms.spoof_ip == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.spoof_ip", 0);
    } else {
        if ( len <= 0 ) {
            *parms.spoof_ip = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= maxiplen) ) {
            strncpy(parms.spoof_ip, ip, len);
            parms.spoof_ip[len] = '\0';
        }
    }
}
char *
get_spoofip() {
    return parms.spoof_ip;
}
//
// get/set client ip6 (if sending back confirmation packets)
//
void
set_clientip6(char * ip, int len) {
    if ( len <= 0 ) {
        *parms.client_ip6 = '\0';
    } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)MAX_IP6_LEN) ) {
        strncpy(parms.client_ip6, ip, len);
        parms.client_ip6[len] = '\0';
    }
    if (get_useip6()) {
        set_clientip((char *)ip, len);
    }
}
char *
get_clientip6() {
    return parms.client_ip6;
}
//
// get/set dns server ip6
//
void
set_dnsip6(char * ip, int len) {
    if ( len <= 0 ) {
        *parms.dns_ip6 = '\0';
    } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)MAX_IP6_LEN) ) {
        strncpy(parms.dns_ip6, ip, len);
        parms.dns_ip6[len] = '\0';
    }
    if (get_useip6()) {
        set_dnsip((char *)ip, len);
    }
}
char *
get_dnsip6() {
    return parms.dns_ip6;
}
//
// get/set external ip6
//
void
set_externalip6(char * ip, int len) {
    if (parms.external_ip6 == NULL) {
        parms.external_ip6 = malloc((int)MAX_IP6_LEN+1);
    }
    if (parms.external_ip6 == NULL) {
        logit(_LOG_ERROR, "unable to malloc parms.external_ip6", 0);
    } else {
        if ( len <= 0 ) {
            *parms.external_ip6 = '\0';
        } else if ( (len >= 0) && (strlen(ip) > 0) && (len <= (int)MAX_IP6_LEN) ) {
            strncpy(parms.external_ip6, ip, len);
            parms.external_ip6[len] = '\0';
        }
        if (get_useip6()) {
            set_externalip((char *)ip, len);
        }
    }
}
char *
get_externalip6() {
    return parms.external_ip6;
}
//
// get/set file replace parameter value (on server)
//
int
get_replace() {
    return parms.breplace;
}
void
set_replace(int u) {
    if (u == FALSE || u == TRUE) {
        parms.breplace = u;
    }
}
//
// get/set DNS randomize case parameter
//
int
get_randomizecase() {
    return parms.brandomizecase;
}
void
set_randomizecase(int u) {
    if (u == FALSE || u == TRUE) {
        parms.brandomizecase = u;
    }
}
//
// get/set file append parameter value (on server)
//
int
get_append() {
    return parms.bappend;
}
void
set_append(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bappend = u;
    }
}
//
// get/set file remove parameter value (from client)
//
int
get_remove() {
    return parms.bremove;
}
void
set_remove(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bremove = u;
    }
}
//
// get/set wipe program parameter value
//
int
get_wipeprogram() {
    return parms.bwipeprogram;
}
void
set_wipeprogram(int u) {
    if (u == FALSE || u == TRUE) {
        parms.bwipeprogram = u;
    }
}
//
// get/set wipe times parameter value
//
int
get_iwipetimes() {
    return parms.iwipetimes;
}
void
set_iwipetimes(int u) {
    if (u >= MIN_WIPE_FILE_TIMES && u <= MAX_WIPE_FILE_TIMES) {
        parms.iwipetimes = u;
    }
}
//
// get/set integer seed parameter
//
void
set_iseed(int s) {
    if (s >= (int)INVIZ_MIN_SEED && s <= (int)INVIZ_MAX_SEED) {
        parms.iseed = s;
    }
}
inline int
get_iseed() {
    return parms.iseed;
}
inline int
get_maxiseed() {
    return (int)INVIZ_MAX_SEED;
}
inline int
get_miniseed() {
    return (int)INVIZ_MIN_SEED;
}
//
// get/set recv buffer len parameter
//
void
set_recvbufferlen(int s) {
    if (s >= get_minrecvbufferlen() && s <= get_maxrecvbufferlen()) {
        parms.irecvpacketlen = s;
    }
}
inline int
get_recvbufferlen() {
    return parms.irecvpacketlen;
}
int
get_maxrecvbufferlen() {
    return (int)MAX_PCAP_PACKET_COUNT;
}
int
get_minrecvbufferlen() {
    return (int)MIN_PCAP_PACKET_COUNT;
}
//
// get/set pcap buffer len parameter
//
void
set_pcapbufferlen(int s) {
    if (s >= get_minpcapbufferlen() && s <= get_maxpcapbufferlen()) {
        parms.pcap_bufsize = s;
    }
}
inline int
get_pcapbufferlen() {
    return parms.pcap_bufsize;
}
int
get_maxpcapbufferlen() {
    return (int)INVIZ_MAX_PCAP_BUFSIZ;
}
int
get_minpcapbufferlen() {
    return (int)INVIZ_MIN_PCAP_BUFSIZ;
}
//
// get/set pcap timeout parameter
//
void
set_pcaptimeout(int s) {
    if (s >= get_minpcaptimeout() && s <= get_maxpcaptimeout()) {
        parms.pcap_timeout = s;
    }
}
inline int
get_pcaptimeout() {
    return parms.pcap_timeout;
}
int
get_maxpcaptimeout() {
    return (int)INVIZ_MAX_PCAP_TIMEOUT;
}
int
get_minpcaptimeout() {
    return (int)INVIZ_MIN_PCAP_TIMEOUT;
}
//
// get/set recv packet set parameter
//
void
set_recvbufferset(int s) {
    if (s == TRUE || s <= FALSE) {
        parms.brecvpacketset = s;
    }
}
int
get_recvbufferset() {
    return parms.brecvpacketset;
}
//
// get/set sleep max parameter
//
void
set_sleepmax(int s) {
    if ( (s >= get_minsleepmax()) && (s <= get_maxsleepmax()) ) {
        parms.isleepmax = s;
    }
}
int
get_sleepmax() {
    return parms.isleepmax;
}
int
get_maxsleepmax() {
    return (int)INVIZ_MAX_SLEEP_SECONDS;
}
int
get_minsleepmax() {
    return parms.isleepmin;
}
void
set_minsleepmax(int s) {
    if ( (s >= 0) && (s <= get_maxsleepmax()) ) {
        parms.isleepmin = s;
    }
}
//
// get/set placeholder for file transfer start
//
void
set_started(int v) {
    parms.bstarted = v;
}
inline int
get_started() {
    return parms.bstarted;
}
//
// get/set placeholder for file transfer finish
//
void
set_finished(int v) {
    parms.bfinished = v;
}
int
get_finished() {
    return parms.bfinished;
}
//
// get/set placeholder for file handle
//
inline FILE *
get_pfile() {
    return parms.pFile;
}
void
set_pfile(FILE *ptr) {
    parms.pFile = ptr;
}
//
// valparms() function to validate startup parameters
//
int
valparms(int argc, char** argv) {
    int i, n, ispecified, parm_len, ilen;
    char logmsg[512];
    FILE *pFile;

    if (argc-1 < min_parms) {
        sprintf(logmsg, "min %i parameters required but %i provided", min_parms, argc);
        logit(_LOG_ERROR, logmsg, 0);
        return (EXIT_FAILURE);
    }
    if (argc-1 > max_parms) {
        sprintf(logmsg, "max %i parameters allowed but %i provided", max_parms, argc);
        logit(_LOG_ERROR, logmsg, 0);
        return (EXIT_FAILURE);
    }
    for ( n=1; n<argc; n++ ) {
        // no op? (just a place holder)
        if (strcmp((char *)argv[n], noop_parm) == 0) {
            continue;
        }
        // enable debug/verbose mode?
        if (strcmp((char *)argv[n], verbose_parm) == 0) {
            i = get_verbose();
            i++;
            set_verbose(i);
            continue;
        }
        if (strcmp((char *)argv[n], v_parm) == 0) {
            i = get_verbose();
            i++;
            set_verbose(i);
            continue;
        }
        if (strcmp((char *)argv[n], quiet_parm) == 0) {
            set_verbose(FALSE);
            continue;
        }
        if (strcmp((char *)argv[n], dups_parm) == 0) {
            i = get_duplicates();
            i++;
            set_duplicates(i);
            continue;
        }
        if (strcmp((char *)argv[n], nodups_parm) == 0) {
            set_duplicates(FALSE);
            continue;
        }
        // enable port knocking beacons
        if (strcmp((char *)argv[n], pkbeacon_parm) == 0) {
            set_pkbeacon(TRUE);
            continue;
        }
        // disable port knocking beacons
        if (strcmp((char *)argv[n], pknobeacon_parm) == 0) {
            set_pkbeacon(FALSE);
            continue;
        }
        // enable testing (nothing actually sent out or processed)
        // not fully implemented yet (as of May 16, 2011)
        if (strcmp((char *)argv[n], testing_parm) == 0) {
            set_testing(TRUE);
            continue;
        }
        // enable tcp server (server only)
        if (strcmp((char *)argv[n], tcpserver_parm) == 0) {
            set_usetcpserver(TRUE);
            continue;
        }
        // enable logging to log file
        if (strcmp((char *)argv[n], log_parm) == 0) {
            set_uselogfile(TRUE);
            if (!get_logfilename() || (strlen(get_logfilename()) == 0)) {
                set_logfilename(INVIZ_DEFAULT_LOG_FILENAME, strlen(INVIZ_DEFAULT_LOG_FILENAME));
            }
            continue;
        }
        // enable logging to log file
        if (strcmp((char *)argv[n], l_parm) == 0) {
            set_uselogfile(TRUE);
            if (!get_logfilename() || (strlen(get_logfilename()) == 0)) {
                set_logfilename(INVIZ_DEFAULT_LOG_FILENAME, strlen(INVIZ_DEFAULT_LOG_FILENAME));
            }
            continue;
        }
        // disable logging to log file
        if (strcmp((char *)argv[n], nolog_parm) == 0) {
            set_uselogfile(FALSE);
            if (get_logfilename() && (strlen(get_logfilename()) > 0)) {
                set_logfilename("", 0);
            }
            continue;
        }
        // enable gpg
        if (strcmp((char *)argv[n], gpg_parm) == 0) {
            set_usegpg(TRUE);
            continue;
        }
        // enable gpg agent
        if (strcmp((char *)argv[n], gpgagent_parm) == 0) {
            set_usegpg_agent(TRUE);
            continue;
        }
        // do pk on local private subnet only
        if (strcmp((char *)argv[n], pkprivate_parm) == 0) {
            set_pkprivatesubnet(TRUE);
            continue;
        }
        // do pk on local private subnet only
        if (strcmp((char *)argv[n], pkp_parm) == 0) {
            set_pkprivatesubnet(TRUE);
            continue;
        }
        // do not do pk on local private subnet
        if (strcmp((char *)argv[n], pknotp_parm) == 0) {
            set_pkprivatesubnet(FALSE);
            continue;
        }
        // enable/allow spa over http
        if (strcmp((char *)argv[n], spahttp_parm) == 0) {
            set_allowspaoverhttp(TRUE);
            continue;
        }
        // enable/allow firewall output
        if (strcmp((char *)argv[n], fwoutput_parm) == 0) {
            set_fwallowoutput(TRUE);
            continue;
        }
        // enable/allow firewall forwarding
        if (strcmp((char *)argv[n], fwforwarding_parm) == 0) {
            set_fwallowforwarding(TRUE);
            continue;
        }
        // enable/allow firewall snat (source NAT)
        if (strcmp((char *)argv[n], fwsnat_parm) == 0) {
            set_fwallowsnat(TRUE);
            continue;
        }
        // enable promiscuous mode (for sniffing with pcap)
        if (strcmp((char *)argv[n], pmode_parm) == 0) {
            set_promiscuous(TRUE);
            continue;
        }
        // enable replace existing file (for server only
        // or for client saving port knocking packet file)
        if (strcmp((char *)argv[n], replace_parm) == 0) {
            set_replace(TRUE);
            continue;
        }
        // enable append to existing file (for server only
        // or for client saving port knocking packet file)
        if (strcmp((char *)argv[n], append_parm) == 0) {
            set_append(TRUE);
            continue;
        }
        // enable remove existing file (for client only)
        if (strcmp((char *)argv[n], remove_parm) == 0) {
            set_remove(TRUE);
            continue;
        }
        // enable wipe program
        if (strcmp((char *)argv[n], wipe_parm) == 0) {
            set_wipeprogram(TRUE);
            continue;
        }
        // disable using transaction id
        if (strcmp((char *)argv[n], notransid_parm) == 0) {
            set_usetransid(FALSE);
            continue;
        }
        // --client same as --mode=client
        if (strcmp((char *)argv[n], clientmode_parm) == 0) {
            set_mode(INVIZ_MODE_SENDER);
            set_interactive(TRUE); // allow interactive password entry on client
            continue;
        }
        // -C same as --mode=client
        if (strcmp((char *)argv[n], cmode_parm) == 0) {
            set_mode(INVIZ_MODE_SENDER);
            set_interactive(TRUE);
            continue;
        }
        // --server same as --mode=server
        if (strcmp((char *)argv[n], servermode_parm) == 0) {
            set_mode(INVIZ_MODE_LISTENER);
            continue;
        }
        // -S same as --mode=server
        if (strcmp((char *)argv[n], smode_parm) == 0) {
            set_mode(INVIZ_MODE_LISTENER);
            continue;
        }
        // use ip4 parameter
        parm_len = strlen(ip4_parm);
        if (strncmp((char *)argv[n], ip4_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified != 0 && ispecified != 1) {
                sprintf(logmsg, "%s parameter must be 0 or 1", ip4_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_useip4(ispecified);
            continue;
        }
        // port knocking beacon min interval seconds parameter
        parm_len = strlen(pkbeaconmininterval_parm);
        if (strncmp((char *)argv[n], pkbeaconmininterval_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < INVIZ_MIN_PKBEACON_INTERVAL || ispecified > INVIZ_MAX_PKBEACON_INTERVAL) {
                sprintf(logmsg, "%s parameter must be from %d to %d",
                        pkbeaconmininterval_parm,
                        INVIZ_MIN_PKBEACON_INTERVAL,
                        INVIZ_MAX_PKBEACON_INTERVAL);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkbeaconmininterval(ispecified);
            continue;
        }
        // port knocking beacon max interval seconds parameter
        parm_len = strlen(pkbeaconmaxinterval_parm);
        if (strncmp((char *)argv[n], pkbeaconmaxinterval_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < INVIZ_MIN_PKBEACON_INTERVAL || ispecified > INVIZ_MAX_PKBEACON_INTERVAL) {
                sprintf(logmsg, "%s parameter must be from %d to %d",
                        pkbeaconmaxinterval_parm,
                        INVIZ_MIN_PKBEACON_INTERVAL,
                        INVIZ_MAX_PKBEACON_INTERVAL);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkbeaconmaxinterval(ispecified);
            continue;
        }
        // fko pcap parameter
        parm_len = strlen(fkopcap_parm);
        if (strncmp((char *)argv[n], fkopcap_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified != FALSE && ispecified != TRUE) {
                sprintf(logmsg, "%s parameter must be 0 or 1", fkopcap_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_usefkopcap(ispecified);
            continue;
        }
        // nat local parameter
        parm_len = strlen(natlocal_parm);
        if (strncmp((char *)argv[n], natlocal_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified != FALSE && ispecified != TRUE) {
                sprintf(logmsg, "%s parameter must be 0 or 1", natlocal_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_natlocal(ispecified);
            continue;
        }
        // use ip6 parameter
        parm_len = strlen(ip6_parm);
        if (strncmp((char *)argv[n], ip6_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified != 0 && ispecified != 1) {
                sprintf(logmsg, "%s parameter must be 0 or 1", ip6_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_useip6(ispecified);
            continue;
        }
        // seed parameter
        // must match on both client and server side
        parm_len = strlen(seed_parm);
        if (strncmp((char *)argv[n], seed_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < get_miniseed() || ispecified > get_maxiseed()) {
                sprintf(logmsg, "%s parameter must be between %i and %i",
                        seed_parm, get_miniseed(), get_maxiseed());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_iseed(ispecified);
            continue;
        }
        // wipe file times parameter
        parm_len = strlen(wipetimes_parm);
        if (strncmp((char *)argv[n], wipetimes_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < MIN_WIPE_FILE_TIMES || ispecified > MAX_WIPE_FILE_TIMES) {
                sprintf(logmsg, "%s parameter must be between %i and %i",
                        wipetimes_parm, MIN_WIPE_FILE_TIMES, MAX_WIPE_FILE_TIMES);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_iwipetimes(ispecified);
            continue;
        }
        // seed constant parameter
        // must match on both client and server side
        parm_len = strlen(seedconstant_parm);
        if (strncmp((char *)argv[n], seedconstant_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < 0 || ispecified > INVIZ_MAX_SEED_CONSTANT) {
                sprintf(logmsg, "%s parameter must be between 0 and %i",
                        seedconstant_parm, INVIZ_MAX_SEED_CONSTANT);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_iseedconstant(ispecified);
            continue;
        }
        // source port number parameter (client only)
        parm_len = strlen(srcport_parm);
        if (strncmp((char *)argv[n], srcport_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < get_minsrcport() || ispecified > get_maxsrcport()) {
                sprintf(logmsg, "%s parameter must be between %i and %i", srcport_parm, get_minsrcport(), get_maxsrcport());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_srcport(ispecified);
            continue;
        }
        // tcp server port number parameter (server only)
        parm_len = strlen(tcpserverport_parm);
        if (strncmp((char *)argv[n], tcpserverport_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified <= get_mintcpserverport() || ispecified > get_maxtcpserverport()) {
                sprintf(logmsg, "%s parameter must be between %i and %i",
                        tcpserverport_parm, get_mintcpserverport(), get_maxtcpserverport());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_tcpserverport(ispecified);
            continue;
        }
        // port knock backdoor port number parameter (to received pk packet)
        // must match on both client and server side
        // proxy type to use (for client)
        parm_len = strlen(pkport_parm);
        if (strncmp((char *)argv[n], pkport_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PKPORT_LEN) {
                sprintf(logmsg, "max port knocking backdoor port length %i exceeded",
                        (int)INVIZ_MAX_PKPORT_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            if (strncmp(argv[n]+parm_len, (char *)INVIZ_RANDOM_DESC, ilen) == 0) {
                //
                // --pkport=random
                //
                if (get_pkportlast() && get_pkportfirst() && (get_pkportlast() >= get_pkportfirst())) {
                    set_pkport((rand() % (get_pkportlast() - get_pkportfirst())) + get_pkportfirst());
                    continue;
                } else {
                    sprintf(logmsg, "port knocking backdoor port cannot be set to %s",
                            (char *)INVIZ_RANDOM_DESC);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
            }
            char *pkpvalue = malloc(ilen+1);
            if (pkpvalue == NULL) {
                sprintf(logmsg, "unexpected port knocking backdoor %s port value",
                        (char *)get_pkportparm());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            memset(pkpvalue, 0, ilen+1);
            strncpy(pkpvalue, argv[n]+parm_len, ilen);
            char *ndx = strchr(pkpvalue, '-');
            if (ndx == NULL) {
                //
                // not a port number range
                // assume is single port number
                // --pkport=n
                //
                ispecified = atoi(pkpvalue);
                set_pkport(ispecified);
                free(pkpvalue); pkpvalue = NULL;
                if (get_pkport() != ispecified) {
                    sprintf(logmsg, "unexpected port knocking backdoor %s port value",
                            (char *)get_pkportparm());
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                continue;
            }
            //
            // assume is range of port numbers
            // --pkport=n-n
            //
            ndx++; // now point to last port in range
            char *pkplast = malloc(ilen-(ndx-pkpvalue)+1);
            if (pkplast == NULL) {
                free(pkpvalue); pkpvalue = NULL;
                sprintf(logmsg, "unexpected port knocking backdoor %s port value",
                        (char *)get_pkportparm());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            memset(pkplast, 0, ilen-(ndx-pkpvalue)+1);
            strncpy(pkplast, ndx, ilen-(ndx-pkpvalue));
            ispecified = atoi(pkplast);
            set_pkportlast(ispecified);
            free(pkplast); pkplast = NULL;
            if (get_pkportlast() != ispecified) {
                free(pkpvalue); pkpvalue = NULL;
                sprintf(logmsg, "unexpected port knocking backdoor %s port value",
                        (char *)get_pkportparm());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            char *pkpfirst = malloc(ndx-pkpvalue);
            if (pkpfirst == NULL) {
                free(pkpvalue); pkpvalue = NULL;
                sprintf(logmsg, "unexpected port knocking backdoor %s port value",
                        (char *)get_pkportparm());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            memset(pkpfirst, 0, ndx-pkpvalue);
            strncpy(pkpfirst, pkpvalue, ndx-pkpvalue-1);
            ispecified = atoi(pkpfirst);
            set_pkportfirst(ispecified);
            free(pkpfirst); pkpfirst = NULL;
            free(pkpvalue); pkpvalue = NULL;
            if (get_pkportfirst() != ispecified) {
                sprintf(logmsg, "unexpected port knocking backdoor %s port value",
                        (char *)get_pkportparm());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            continue;
        }
        // port knock type parameter
        // must match on both client and server side
        // although --pk=any can be specified on listening side to expect
        // any kind of incoming pk packets
        parm_len = strlen(pktype_parm);
        if (strncmp((char *)argv[n], pktype_parm, parm_len) == 0) {
            //
            int value_len = strlen( argv[n]+parm_len );
            if ((value_len == strlen(INVIZ_ANY_DESC)) &&
                    (strncmp(argv[n]+parm_len, INVIZ_ANY_DESC, value_len) == 0)) {
                ispecified = INVIZ_PKTYPE_ANY;
            } else if ((value_len == strlen(INVIZ_WIPE_DESC)) &&
                    (strncmp(argv[n]+parm_len, INVIZ_WIPE_DESC, value_len) == 0)) {
                ispecified = INVIZ_PKTYPE_WIPE;
            } else if ((value_len == strlen(INVIZ_ACCESS_DESC)) &&
                    (strncmp(argv[n]+parm_len, INVIZ_ACCESS_DESC, value_len) == 0)) {
                ispecified = INVIZ_PKTYPE_ACCESS;
            } else if ((value_len == strlen(INVIZ_COMMAND_DESC)) &&
                    (strncmp(argv[n]+parm_len, INVIZ_COMMAND_DESC, value_len) == 0)) {
                ispecified = INVIZ_PKTYPE_COMMAND;
            } else if ((value_len == strlen(INVIZ_INOTIFY_DESC)) &&
                    (strncmp(argv[n]+parm_len, INVIZ_INOTIFY_DESC, value_len) == 0)) {
                ispecified = INVIZ_PKTYPE_INOTIFY;
            } else if ((value_len == strlen(INVIZ_TRANSFER_DESC)) &&
                    (strncmp(argv[n]+parm_len, INVIZ_TRANSFER_DESC, value_len) == 0)) {
                ispecified = INVIZ_PKTYPE_TRANSFER;
            } else if ((value_len == strlen(INVIZ_BEACON_DESC)) &&
                    (strncmp(argv[n]+parm_len, INVIZ_BEACON_DESC, value_len) == 0)) {
                ispecified = INVIZ_PKTYPE_BEACON;
            } else if ((value_len == strlen(INVIZ_KEYLOG_DESC)) &&
                    (strncmp(argv[n]+parm_len, INVIZ_KEYLOG_DESC, value_len) == 0)) {
                ispecified = INVIZ_PKTYPE_KEYLOG;
            } else if ((value_len == strlen(INVIZ_PASSWORD_DESC)) &&
                    (strncmp(argv[n]+parm_len, INVIZ_PASSWORD_DESC, value_len) == 0)) {
                ispecified = INVIZ_PKTYPE_PASS;
            } else if ((value_len == strlen(INVIZ_PASSWORD_DESC_ALT1)) &&
                    (strncmp(argv[n]+parm_len, INVIZ_PASSWORD_DESC_ALT1, value_len) == 0)) {
                ispecified = INVIZ_PKTYPE_PASS;
            } else if ((value_len == strlen(INVIZ_PASSWORD_DESC_ALT2)) &&
                    (strncmp(argv[n]+parm_len, INVIZ_PASSWORD_DESC_ALT2, value_len) == 0)) {
                ispecified = INVIZ_PKTYPE_PASS;
            } else {
                ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            }
            set_pktype(ispecified); // validation done in here
            if ( ispecified != get_pktype() ) {
                sprintf(logmsg, "invalid %s parameter value %i ", pktype_parm, ispecified);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            continue;
        }
        // port knock digest parameter
        // client only - server calculates automatically
        parm_len = strlen(pkdigest_parm);
        if (strncmp((char *)argv[n], pkdigest_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            set_pkdigest((short)ispecified); // validation done in here
            if ( (short)ispecified != get_pkdigest() ) {
                sprintf(logmsg, "invalid %s parameter value %i ", pkdigest_parm, ispecified);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            continue;
        }
        // recv buffer len parameter (server only)
        parm_len = strlen(recvbuff_parm);
        if (strncmp((char *)argv[n], recvbuff_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < get_minrecvbufferlen() || ispecified > get_maxrecvbufferlen()) {
                sprintf(logmsg, "%s parameter must be between %i and %i",
                        (char *)recvbuff_parm, get_minrecvbufferlen(), get_maxrecvbufferlen());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            if (get_iconfirminterval() > 0) {
                sprintf(logmsg, "%s parameter value will be ignored because %s parameter is being used",
                        (char *)recvbuff_parm, confirminterval_parm);
                logit(_LOG_WARNING, logmsg, 0);
            } else {
                set_recvbufferlen(ispecified);
                set_recvbufferset(TRUE);
            }
            continue;
        }
        // pcap buffer len parameter
        parm_len = strlen(pcapbuff_parm);
        if (strncmp((char *)argv[n], pcapbuff_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < get_minpcapbufferlen() || ispecified > get_maxpcapbufferlen()) {
                sprintf(logmsg, "%s parameter must be between %i and %i",
                        (char *)pcapbuff_parm, get_minpcapbufferlen(), get_maxpcapbufferlen());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pcapbufferlen(ispecified);
            continue;
        }
        // pcap timeout parameter
        parm_len = strlen(pcaptimeout_parm);
        if (strncmp((char *)argv[n], pcaptimeout_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < get_minpcaptimeout() || ispecified > get_maxpcaptimeout()) {
                sprintf(logmsg, "%s parameter must be between %i and %i",
                        (char *)pcaptimeout_parm, get_minpcaptimeout(), get_maxpcaptimeout());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pcaptimeout(ispecified);
            continue;
        }
        // port knocking access timeout parameter
        parm_len = strlen(pktimeout_parm);
        if (strncmp((char *)argv[n], pktimeout_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < get_minpktimeout() || ispecified > get_maxpktimeout()) {
                sprintf(logmsg, "%s parameter must be between %i and %i",
                        (char *)pktimeout_parm, get_minpktimeout(), get_maxpktimeout());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pktimeout(ispecified);
            continue;
        }
        // duplicate packet interval (number of seconds)
        parm_len = strlen(dupinterval_parm);
        if (strncmp((char *)argv[n], dupinterval_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < 0) {
                sprintf(logmsg, "%s parameter must be >= 0", dupinterval_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_iduplicateinterval(ispecified);
            continue;
        }
        // confirmation interval (number of packets)
        // must match on both server and client
        parm_len = strlen(confirminterval_parm);
        if (strncmp((char *)argv[n], confirminterval_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < 0) {
                sprintf(logmsg, "%s parameter must be >= 0", confirminterval_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_iconfirminterval(ispecified);
            set_iwatchdog(get_iconfirminterval());
            if (get_recvbufferset()) {
                sprintf(logmsg, "%s parameter value will be ignored because %s parameter is being used",
                        (char *)recvbuff_parm, confirminterval_parm);
                logit(_LOG_WARNING, logmsg, 0);
            }
            continue;
        }
        // short version of confirmation interval (number of packets)
        // must match on both server and client
        parm_len = strlen(ci_parm);
        if (strncmp((char *)argv[n], ci_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < 0) {
                sprintf(logmsg, "%s parameter must be >= 0", ci_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_iconfirminterval(ispecified);
            set_iwatchdog(get_iconfirminterval());
            if (get_recvbufferset()) {
                sprintf(logmsg, "%s parameter value will be ignored because %s parameter is being used",
                        (char *)recvbuff_parm, ci_parm);
                logit(_LOG_WARNING, logmsg, 0);
            }
            continue;
        }
        // confirmation expiry (number of seconds)
        // for client only
        parm_len = strlen(confirmexpiry_parm);
        if (strncmp((char *)argv[n], confirmexpiry_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < get_minconfirmexpiry() || ispecified > get_maxconfirmexpiry()) {
                sprintf(logmsg, "%s parameter must be >= %i and <= %i", confirmexpiry_parm, get_minconfirmexpiry(), get_maxconfirmexpiry());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_iconfirmexpiry(ispecified);
            continue;
        }
        // short version of confirmation expiry (number of seconds)
        // for client only
        parm_len = strlen(ce_parm);
        if (strncmp((char *)argv[n], ce_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < get_minconfirmexpiry() || ispecified > get_maxconfirmexpiry()) {
                sprintf(logmsg, "%s parameter must be >= %i and <= %i", ce_parm, get_minconfirmexpiry(), get_maxconfirmexpiry());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_iconfirmexpiry(ispecified);
            continue;
        }
        // min sleep max parameter
        // allows override of built-in min
        // there is no override of built-in max
        parm_len = strlen(sleepmin_parm);
        if (strncmp((char *)argv[n], sleepmin_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < 0 || ispecified > get_maxsleepmax()) {
                sprintf(logmsg, "%s parameter must be between 0 and %i", sleepmin_parm, get_maxsleepmax());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_minsleepmax(ispecified);
            continue;
        }
        // sleep max parameter
        parm_len = strlen(sleepmax_parm);
        if (strncmp((char *)argv[n], sleepmax_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < get_minsleepmax() || ispecified > get_maxsleepmax()) {
                sprintf(logmsg, "%s parameter must be between %i and %i", sleepmax_parm, get_minsleepmax(), get_maxsleepmax());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_sleepmax(ispecified);
            continue;
        }
        // short version of sleep max parameter
        parm_len = strlen(sm_parm);
        if (strncmp((char *)argv[n], sm_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified < get_minsleepmax() || ispecified > get_maxsleepmax()) {
                sprintf(logmsg, "%s parameter must be between %i and %i", sm_parm, get_minsleepmax(), get_maxsleepmax());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_sleepmax(ispecified);
            continue;
        }
        // use tcp parameter
        parm_len = strlen(tcp_parm);
        if (strncmp((char *)argv[n], tcp_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified != 0 && ispecified != 1) {
                sprintf(logmsg, "%s parameter must be 0 or 1", tcp_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_usetcp(ispecified);
            continue;
        }
        // use raw sockets parameter
        parm_len = strlen(raw_parm);
        if (strncmp((char *)argv[n], raw_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified != FALSE && ispecified != TRUE) {
                sprintf(logmsg, "%s parameter must be 0 or 1", raw_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_useraw(ispecified);
            continue;
        }
        // use udp parameter
        parm_len = strlen(udp_parm);
        if (strncmp((char *)argv[n], udp_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified != 0 && ispecified != 1) {
                sprintf(logmsg, "%s parameter must be 0 or 1", udp_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_useudp(ispecified);
            continue;
        }
        // use icmp parameter
        parm_len = strlen(icmp_parm);
        if (strncmp((char *)argv[n], icmp_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified != 0 && ispecified != 1) {
                sprintf(logmsg, "%s parameter must be 0 or 1", icmp_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_useicmp(ispecified);
            continue;
        }
        // log file name to use
        parm_len = strlen(logfilename_parm);
        if (strncmp((char *)argv[n], logfilename_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > MAX_FILENAME_LEN) {
                sprintf(logmsg, "max log file name length %i exceeded", MAX_FILENAME_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_logfilename(argv[n]+parm_len, ilen);
            set_uselogfile(TRUE);
            continue;
        }
        // proxy address to use (for client)
        parm_len = strlen(proxy_parm);
        if (strncmp((char *)argv[n], proxy_parm, parm_len) == 0) {
            int maxiplen = (int)MAX_IP4_LEN;
            if (get_useip6()) {
                maxiplen = (int)MAX_IP6_LEN;
            }
            ilen = strlen( argv[n]+parm_len );
            if (ilen > maxiplen) {
                sprintf(logmsg, "max proxy address length %i exceeded", maxiplen);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_proxy(argv[n]+parm_len, ilen);
            continue;
        }
        // proxy type to use (for client)
        parm_len = strlen(proxytype_parm);
        if (strncmp((char *)argv[n], proxytype_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PROXYTYPE_LEN) {
                sprintf(logmsg, "max proxy type length %i exceeded", (int)INVIZ_MAX_PROXYTYPE_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_proxytype(argv[n]+parm_len, ilen);
            continue;
        }
        // user-agent to use (for client)
        parm_len = strlen(ua_parm);
        if (strncmp((char *)argv[n], ua_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > INVIZ_MAX_UA_LEN) {
                sprintf(logmsg, "max user agent length %i exceeded", (int)INVIZ_MAX_UA_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_useragent(argv[n]+parm_len, ilen);
            continue;
        }
        // pksavefilename to save pk packets (client only)
        parm_len = strlen(pksavefilename_parm);
        if (strncmp((char *)argv[n], pksavefilename_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_FILENAME_LEN) {
                sprintf(logmsg, "max pksavefilename length %i exceeded", (int)MAX_FILENAME_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pksavefilename(argv[n]+parm_len, ilen);
            continue;
        }
        // use pkaccess file (server only)
        parm_len = strlen(pkaccessfile_parm);
        if (strncmp((char *)argv[n], pkaccessfile_parm, parm_len) == 0) {
            ispecified = (int)strtol(argv[n]+parm_len, NULL, 10);
            if (ispecified != FALSE && ispecified != TRUE) {
                sprintf(logmsg, "%s parameter must be 0 or 1", pkaccessfile_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_usepkaccessfile(ispecified);
            continue;
        }
        // pkaccessfilename (server only)
        // if using non-default access filename
        parm_len = strlen(pkaccessfilename_parm);
        if (strncmp((char *)argv[n], pkaccessfilename_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_FILENAME_LEN) {
                sprintf(logmsg, "max pkaccessfilename length %i exceeded", (int)MAX_FILENAME_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkaccessfilename(argv[n]+parm_len, ilen);
            continue;
        }
        // pkconfigfilename (server only)
        parm_len = strlen(pkconfigfilename_parm);
        if (strncmp((char *)argv[n], pkconfigfilename_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_FILENAME_LEN) {
                sprintf(logmsg, "max pkconfigfilename length %i exceeded", (int)MAX_FILENAME_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkconfigfilename(argv[n]+parm_len, ilen);
            continue;
        }
        // filename to be uploaded to server (or received from client)
        parm_len = strlen(filename_parm);
        if (strncmp((char *)argv[n], filename_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_FILENAME_LEN) {
                sprintf(logmsg, "max filename length %i exceeded", (int)MAX_FILENAME_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_filename(argv[n]+parm_len, ilen);
            continue;
        }
        // dev random filename
        parm_len = strlen(devrandom_parm);
        if (strncmp((char *)argv[n], devrandom_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_FILENAME_LEN) {
                sprintf(logmsg, "dev random length %i exceeded", (int)MAX_FILENAME_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_devrandom(argv[n]+parm_len, ilen);
            continue;
        }
        // short version of filename to be uploaded to server (or received from client)
        parm_len = strlen(f_parm);
        if (strncmp((char *)argv[n], f_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_FILENAME_LEN) {
                sprintf(logmsg, "max filename length %i exceeded", (int)MAX_FILENAME_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_filename(argv[n]+parm_len, ilen);
            continue;
        }
        // short version of filename/remove to be uploaded to server (from client)
        parm_len = strlen(frm_parm);
        if (strncmp((char *)argv[n], frm_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_FILENAME_LEN) {
                sprintf(logmsg, "max filename length %i exceeded", (int)MAX_FILENAME_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_filename(argv[n]+parm_len, ilen);
            set_remove(TRUE);
            continue;
        }
        // short version of filename/append to be received by server
        parm_len = strlen(fa_parm);
        if (strncmp((char *)argv[n], fa_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_FILENAME_LEN) {
                sprintf(logmsg, "max filename length %i exceeded", (int)MAX_FILENAME_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_filename(argv[n]+parm_len, ilen);
            set_append(TRUE);
            continue;
        }
        // short version of filename/replace to be received by server
        parm_len = strlen(fre_parm);
        if (strncmp((char *)argv[n], fre_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_FILENAME_LEN) {
                sprintf(logmsg, "max filename length %i exceeded", (int)MAX_FILENAME_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_filename(argv[n]+parm_len, ilen);
            set_replace(TRUE);
            continue;
        }
        // key seed to be used
        parm_len = strlen(keyseed_parm);
        if (strncmp((char *)argv[n], keyseed_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_KEYSEED_LEN) {
                sprintf(logmsg, "max key length %i exceeded", (int)INVIZ_MAX_KEYSEED_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_keyseed(argv[n]+parm_len, ilen);
            continue;
        }
        // port knocking server command to be executed
        parm_len = strlen(pkcommand_parm);
        if (strncmp((char *)argv[n], pkcommand_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PKCOMMAND_LEN) {
                sprintf(logmsg, "max port knocking command length %i exceeded",
                        (int)INVIZ_MAX_PKCOMMAND_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkservercommand(argv[n]+parm_len, ilen);
            // no need to specify --pk=2 or --pk=command if --pkcommand= specified
            set_pktype(INVIZ_PKTYPE_COMMAND);
            continue;
        }
        // port knocking beacon command to be executed
        parm_len = strlen(pkbeaconcommand_parm);
        if (strncmp((char *)argv[n], pkbeaconcommand_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PKCOMMAND_LEN) {
                sprintf(logmsg, "max port knocking beacon length %i exceeded",
                        (int)INVIZ_MAX_PKCOMMAND_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkbeaconcommand(argv[n]+parm_len, ilen);
            // no need to specify --pk=8 or --pk=beacon if --pkbeacon= specified
            set_pktype(INVIZ_PKTYPE_BEACON);
            continue;
        }
        // port knocking filename to be transferred from server
        parm_len = strlen(pktransfercommand_parm);
        if (strncmp((char *)argv[n], pktransfercommand_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_FILENAME_LEN) {
                sprintf(logmsg, "max pk transfer filename length %i exceeded",
                        (int)MAX_FILENAME_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pktransfercommand(argv[n]+parm_len, ilen);
            // no need to specify --pk=3 or --pk=transfer if --pktransfer= specified
            set_pktype(INVIZ_PKTYPE_TRANSFER);
            continue;
        }
        // port knocking inotify command to be executed on server
        parm_len = strlen(pkinotifycommand_parm);
        if (strncmp((char *)argv[n], pkinotifycommand_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PKCOMMAND_LEN) {
                sprintf(logmsg, "max pk inotify command length %i exceeded",
                        (int)INVIZ_MAX_PKCOMMAND_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkinotifycommand(argv[n]+parm_len, ilen);
            // no need to specify --pk=4 or --pk=inotify if --pkinotify= specified
            set_pktype(INVIZ_PKTYPE_INOTIFY);
            continue;
        }
        // port knocking wipe command to be executed on server
        parm_len = strlen(pkwipecommand_parm);
        if (strncmp((char *)argv[n], pkwipecommand_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PKCOMMAND_LEN) {
                sprintf(logmsg, "max pk wipe command length %i exceeded",
                        (int)INVIZ_MAX_PKCOMMAND_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkwipecommand(argv[n]+parm_len, ilen);
            // no need to specify --pk=6 or --pk=wipe if --pkwipe= specified
            set_pktype(INVIZ_PKTYPE_WIPE);
            continue;
        }
        // port knocking password command to be executed on server
        parm_len = strlen(pkpasscommand_parm);
        if (strncmp((char *)argv[n], pkpasscommand_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PKCOMMAND_LEN) {
                sprintf(logmsg, "max pk password command length %i exceeded",
                        (int)INVIZ_MAX_PKCOMMAND_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkpasscommand(argv[n]+parm_len, ilen);
            // no need to specify --pk=7 or --pk=pass if --pkpass= specified
            set_pktype(INVIZ_PKTYPE_PASS);
            continue;
        }
        // port knocking key logging command to be executed on server
        parm_len = strlen(pkkeylogcommand_parm);
        if (strncmp((char *)argv[n], pkkeylogcommand_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PKCOMMAND_LEN) {
                sprintf(logmsg, "max pk key logging command length %i exceeded",
                        (int)INVIZ_MAX_PKCOMMAND_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkkeylogcommand(argv[n]+parm_len, ilen);
            // no need to specify --pk=5 or --pk=keylog if --pkkeylog= specified
            set_pktype(INVIZ_PKTYPE_KEYLOG);
            continue;
        }
        // port knocking password
        parm_len = strlen(pkpassword_parm);
        if (strncmp((char *)argv[n], pkpassword_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PASSWORD_LEN) {
                sprintf(logmsg, "max %s password length %i exceeded",
                        pkpassword_parm,
                        (int)INVIZ_MAX_PASSWORD_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkpassword(argv[n]+parm_len, ilen);
            continue;
        }
        // port knocking password alternate
        parm_len = strlen(pkpasswordalt1_parm);
        if (strncmp((char *)argv[n], pkpasswordalt1_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PASSWORD_LEN) {
                sprintf(logmsg, "max %s password length %i exceeded",
                        pkpasswordalt1_parm,
                        (int)INVIZ_MAX_PASSWORD_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkpassword(argv[n]+parm_len, ilen);
            continue;
        }
        // port knocking password alternate
        parm_len = strlen(pkpasswordalt2_parm);
        if (strncmp((char *)argv[n], pkpasswordalt2_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PASSWORD_LEN) {
                sprintf(logmsg, "max %s password length %i exceeded",
                        pkpasswordalt2_parm,
                        (int)INVIZ_MAX_PASSWORD_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkpassword(argv[n]+parm_len, ilen);
            continue;
        }
        // gpg recipient key
        parm_len = strlen(gpgrecipientkey_parm);
        if (strncmp((char *)argv[n], gpgrecipientkey_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_GPGKEY_LEN) {
                sprintf(logmsg, "GPG recipient key length %i exceeded",
                        (int)INVIZ_MAX_GPGKEY_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_gpg_recipientkey(argv[n]+parm_len, ilen);
            set_usegpg(TRUE);
            continue;
        }
        // gpg signer key
        parm_len = strlen(gpgsignerkey_parm);
        if (strncmp((char *)argv[n], gpgsignerkey_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_GPGKEY_LEN) {
                sprintf(logmsg, "GPG signer key length %i exceeded",
                        (int)INVIZ_MAX_GPGKEY_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_gpg_signerkey(argv[n]+parm_len, ilen);
            set_usegpg(TRUE);
            continue;
        }
        // gpg home directory
        parm_len = strlen(gpghomedir_parm);
        if (strncmp((char *)argv[n], gpghomedir_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PATH_LEN) {
                sprintf(logmsg, "GPG home dir path length %i exceeded",
                        (int)INVIZ_MAX_PATH_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_gpg_homedir(argv[n]+parm_len, ilen);
            set_usegpg(TRUE);
            continue;
        }
        // gpg key file
        parm_len = strlen(gpgkeyfile_parm);
        if (strncmp((char *)argv[n], gpgkeyfile_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PATH_LEN) {
                sprintf(logmsg, "GPG key file path length %i exceeded",
                        (int)INVIZ_MAX_PATH_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_gpg_keyfile(argv[n]+parm_len, ilen);
            set_usegpg(TRUE);
            continue;
        }
        // port knocking access request to be sent to server
        parm_len = strlen(pkaccess_parm);
        if (strncmp((char *)argv[n], pkaccess_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PKACCESS_LEN) {
                sprintf(logmsg, "max port knocking access request length %i exceeded",
                        (int)INVIZ_MAX_PKACCESS_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkaccessrequest(argv[n]+parm_len, ilen);
            // no need to specify --pk=1 or --pk=access if --pkaccess= specified
            set_pktype(INVIZ_PKTYPE_ACCESS);
            continue;
        }
        // nat access request (to be sent to server?)
        parm_len = strlen(nataccess_parm);
        if (strncmp((char *)argv[n], nataccess_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PKACCESS_LEN) {
                sprintf(logmsg, "max nat access request length %i exceeded",
                        (int)INVIZ_MAX_PKACCESS_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_nataccessrequest(argv[n]+parm_len, ilen);
            continue;
        }
        // ps mask to be used
        parm_len = strlen(psmask_parm);
        if (strncmp((char *)argv[n], psmask_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)INVIZ_MAX_PSMASK_LEN) {
                sprintf(logmsg, "max psmask length %i exceeded",
                        (int)INVIZ_MAX_PSMASK_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            if (strncmp((char *)argv[n]+parm_len, INVIZ_RANDOM_DESC, strlen(INVIZ_RANDOM_DESC)) == 0) {
                // --psmask=random
                set_randompsmask();
                set_randomizepsmask(TRUE);
            } else {
                set_psmask(argv[n]+parm_len, ilen);
                set_randomizepsmask(FALSE);
            }
            continue;
        }
        // server ip4 address
        parm_len = strlen(serverip4_parm);
        if (strncmp((char *)argv[n], serverip4_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_IP4_LEN) {
                sprintf(logmsg, "max server ip4 length %i exceeded", MAX_IP4_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_serverip4(argv[n]+parm_len, ilen);
            continue;
        }
        // pk access ip4 address
        parm_len = strlen(pkallowip4_parm);
        if (strncmp((char *)argv[n], pkallowip4_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_IP4_LEN) {
                sprintf(logmsg, "max port knocking allow ip4 length %i exceeded", MAX_IP4_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkallowip4(argv[n]+parm_len, ilen);
            continue;
        }
        // spoof ip4 address
        parm_len = strlen(spoofip4_parm);
        if (strncmp((char *)argv[n], spoofip4_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_IP4_LEN) {
                sprintf(logmsg, "max spoof ip4 length %i exceeded", MAX_IP4_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_spoofip4(argv[n]+parm_len, ilen);
            continue;
        }
        // client ip4 address
        parm_len = strlen(clientip4_parm);
        if (strncmp((char *)argv[n], clientip4_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_IP4_LEN) {
                sprintf(logmsg, "max client ip4 length %i exceeded", MAX_IP4_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_clientip4(argv[n]+parm_len, ilen);
            continue;
        }
        // external ip4 address
        parm_len = strlen(externalip4_parm);
        if (strncmp((char *)argv[n], externalip4_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_IP4_LEN) {
                sprintf(logmsg, "max external/public ip4 length %i exceeded", MAX_IP4_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_externalip4(argv[n]+parm_len, ilen);
            continue;
        }
        // external address
        parm_len = strlen(externalip_parm);
        if (strncmp((char *)argv[n], externalip_parm, parm_len) == 0) {
            if (get_useip4()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP4_LEN) {
                    sprintf(logmsg, "max external/public ip4 length %i exceeded", MAX_IP4_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_externalip4(argv[n]+parm_len, ilen);
            } else if (get_useip6()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP6_LEN) {
                    sprintf(logmsg, "max external/public ip6 length %i exceeded", MAX_IP6_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_externalip6(argv[n]+parm_len, ilen);
            }
            continue;
        }
        // server address
        parm_len = strlen(serverip_parm);
        if (strncmp((char *)argv[n], serverip_parm, parm_len) == 0) {
            if (get_useip4()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP4_LEN) {
                    sprintf(logmsg, "max server ip4 length %i exceeded", MAX_IP4_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_serverip4(argv[n]+parm_len, ilen);
            } else if (get_useip6()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP6_LEN) {
                    sprintf(logmsg, "max server ip6 length %i exceeded", MAX_IP6_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_serverip6(argv[n]+parm_len, ilen);
            }
            continue;
        }
        // port knocking access address
        parm_len = strlen(pkallowip_parm);
        if (strncmp((char *)argv[n], pkallowip_parm, parm_len) == 0) {
            if (get_useip4()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP4_LEN) {
                    sprintf(logmsg, "max port knocking allow ip4 length %i exceeded", MAX_IP4_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_pkallowip4(argv[n]+parm_len, ilen);
            } else if (get_useip6()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP6_LEN) {
                    sprintf(logmsg, "max port knocking allow ip6 length %i exceeded", MAX_IP6_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_pkallowip6(argv[n]+parm_len, ilen);
            }
            continue;
        }
        // client address
        parm_len = strlen(clientip_parm);
        if (strncmp((char *)argv[n], clientip_parm, parm_len) == 0) {
            if (get_useip4()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP4_LEN) {
                    sprintf(logmsg, "max client ip4 length %i exceeded", MAX_IP4_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_clientip4(argv[n]+parm_len, ilen);
            } else if (get_useip6()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP6_LEN) {
                    sprintf(logmsg, "max client ip6 length %i exceeded", MAX_IP6_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_clientip6(argv[n]+parm_len, ilen);
            }
            continue;
        }
        // short version of server address
        parm_len = strlen(sip_parm);
        if (strncmp((char *)argv[n], sip_parm, parm_len) == 0) {
            if (get_useip4()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP4_LEN) {
                    sprintf(logmsg, "max server ip4 length %i exceeded", MAX_IP4_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_serverip4(argv[n]+parm_len, ilen);
            } else if (get_useip6()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP6_LEN) {
                    sprintf(logmsg, "max server ip6 length %i exceeded", MAX_IP6_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_serverip6(argv[n]+parm_len, ilen);
            }
            continue;
        }
        // short version of client address
        parm_len = strlen(cip_parm);
        if (strncmp((char *)argv[n], cip_parm, parm_len) == 0) {
            if (get_useip4()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP4_LEN) {
                    sprintf(logmsg, "max client ip4 length %i exceeded", MAX_IP4_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_clientip4(argv[n]+parm_len, ilen);
            } else if (get_useip6()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP6_LEN) {
                    sprintf(logmsg, "max client ip6 length %i exceeded", MAX_IP6_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_clientip6(argv[n]+parm_len, ilen);
            }
            continue;
        }
        // dns server ip4 address
        parm_len = strlen(dnsip4_parm);
        if (strncmp((char *)argv[n], dnsip4_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_IP4_LEN) {
                sprintf(logmsg, "max dns server ip4 length %i exceeded", MAX_IP4_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_dnsip4(argv[n]+parm_len, ilen);
            continue;
        }
        // server ip6 address
        parm_len = strlen(serverip6_parm);
        if (strncmp((char *)argv[n], serverip6_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_IP6_LEN) {
                sprintf(logmsg, "max server ip6 length %i exceeded", MAX_IP6_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_serverip6(argv[n]+parm_len, ilen);
            continue;
        }
        // port knocking access ip6 address
        parm_len = strlen(pkallowip6_parm);
        if (strncmp((char *)argv[n], pkallowip6_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_IP6_LEN) {
                sprintf(logmsg, "max port knocking allow ip6 length %i exceeded", MAX_IP6_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_pkallowip6(argv[n]+parm_len, ilen);
            continue;
        }
        // spoof ip6 address
        parm_len = strlen(spoofip6_parm);
        if (strncmp((char *)argv[n], spoofip6_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_IP6_LEN) {
                sprintf(logmsg, "max spoof ip6 length %i exceeded", MAX_IP6_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_spoofip6(argv[n]+parm_len, ilen);
            continue;
        }
        // client ip6 address
        parm_len = strlen(clientip6_parm);
        if (strncmp((char *)argv[n], clientip6_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_IP6_LEN) {
                sprintf(logmsg, "max client ip6 length %i exceeded", MAX_IP6_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_clientip6(argv[n]+parm_len, ilen);
            continue;
        }
        // server dns ip6 address
        parm_len = strlen(dnsip6_parm);
        if (strncmp((char *)argv[n], dnsip6_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen > (int)MAX_IP6_LEN) {
                sprintf(logmsg, "max dns server ip6 length %i exceeded", MAX_IP6_LEN);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_dnsip6(argv[n]+parm_len, ilen);
            continue;
        }
        // dns server address
        parm_len = strlen(dnsip_parm);
        if (strncmp((char *)argv[n], dnsip_parm, parm_len) == 0) {
            if (get_useip4()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP4_LEN) {
                    sprintf(logmsg, "max dns server ip4 length %i exceeded", MAX_IP4_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_dnsip4(argv[n]+parm_len, ilen);
            } else if (get_useip6()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP6_LEN) {
                    sprintf(logmsg, "max dns server ip6 length %i exceeded", MAX_IP6_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_dnsip6(argv[n]+parm_len, ilen);
            }
            continue;
        }
        // spoof address
        parm_len = strlen(spoofip_parm);
        if (strncmp((char *)argv[n], spoofip_parm, parm_len) == 0) {
            if (get_useip4()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP4_LEN) {
                    sprintf(logmsg, "max spoof ip4 length %i exceeded", MAX_IP4_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_spoofip4(argv[n]+parm_len, ilen);
            } else if (get_useip6()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP6_LEN) {
                    sprintf(logmsg, "max spoof ip6 length %i exceeded", MAX_IP6_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_spoofip6(argv[n]+parm_len, ilen);
            }
            continue;
        }
        // short version of dns server address
        parm_len = strlen(dip_parm);
        if (strncmp((char *)argv[n], dip_parm, parm_len) == 0) {
            if (get_useip4()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP4_LEN) {
                    sprintf(logmsg, "max dns server ip4 length %i exceeded", MAX_IP4_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_dnsip4(argv[n]+parm_len, ilen);
            } else if (get_useip6()) {
                ilen = strlen( argv[n]+parm_len );
                if (ilen > (int)MAX_IP6_LEN) {
                    sprintf(logmsg, "max dns server ip6 length %i exceeded", MAX_IP6_LEN);
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                set_dnsip6(argv[n]+parm_len, ilen);
            }
            continue;
        }
        // server or client mode
        parm_len = strlen(mode_parm);
        if (strncmp((char *)argv[n], mode_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if ( strncmp(argv[n]+parm_len, INVIZ_LISTENER_DESC, strlen(INVIZ_LISTENER_DESC)) == 0 ) {
                set_mode(INVIZ_MODE_LISTENER);
            } else if ( strncmp(argv[n]+parm_len, INVIZ_SENDER_DESC, strlen(INVIZ_SENDER_DESC)) == 0 ) {
                set_mode(INVIZ_MODE_SENDER);
                set_interactive(TRUE);
            } else {
                sprintf(logmsg, "%s parameter must be either %s or %s",
                        mode_parm,
                        INVIZ_LISTENER_DESC,
                        INVIZ_SENDER_DESC);
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            continue;
        }
        // interface name
        parm_len = strlen(ifname_parm);
        if (strncmp((char *)argv[n], ifname_parm, parm_len) == 0) {
            ilen = strlen( argv[n]+parm_len );
            if (ilen < get_minifnamelen() || ilen > get_maxifnamelen()) {
                sprintf(logmsg, "interface name length must be between %i and %i",
                        get_minifnamelen(), get_maxifnamelen());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
            set_ifname(argv[n]+parm_len, ilen);
            continue;
        }
        sprintf(logmsg, "parameter %s ignored", (char *)argv[n]);
        logit(_LOG_WARNING, logmsg, 0);
    }
    //
    // now validate parameters
    //
    char * used_protocol = malloc(MAX_PROTOCOL_DESC_LEN+1);
    if (used_protocol == NULL) {
        sprintf(logmsg, "unable to malloc used_protocol [%s]", strerror(errno));
        logit(_LOG_ERROR, logmsg, 0);
        return ( EXIT_FAILURE );
    } else {
        strcpy(used_protocol, get_usedprotocoldesc(get_usedprotocol()));
    }
    if (strcmp(used_protocol, INVIZ_PROTOCOL_UNKNOWN_DESC) == 0) {
        sprintf(logmsg, "unknown protocol to use (%d)", get_usedprotocol());
        logit(_LOG_ERROR, logmsg, 0);
        return ( EXIT_FAILURE );
    }
    if ( (strcmp(used_protocol, IPPROTO_SCTP_DESC) == 0) ||
            (strcmp(used_protocol, IPPROTO_ICMP_DESC) == 0) ||
            (strcmp(used_protocol, IPPROTO_PROXY_DESC) == 0) ||
            (strcmp(used_protocol, IPPROTO_IP_DESC) == 0) ) {
        sprintf(logmsg, "%d protocol not fully supported as yet", get_usedprotocol());
        logit(_LOG_ERROR, logmsg, 0);
        return ( EXIT_FAILURE );
    }
    set_protocoldesc(used_protocol, strlen(used_protocol));
    free(used_protocol);
    //
    if (get_mode() == INVIZ_MODE_SENDER) { // this is client side
        if (get_verbose()) {
            logit(_LOG_INFO, "set to run in sending mode", 0);
        }
        if ( (strlen(get_filename()) <= 0) && !get_usepk() ) {
            //
            // if doing port knocking, then file name is not required
            //
            sprintf(logmsg, "file name (to be sent) must be specified with %s parameter", filename_parm);
            logit(_LOG_ERROR, logmsg, 0);
            return ( EXIT_FAILURE );
        } else if (strlen(get_filename()) > 0) {
            pFile = fopen((char *)get_filename(), "r");
            if (pFile == NULL) {
                sprintf(logmsg, "unable to open file (%s) to be sent [%s]", get_filename(), strerror(errno));
                logit(_LOG_ERROR, logmsg, 0);
                return ( EXIT_FAILURE );
            }
            fseek(pFile, 0, SEEK_END);
            if (ferror(pFile)) {
                sprintf(logmsg, "unable to read file (%s) to be sent [%s]", get_filename(), strerror(errno));
                logit(_LOG_ERROR, logmsg, 0);
                fclose(pFile);
                return ( EXIT_FAILURE );
            } else {
                long lfilesize = ftell(pFile);
                set_filesize(lfilesize);
                set_filedone(0);
            }
            fclose(pFile);
        }
        if (get_useip4()) {
            if (strlen(get_serverip4()) <= 0) {
                sprintf(logmsg, "server ip4 address must be specified with %s parameter",
                        serverip4_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return ( EXIT_FAILURE );
            } else if (!get_usepk()) {
                sprintf(logmsg, "will send file (%s) to server at %s with seed=%i, key=%s",
                        get_filename(),
                        get_serverip4(),
                        get_iseed(),
                        (char *)get_keyseed());
                logit(_LOG_INFO, logmsg, 0);
            }
        } else if (get_useip6()) {
            if (strlen(get_serverip6()) <= 0) {
                sprintf(logmsg, "server ip6 address must be specified with %s parameter",
                        serverip6_parm);
                logit(_LOG_ERROR, logmsg, 0);
                return ( EXIT_FAILURE );
            } else if (!get_usepk()) {
                sprintf(logmsg, "will send file (%s) to server at %s with seed=%i, key=%s",
                        get_filename(),
                        get_serverip6(),
                        get_iseed(),
                        (char *)get_keyseed());
                logit(_LOG_INFO, logmsg, 0);
            }
        }
        if (get_usepk()) {
            if ( get_pkpassword() && strlen(get_pkpassword()) == strlen(INVIZ_DEFAULT_DESC) &&
                    (strncmp(get_pkpassword(), INVIZ_DEFAULT_DESC, strlen(get_pkpassword())) == 0) ) {
                //
                // if client/sending side started up with --pwd=default then
                // default password is automatically set
                // if no --pwd= specified then must be interactively supplied
                // by user when needed
                // note that --pwd= by itself will set password to empty string
                // which is allowed (although maybe should not be allowed)
                //
                set_pkpassword(INVIZ_DEFAULT_PASSWORD"+"INVIZ_MAJOR_VERSION,
                        strlen(INVIZ_DEFAULT_PASSWORD)+1+strlen(INVIZ_MAJOR_VERSION));
                sprintf(logmsg, "default password now set");
                logit(_LOG_INFO, logmsg, 0);
            }
            if (get_pktype() == (int)INVIZ_PKTYPE_ACCESS) {
                if (!get_pkaccessrequest() || (strlen(get_pkaccessrequest()) <= 0)) {
                    set_pkaccessrequest(INVIZ_DEFAULT_PKACCESS, strlen(INVIZ_DEFAULT_PKACCESS));
                }
                sprintf(logmsg, "port knocking will send access request to %s:%i over %s",
                    (char *)get_serverip(),
                    get_pkport(),
                    (char *)get_protocoldesc());
                logit(_LOG_INFO, logmsg, 0);
            } else if (get_pktype() == INVIZ_PKTYPE_COMMAND) {
                if (!get_pkservercommand() || (strlen(get_pkservercommand()) <= 0)) {
                    set_pkservercommand(INVIZ_DEFAULT_PKCOMMAND, strlen(INVIZ_DEFAULT_PKCOMMAND));
                }
                sprintf(logmsg, "port knocking will send server command to %s:%i over %s",
                    (char *)get_serverip(),
                    get_pkport(),
                    (char *)get_protocoldesc());
                logit(_LOG_INFO, logmsg, 0);
            } else if (get_pktype() == INVIZ_PKTYPE_BEACON) {
                if (!get_pkbeaconcommand() || (strlen(get_pkbeaconcommand()) <= 0)) {
                    set_pkbeaconcommand(INVIZ_DEFAULT_PKBEACON, strlen(INVIZ_DEFAULT_PKBEACON));
                }
                sprintf(logmsg, "port knocking will send beacon to %s:%i over %s",
                    (char *)get_serverip(),
                    get_pkport(),
                    (char *)get_protocoldesc());
                logit(_LOG_INFO, logmsg, 0);
            } else if (get_pktype() == INVIZ_PKTYPE_TRANSFER) {
                if (!get_pktransfercommand() || (strlen(get_pktransfercommand()) <= 0)) {
                    sprintf(logmsg, "file name to be transferred must be specified with %s parameter",
                            (char *)get_pktransferparm());
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                sprintf(logmsg, "port knocking will send file transfer request to %s:%i over %s",
                    (char *)get_serverip(),
                    get_pkport(),
                    (char *)get_protocoldesc());
                logit(_LOG_INFO, logmsg, 0);
            } else if (get_pktype() == INVIZ_PKTYPE_INOTIFY) {
                if (!get_pkinotifycommand() || (strlen(get_pkinotifycommand()) <= 0)) {
                    sprintf(logmsg, "inotify command must be specified with %s parameter",
                            (char *)get_pkinotifyparm());
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                sprintf(logmsg, "port knocking will send inotify command to %s:%i over %s",
                    (char *)get_serverip(),
                    get_pkport(),
                    (char *)get_protocoldesc());
                logit(_LOG_INFO, logmsg, 0);
            } else if (get_pktype() == INVIZ_PKTYPE_WIPE) {
                if (!get_pkwipecommand() || (strlen(get_pkwipecommand()) <= 0)) {
                    set_pkwipecommand(INVIZ_DEFAULT_PKWIPE, strlen(INVIZ_DEFAULT_PKWIPE));
                }
                sprintf(logmsg, "port knocking will send wipe command to %s:%i over %s",
                    (char *)get_serverip(),
                    get_pkport(),
                    (char *)get_protocoldesc());
                logit(_LOG_INFO, logmsg, 0);
            } else if (get_pktype() == INVIZ_PKTYPE_KEYLOG) {
                if (!get_pkkeylogcommand() || (strlen(get_pkkeylogcommand()) <= 0)) {
                    sprintf(logmsg, "key logging command must be specified with %s parameter",
                            (char *)get_pkkeylogparm());
                    logit(_LOG_ERROR, logmsg, 0);
                    return (EXIT_FAILURE);
                }
                sprintf(logmsg, "port knocking will send key logging command to %s:%i over %s",
                    (char *)get_serverip(),
                    get_pkport(),
                    (char *)get_protocoldesc());
                logit(_LOG_INFO, logmsg, 0);
            } else if (get_pktype() == INVIZ_PKTYPE_PASS) {
                if (!get_pkpasscommand() ||
                        (strlen(get_pkpasscommand()) <= 0) ||
                        (strncmp(get_pkpasscommand(), INVIZ_INTERACTIVE_DESC, strlen(get_pkpasscommand())) == 0)) {
                    //
                    // get user to enter password now
                    // to be sent to listener/server
                    //
                    char spa_encryption_key[INVIZ_MAX_PASSWORD_LEN+1];
                    memset(spa_encryption_key, 0x0, INVIZ_MAX_PASSWORD_LEN+1);
                    while (!sig_interrupted && strlen(spa_encryption_key) == 0) {
                        strncpy(spa_encryption_key, get_user_pw_inviz(-1), INVIZ_MAX_PASSWORD_LEN);
                    }
                    set_pkpasscommand(spa_encryption_key, strlen(spa_encryption_key));
                }
                if (!get_pkpassword() || (strlen(get_pkpassword()) <= 0)) {
                    //
                    // password command specified
                    // but password not
                    //
                    if (!get_pkpassword() || (strlen(get_pkpassword()) <= 0)) {
                        set_pkpassword(INVIZ_DEFAULT_PASSWORD"+"INVIZ_MAJOR_VERSION,
                                strlen(INVIZ_DEFAULT_PASSWORD)+1+strlen(INVIZ_MAJOR_VERSION));
                        sprintf(logmsg, "default password on listener presumed");
                        logit(_LOG_INFO, logmsg, 0);
                    }
                }
                sprintf(logmsg, "port knocking will send password command to %s:%i over %s",
                    (char *)get_serverip(),
                    get_pkport(),
                    (char *)get_protocoldesc());
                logit(_LOG_INFO, logmsg, 0);
            } else {
                sprintf(logmsg, "unrecognized or unsupported port knocking type %d",
                        get_pktype());
                logit(_LOG_ERROR, logmsg, 0);
                return (EXIT_FAILURE);
            }
        }
        if (get_usetcp() && get_useudp()) {
            logit(_LOG_ERROR, "cannot use both "IPPROTO_TCP_DESC" and "IPPROTO_UDP_DESC" protocols", 0);
            return (EXIT_FAILURE);
        }
        if (get_usetcp() && get_useicmp()) {
            logit(_LOG_ERROR, "cannot use both "IPPROTO_TCP_DESC" and "IPPROTO_ICMP_DESC" protocols", 0);
            return (EXIT_FAILURE);
        }
        if (get_useudp() && get_useicmp()) {
            logit(_LOG_ERROR, "cannot use both "IPPROTO_UDP_DESC" and "IPPROTO_ICMP_DESC" protocols", 0);
            return (EXIT_FAILURE);
        }
    } else if (get_mode() == INVIZ_MODE_LISTENER) { // this is server side
        if (get_verbose()) {
            logit(_LOG_INFO, "set to run in listening mode", 0);
        }
        if (get_usepk()) {
            if (!get_pkpassword()) {
                //
                // listening/server side can be started up without --pwd= specified
                // and still have a password that is known to the sending/client side
                //
                set_pkpassword(INVIZ_DEFAULT_PASSWORD"+"INVIZ_MAJOR_VERSION,
                        strlen(INVIZ_DEFAULT_PASSWORD)+1+strlen(INVIZ_MAJOR_VERSION));
                sprintf(logmsg, "default password set");
                logit(_LOG_INFO, logmsg, 0);
            }
            //
            if (set_pklistener_defaults() != EXIT_SUCCESS) {
                return ( EXIT_FAILURE );
            }
            //
            if (get_pktype() == INVIZ_PKTYPE_ACCESS) {
                sprintf(logmsg, "listening for incoming port knocking access requests");
            } else if (get_pktype() == INVIZ_PKTYPE_COMMAND) {
                sprintf(logmsg, "listening for incoming port knocking remote execution commands");
            } else if (get_pktype() == INVIZ_PKTYPE_TRANSFER) {
                sprintf(logmsg, "listening for incoming port knocking file transfer requests");
            } else if (get_pktype() == INVIZ_PKTYPE_INOTIFY) {
                sprintf(logmsg, "listening for incoming port knocking inotify commands");
            } else if (get_pktype() == INVIZ_PKTYPE_KEYLOG) {
                sprintf(logmsg, "listening for incoming port knocking key logging commands");
            } else if (get_pktype() == INVIZ_PKTYPE_PASS) {
                sprintf(logmsg, "listening for incoming port knocking password commands");
            } else if (get_pktype() == INVIZ_PKTYPE_WIPE) {
                sprintf(logmsg, "listening for incoming port knocking wipe commands");
            } else if (get_pktype() == INVIZ_PKTYPE_ANY) {
                sprintf(logmsg, "listening for any incoming port knocking packets");
            } else {
                sprintf(logmsg, "unrecognized port knocking type (%i)", get_pktype());
                logit(_LOG_ERROR, logmsg, 0);
                return ( EXIT_FAILURE );
            }
            if (get_verbose()) {
                logit(_LOG_INFO, logmsg, 0);
            }
        } else {
            if (!get_filename() || strlen(get_filename()) <= 0) {
                set_filename(INVIZ_DEFAULT_RECEIVE_FILE, strlen(INVIZ_DEFAULT_RECEIVE_FILE));
                set_replace(TRUE); // by default replace existing file
            }
            pFile = fopen((char *)get_filename(), "r");
            if (pFile != NULL) {
                fclose(pFile); // file exists
                if (get_replace() && get_append()) {
                    sprintf(logmsg, "file (%s) already exists but cannot be both replaced and appended",
                            get_filename());
                    logit(_LOG_ERROR, logmsg, 0);
                    return ( EXIT_FAILURE );
                } else if (!get_replace() && !get_append()) {
                    sprintf(logmsg, "file (%s) already exists", get_filename());
                    logit(_LOG_ERROR, logmsg, 0);
                    sprintf(logmsg, "use %s parameter to replace existing file (%s) or %s parameter to append to it",
                        replace_parm,
                        get_filename(),
                        append_parm);
                    logit(_LOG_ERROR, logmsg, 0);
                    return ( EXIT_FAILURE );
                }
            }
            if (get_iconfirminterval() > 0) {
                // sending back confirmation packets to client ...
                if (get_useip4()) {
                    if (strlen(get_clientip4()) <= 0) {
                        sprintf(logmsg, "sending confirmation packets requires the client ip4 address must be specified with %s parameter",
                                clientip4_parm);
                        logit(_LOG_ERROR, logmsg, 0);
                        return ( EXIT_FAILURE );
                    } else {
                        sprintf(logmsg, "will send confirmation packets to client at %s",
                                get_clientip4());
                    }
                } else if (get_useip6()) {
                    if (strlen(get_clientip6()) <= 0) {
                        sprintf(logmsg, "sending confirmation packets requires the client ip6 address must be specified with %s parameter",
                                clientip6_parm);
                        logit(_LOG_ERROR, logmsg, 0);
                        return ( EXIT_FAILURE );
                    } else {
                        sprintf(logmsg, "will send confirmation packets to client at %s",
                                get_clientip6());
                    }
                }
                logit(_LOG_INFO, logmsg, 0);
            }
            sprintf(logmsg, "expecting to receive file (%s) from client with seed=%i, key=%s",
                get_filename(),
                get_iseed(),
                (char *)get_keyseed());
            logit(_LOG_INFO, logmsg, 0);
        }
    } else {
        sprintf(logmsg, "must be run as either client or server as specified by %s parameter",
                mode_parm);
        logit(_LOG_ERROR, logmsg, 0);
    }
    return ( EXIT_SUCCESS );
}
